/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2009 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#include <sys/types.h>
#include <string.h>
#include <stdio.h>
#include <unistd.h>

#include "basics.h"
#include "mdesc_impl.h"
#include "ldom.h"
#include "backend.h"

/*
 * This initialization list to eventually be auto-generated - FIXME
 * First list element is considered the default
 */

extern backend_t	hv_be;
#if ZDBG
extern backend_t	legion_be;
extern backend_t	testldom_be;
#endif

backend_t *be_list[] = {
	&hv_be,
#if ZDBG
	&legion_be,
	&testldom_be,
#endif
	NULL
};

/*
 * This initialization list to eventually be auto-generated - FIXME
 * First list element is considered the default
 */

extern target_t		niagara1_tgt;

target_t *tgt_list[] = {
	&niagara1_tgt,
	NULL
};


backend_t backend;


bool_t
backend_select(char *namep)
{
	int i;

	for (i = 0; be_list[i] != NULL; i++) {
		ASSERT(be_list[i]->magic == BACKEND_MAGIC);
		if (namep == NULL || strcmp(namep, be_list[i]->namep) == 0) {
			backend = *be_list[i];
			backend.init();
			return (true);
		}
	}

	return (false);
}

void
backend_list(void)
{
	int i;

	fprintf(stderr, "Available backends:\n");
	for (i = 0; be_list[i] != NULL; i++) {
		ASSERT(be_list[i]->magic == BACKEND_MAGIC);
		fprintf(stderr, "  %-10s  %s\n", be_list[i]->namep,
		    be_list[i]->descriptionp);
	}
	fprintf(stderr, "\nDefault selection: %s\n", be_list[0]->namep);
}


target_t target;

bool_t
target_select(char *namep)
{
	int i;

	for (i = 0; tgt_list[i] != NULL; i++) {
		ASSERT(tgt_list[i]->magic == TARGET_MAGIC);
		if (namep == NULL || strcmp(namep, tgt_list[i]->namep) == 0) {
			target = *tgt_list[i];
			target.init();
			return (true);
		}
	}

	return (false);
}

void
target_list(void)
{
	int i;

	fprintf(stderr, "Available targets:\n");
	for (i = 0; tgt_list[i] != NULL; i++) {
		ASSERT(tgt_list[i]->magic == TARGET_MAGIC);
		fprintf(stderr, "  %-10s  %s\n", tgt_list[i]->namep,
		    tgt_list[i]->descriptionp);
	}
	fprintf(stderr, "\nDefault selection: %s\n", tgt_list[0]->namep);
}
