/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2009 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#ifndef	_CONN_H
#define	_CONN_H

#pragma ident	"@(#)cli_conn.h	1.13	09/03/24 SMI"

#ifdef __cplusplus
extern "C" {
#endif

#include <sys/types.h>
#include <time.h>
#include <thread.h>
#include <synch.h>
#include <libxml/parser.h>
#include <openssl/ssl.h>

#include "audit_ldom.h"
#include "response.h"
#include "ldm_protocol.h"
#include "sunmc_client.h"
#include "waiter.h"

typedef enum {
	CONN_INIT		= 0x1,
	CONN_UP			= 0x2,
	CONN_ERR		= 0x3,
	SOCKET_WAITING		= 0x4,
	STREAM_TAG_ONE_SENT	= 0x5,
	STREAM_TLS_SENT		= 0x6,
	TLS_START		= 0x7,
	STREAM_TAG_TWO_SENT	= 0x8,
	STREAM_MECH_SENT	= 0x9,
	CHALLENGE_ONE_SENT	= 0xa,
	CHALLENGE_TWO_SENT	= 0xb,
	SUCCESS_SENT		= 0xc,
	STREAM_TAG_THREE_SENT	= 0xd,
	CONNECTION_COMPLETE	= 0xe,
	LDM_INTERFACE_STARTED	= 0xf,
	LDM_INTERFACE_FINISHED	= 0x10,		/* Obsolete */
	STREAM_DONE		= 0x11,
	XMPP_FAILURE		= 0x12,
	SASL_STARTED		= 0x13,
	TLS_STARTED		= 0x14,
	LDM_INTERFACE_RESET	= 0x15,
} conn_state_t;

/*
 * State of a connection between the LDom Manager and a CLI client.
 */
typedef struct cli_conn {
	/* callback to invoke when data is available to read or write */
	void		*(*callback)(struct cli_conn *);
	void		*sock_conn;	/* original socket connection struct */
	int		fd;		/* connect socket file descriptor */
	uid_t		euid;		/* EUID of socket peer, -1 if unknown */
	char		*uname;		/* User name */
	char		*password;
	audit_handle_t	audit_handle;	/* audit handle */
	ldm_hdr_t	hdr;		/* space to buffer message header */
	int		left;		/* # of bytes left to transfer */
	char		*parse_buf;	/* XML Parsing buffer */
	int		parse_len;	/* length of the parsing buffer */
	char		*buf;		/* input data buffer */
	char		*bufp;		/* current position in IO buffer */
	char		*event_buf;	/* for sending event XML */
	int		position;	/* position counter in buf */
	time_t		conntime;	/* time connection made */
	conn_state_t	state;		/* state of the connection */
	uint32_t	reqid;		/* request id */
	uint32_t	class;		/* client class for the connection */
	response_t 	*resp;		/* holds reponse to client */
	struct callout	*callout;	/* timer for event check */
	bool_t		tls_started;	/* Socket is secured? */
	xmlParserCtxtPtr	parser;
	int		buf_size;
	int		event_buf_size;
	mutex_t		mp;
	cond_t		cvp;
	bool_t		seq_finished;
	bool_t		xmpp_msg;
	bool_t		local_socket;
	bool_t		xmpp_server;
	bool_t		send_resp;	/* send a XMPP response or not? */
	xmlNodePtr	xml_command;
	SSL		*ssl;
	CHAIN(struct waiter, waiters);	/* ldom events being waited on */
	// mig fixme add link for list of all clients - might need lock
	char		*cookie;
} cli_conn_t;

typedef struct cli_reply_data {
	uint32_t	length;
	uint32_t	condition_code;
	uint32_t	text_length;
	char 		text[];
} cli_reply_data_t;


void client_write_response(cli_conn_t *conn);
void client_write_buffer(cli_conn_t *conn, void *bufp);
void handle_sunmc_client(cli_conn_t *conn);
void handle_mig_msg(cli_conn_t *conn);

void protocol_error(cli_conn_t *conn, uint32_t errcode);

#if ZDBG
void dump_protocol_versions(response_t *resp);
#endif

#ifdef __cplusplus
}
#endif

#endif	/* _CONN_H */
