/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2009 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#ifndef _CLP_H
#define	_CLP_H

#ifdef __cplusplus
extern "C" {
#endif

#include "basics.h"

/*
 * CLI shell status values (0 - 255)
 */
#define	LDOM_OK			0
#define	LDOM_GENERAL_FAILURE	1
#define	LDOM_SECURITY_FAILURE	9

#define	MAXTEXTBLOCKLEN 8192
#define	MAXARGC 128

#define	MAXLEN_OBJECT_NAME	256
#define	MAXLEN_VALID_NAME	MAXLEN_OBJECT_NAME
#define	MAXLEN_DEVICE_NAME	MAXLEN_OBJECT_NAME
#define	MAXLEN_VARIABLE_NAME	MAXLEN_OBJECT_NAME
#define	MAXLEN_VARIABLE_VALUE	1<<14	/* 16K for nvram scripts */

typedef enum {
	LDOM_CLI_SUCCESS = 0,
	LDOM_CLI_FAILURE,
	LDOM_CLI_INTERRUPTED,
	LDOM_CLI_CONFERR,
} zeus_resp_cc_t;

typedef enum {
	LDM_CREATE = 0,
	LDM_SET,
	LDM_DELETE,
	LDM_ADD_VCPU,
	LDM_MODIFY_VCPU,
	LDM_REMOVE_VCPU,
	LDM_ADD_CRYPTO,
	LDM_MODIFY_CRYPTO,
	LDM_REMOVE_CRYPTO,
	LDM_ADD_MEM,
	LDM_MODIFY_MEM,
	LDM_REMOVE_MEM,
	LDM_ADD_IO,
	LDM_REMOVE_IO,
	LDM_ADD_VSW,
	LDM_MODIFY_VSW,
	LDM_REMOVE_VSW,
	LDM_ADD_VNET,
	LDM_MODIFY_VNET,
	LDM_REMOVE_VNET,
	LDM_ADD_VDS,
	LDM_REMOVE_VDS,
	LDM_ADD_VDSDEV,
	LDM_MODIFY_VDSDEV,
	LDM_REMOVE_VDSDEV,
	LDM_ADD_VDISK,
	LDM_MODIFY_VDISK,
	LDM_REMOVE_VDISK,
	LDM_ADD_VCC,
	LDM_MODIFY_VCC,
	LDM_REMOVE_VCC,
	LDM_MODIFY_VCONS,
	LDM_SET_VAR,
	LDM_UNSET_VAR,
	LDM_GET,
	LDM_START,
	LDM_STOP,
	LDM_BIND,
	LDM_UNBIND,
	LDM_ADD_SET,
	LDM_REMOVE_SET,
	LDM_USE_SET,
	LDM_LIST,
	LDM_LIST_BINDINGS,
	LDM_LIST_SERVICES,
	LDM_LIST_CONSTRAINTS,
	LDM_LIST_DEVICES,
	LDM_LIST_SETS,
	LDM_LIST_DEL_RECONF,
	LDM_CANCEL_DEL_RECONF,
	LDM_ADD_VDPCS,
	LDM_REMOVE_VDPCS,
	LDM_ADD_VDPCC,
	LDM_REMOVE_VDPCC,
	LDM_PANIC,
	LDM_MIGRATE_DOMAIN,
	LDM_CANCEL_OPERATION,
	LDM_REBOOT_PRIMARY,
	LDM_REG_DOM_EVENT,
	LDM_REG_RES_EVENT,
	LDM_REG_HW_EVENT,
	LDM_REG_PROGRESS_EVENT,
	LDM_UNREG_DOM_EVENT,
	LDM_UNREG_RES_EVENT,
	LDM_UNREG_HW_EVENT,
	LDM_UNREG_PROGRESS_EVENT,
	LDM_ADD_POLICY,
	LDM_MODIFY_POLICY,
	LDM_REMOVE_POLICY,
} ldm_cmd_num_t;

typedef enum {
	CLI_VERB_ADD,
	CLI_VERB_SET,
	CLI_VERB_REMOVE,
	CLI_VERB_LIST,
	CLI_VERB_START,
	CLI_VERB_STOP,
	CLI_VERB_BIND,
	CLI_VERB_UNBIND,
	CLI_VERB_PANIC,
	CLI_VERB_MIGRATE,
	CLI_VERB_CANCEL,
} action_verb_t;

struct cli_conn;

typedef struct {
	ldm_cmd_num_t	cmd_num;	/* command number */
	char		*cmd_name;	/* command name */
	int		(*funcp)(bool_t argc, char *argv[],
			    struct cli_conn *client, ldm_cmd_num_t cmd_num);
					/* function to handle this cmd */
	char		*help;		/* command help string */
} ldm_cmd_t;

typedef enum {
	INTEGER,
	TEXTBLOCK,
	OPTION,
	NONE
} zeus_arg_type_t;

typedef struct {
	zeus_arg_type_t	arg_type;
	int		arg_len;
	void		*arg;
} zeus_cmd_arg_t;

/* Command message on the wire */
typedef struct {
	int		len;
	int		argc;
	char		data[];
} zeus_cmd_msg_t;

int ldm_cli_command(struct cli_conn *client);
void free_cmd_arg(zeus_cmd_arg_t *cmd_arg, int argc);

#ifdef __cplusplus
}
#endif

#endif /* _CLP_H */
