/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2010 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#ifndef	_CONSOLE_H
#define	_CONSOLE_H

#ifdef __cplusplus
extern "C" {
#endif

#include "ldmd_types.h"
#include "ldcs.h"
#include "ldom.h"
#include "mdesc_impl.h"
#include "response.h"

/* LDC console ino */
#define	LDC_CONSOLE_INO	0x11

typedef enum {
	CONSOLE_SERVICE = 0x1,
	CONSOLE_GROUP = 0x2,
	CONSOLE_PORT = 0x4
} vcons_args_t;

/*
 * Guest console.
 */
struct console {
	ldom_t		*ldp;		/* owner ldom */
	char		*service_name;
	char		*group_name;
	int		tcp_port;
	console_binding_t *binding;
};

struct console_binding {
	ldcs_t		ldcs;		/* LDC channel assigned to console */
	resid_t		*id;		/* HV console ID */
	vcc_console_t	*vcc_cons;	/* server side binding */
	uint64_t	uartbase;	/* If uart based console */
	bool_t		sp_cons;	/* is this a SP console */
};

bool_t is_console_set(ldom_t *ldp);
console_t *console_new(ldom_t *ldp, char *service, char *group, int tcp_port);
void console_add_from_md(ldom_t *ldp, md_t *hv_mdp, md_node_t *hv_guestp);
int console_modify(ldom_t *ldp, char *service, char *group, int port,
    response_t *resp);
char *console_print(ldom_t *ldp);
int link_console(ldom_t *ldp);
void console_list(ldom_t *ldp, response_t *resp, dispctl_t *dctl);
void console_gen_ldc_endpoints(ldom_t *ldp, md_t *hv_md, md_node_t *hv_guest,
    md_node_t *ldc_endpoints);
int console_bind(ldom_t *ldp, response_t *resp);
void console_unbind(ldom_t *ldp);
void console_delete(ldom_t *ldp);
int console_bind_check(ldom_t *ldp, response_t *resp);
int console_mig_add_bindings(ldom_t *ldp, md_t *mdp);
int console_mig_bind(ldom_t *ldp, md_t *mdp, md_t *hvmdp, response_t *resp);
void console_print_bindings(ldom_t *ldp, response_t *resp);
bool_t is_console_constraints_set(ldom_t *ldp);

#ifdef __cplusplus
}
#endif

#endif	/* _CONSOLE_H */
