/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2007 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#pragma ident	"@(#)control_class.c	1.4	07/04/27 SMI"

#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include "ldom.h"
#include "zpri.h"
#include "allocate.h"
#include "cli_conn.h"
#include "control_class.h"

void
cntrl_pri_req(cli_conn_t *conn)
{
	cntrl_pri_req_t		*pri_req;
	ldm_hdr_t		*hdr;
	cntrl_pri_reply_t	*pri_reply;
	char			*bufp;
	size_t			buf_size;
	int			md_size;
	uint8_t			*mdbuf;

	md_size = md_gen_bin(pri.core_mdp, &mdbuf);

	pri_req = (cntrl_pri_req_t *)conn->bufp;

	buf_size = sizeof (ldm_hdr_t) + sizeof (cntrl_pri_reply_t) + md_size;
	bufp = Xmalloc(buf_size);

	/* set LDM protocol headers */
	hdr = (ldm_hdr_t *)bufp;
	hdr->msg_type = hton32(LDM_CNTRL_PRI_REPLY);
	hdr->payload_len = hton32(sizeof (cntrl_pri_reply_t) + md_size);

	pri_reply = (cntrl_pri_reply_t *)(bufp + sizeof (ldm_hdr_t));
	pri_reply->rp_reqid = hton32(pri_req->rq_id);
	memcpy(pri_reply->rp_data, mdbuf, md_size);

	free(conn->buf);
	conn->buf = 0;

	client_write_buffer(conn, bufp);

	Xfree(mdbuf);
}
