/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2010 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#ifndef _DATABASE_H
#define	_DATABASE_H

#ifdef __cplusplus
extern "C" {
#endif

#include <inttypes.h>

#include <libxml/tree.h>
#include <libxml/parser.h>
#include <libxml/xpath.h>
#include <libxml/xpathInternals.h>

#include "ldom.h"
#include "mac_addrs.h"

int database_init(void);
int database_add_unbound_ldoms(void);
int database_add_masters(void);
int database_save(void);
void database_ldom_create(ldom_t *ldp);
void database_ldom_destroy(ldom_t *ldp);
void database_ldom_destroy_policy_inst(ldom_t *ldp, char *policy_name);
int database_ldom_set_cpus(ldom_t *ldp, int ncpus);
int database_ldom_set_maus(ldom_t *ldp, int nmaus);
int database_ldom_set_memory(ldom_t *ldp, uint64_t memsize);
int database_ldom_set_mac_addr(ldom_t *ldp, uint64_t mac_addr);
int database_ldom_set_hostid(ldom_t *ldp, uint64_t hostid);
int database_ldom_set_failure_policy(ldom_t *ldp, uint64_t failure_policy);
int database_ldom_set_master(ldom_t *ldp);
int database_ldom_set_policy(ldom_t *ldp, policy_t *polp);
int database_ldom_set_uuid(ldom_t *ldp);
int database_ldom_get_uuid(ldom_t *ldp);
int database_ldom_add_console_service(ldom_t *ldp, char *service_name,
    int min_port, int max_port);
int database_ldom_modify_console_service(ldom_t *ldp, char *service_name,
    int min_port, int max_port);
int database_ldom_add_console_client(ldom_t *ldp,
    char *service_name, char *group, int port);
int database_ldom_modify_console_client(ldom_t *ldp, vcons_args_t mod,
    char *new_service_name, char *new_group_name, int port);
int database_ldom_add_disk_service(ldom_t *ldp, char *service_name);
int database_ldom_add_disk_service_vol(ldom_t *ldp, char *service,
    char *vol_name, char *block_dev, char *options, char *mpgname);
int database_ldom_modify_vdsdev_props(ldom_t *ldp, char *service_name,
    char *vdsdev_name, char *options, char *mpgname);
int database_ldom_add_disk_client(ldom_t *ldp, char *vdisk_name,
    char *service_name, char *vol_name, int timeout, int id);
int database_ldom_modify_disk_client(ldom_t *ldp, char *vdisk_name,
    char *service_name, char *volume_name, int timeout);
int database_ldom_add_net_service(ldom_t *ldp, char *service_name,
    mac_addr_t mac_address, char *dev_path, uint64_t phys_state, char *policies,
    int default_vid, int pvid, char *vlans, int mtu, int id);
int database_ldom_add_iodevice(ldom_t *ldp, char *name, bool_t bypass);
int database_ldom_add_pcie_device(ldom_t *ldp, char *name);
int database_ldom_add_vdpc_client(ldom_t *ldp, char *vdpc_name,
    char *service_name);
bool_t database_ldom_del_vdpc_client(ldom_t *ldp, char *client_name);
int database_ldom_add_vdpc_service(ldom_t *ldp, char *service_name);
bool_t database_ldom_del_vdpc_service(ldom_t *ldp, char *service_name);
int database_ldom_modify_net_service(ldom_t *ldp, int mod, char *service_name,
    mac_addr_t mac_address, char *dev_path, uint64_t phys_state, char *policies,
    int pvid, int num_vlans, char *vlans, int mtu);
int database_ldom_add_net_client(ldom_t *ldp, char *vnet_name,
    char *service_name, mac_addr_t mac_address, int mtu, uint64_t phys_state,
    char *mode, int pvid, char *vlans, int id);
int database_ldom_modify_net_client(ldom_t *ldp, int mod, char *client_name,
    char *service_name, mac_addr_t mac_address, int mtu, uint64_t phys_state,
    char *vnet_mode, int pvid, int num_vlans, char *vlans);
bool_t database_ldom_del_console_client(ldom_t *ldp);
bool_t database_ldom_del_console_service(ldom_t *ldp, char *service_name);
bool_t database_ldom_del_disk_client(ldom_t *ldp, char *client_name);
bool_t database_ldom_del_disk_service(ldom_t *ldp, char *service_name);
bool_t database_ldom_del_disk_service_block(ldom_t *ldp, char *service_name,
    char *block_dev);
bool_t database_ldom_del_net_client(ldom_t *ldp, char *client_name);
bool_t database_ldom_del_net_service(ldom_t *ldp, char *service_name);
bool_t database_ldom_add_variable(ldom_t *ldp, char *name, char *value);
bool_t database_ldom_del_variable(ldom_t *ldp, char *name);
bool_t database_ldom_del_iodevice(ldom_t *ldp, char *iodevice);
bool_t database_ldom_del_pcie_device(ldom_t *ldp, char *name);
bool_t database_get_vcc_properties(ldom_t *ldp, char *service_name,
    uint64_t *min_port, uint64_t *max_port);
char *get_service_name_from_db(ldom_t *ldp, char *type, int instance);
char *get_vnet_name_from_db(ldom_t *ldp, int instance);
char *get_vdisk_name_from_db(ldom_t *ldp, int instance);
bool_t get_vdisk_props_from_db(ldom_t *ldp, char *vdisk_name,
    char **vds_name, char **vdsdev_name, int *id);
bool_t get_vsw_props_from_db(ldom_t *ldp, char *vsw_name, int *mtu);
bool_t get_vsw_linkprop_from_db(ldom_t *ldp, char *vsw_name,
    uint64_t *phys_state);
bool_t get_vsw_id_from_db(ldom_t *ldp, char *vsw_name, int *id);
bool_t get_vnet_props_from_db(ldom_t *ldp, char *vnet_name, int *mtu);
bool_t get_vnet_linkprop_from_db(ldom_t *ldp, char *vnet_name,
    uint64_t *phys_state);
bool_t get_vnet_id_from_db(ldom_t *ldp, char *vnet_name, int *id);
xmlDocPtr get_dbase(void);
xmlNodePtr find_ldom(xmlDocPtr db, const xmlChar *name);
xmlNodePtr find_ldom_node_instance(xmlNodePtr ldom, const xmlChar *name,
    int instance);
int database_ldom_get_lowest_id(ldom_t *ldp, xmlChar *node_name);
bool_t database_ldom_check_dup_id(ldom_t *ldp, xmlChar *node_name,
    int id);

/*
 * From the old db set the reqs amt from cpu or mau
 */
int database_ldom_get_cpu(ldom_t *ldp);
bool_t database_get_console(ldom_t *ldp, int *port, char **group,
	char **service, bool_t *flag);
int database_ldom_get_mau(ldom_t *ldp);

#ifdef __cplusplus
}
#endif

#endif	/* _DATABASE_H */
