/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2010 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#ifndef _DATABASE_INTERNAL_H
#define	_DATABASE_INTERNAL_H

#ifdef __cplusplus
extern "C" {
#endif

#include <libxml/tree.h>
#include <libxml/xmlstring.h>

#include "basics.h"
#include "database.h"

#define	DATABASE_SCHEMAFILE	"ldom_database.xsd"

#define	DB_SUPPORTS_UUID()      \
		(db_major == 3 && db_minor >= 8) || (db_major > 3)

#define	DB_SUPPORTS_SDIO()      \
		(db_major == 3 && db_minor >= 9) || (db_major > 3)

struct ldom_database {
	xmlNodePtr ldom_node;
};

#define	CONSOLE_INST	0

/*
 * XML node and attribute names
 */
#define	CONSOLE_INSTANCE_NODE	((xmlChar *)"console_instance")
#define	CONSOLE_NODE		((xmlChar *)"console")
#define	CPU_NODE		((xmlChar *)"cpu")
#define	MAU_NODE		((xmlChar *)"mau")
#define	DEV_PATH_NODE		((xmlChar *)"dev_path")
#define	LINKPROP_NODE		((xmlChar *)"linkprop")
#define	GROUP_NODE		((xmlChar *)"group")
#define	VCONS_PORT_NODE		((xmlChar *)"port")
#define	INSTANCE_NODE		((xmlChar *)"instance")
#define	LDOM_DATABASE_NODE	((xmlChar *)"ldom_database")
#define	LDOM_NAME_NODE		((xmlChar *)"ldom_name")
#define	LDOM_NODE		((xmlChar *)"ldom")
#define	MAC_ADDRESS_NODE	((xmlChar *)"mac_address")
#define	MAX_PORT_NODE		((xmlChar *)"max_port")
#define	MEMORY_NODE		((xmlChar *)"memory")
#define	MIN_PORT_NODE		((xmlChar *)"min_port")
#define	MTU_NODE		((xmlChar *)"mtu")
#define	NAME_NODE		((xmlChar *)"name")
#define	NUMBER_NODE		((xmlChar *)"number")
#define	PHYSIO_MINOR_NODE	((xmlChar *)"minor")
#define	VOLUME_NAME_NODE	((xmlChar *)"vol_name")
#define	VOLUME_OPTS_NODE	((xmlChar *)"vol_opts")
#define	VOLUME_MPGROUP_NODE	((xmlChar *)"mpgroup")
#define	BLOCKDEV_NODE		((xmlChar *)"block_dev")
#define	PHYSIO_NODE		((xmlChar *)"physio")
#define	SERVICE_NAME_NODE	((xmlChar *)"service_name")
#define	SERVER_INSTANCE_NODE	((xmlChar *)"server_instance")
#define	SERVER_NODE		((xmlChar *)"server")
#define	SHARING_NODE		((xmlChar *)"sharing")
#define	SIZE_NODE		((xmlChar *)"size")
#define	STATE_NODE		((xmlChar *)"state")
#define	VALUE_NODE		((xmlChar *)"value")
#define	VARIABLES_NODE		((xmlChar *)"variables")
#define	VAR_NODE		((xmlChar *)"var")
#define	VCC_INSTANCE_NODE	((xmlChar *)"vcc_instance")
#define	VCC_NODE		((xmlChar *)"vcc")
#define	DISK_INSTANCE_NODE	((xmlChar *)"disk_instance")
#define	DISK_NODE		((xmlChar *)"disk")
#define	VDISK_NAME_NODE		((xmlChar *)"vdisk_name")
#define	VDS_VOLUME_NODE		((xmlChar *)"vds_volume")
#define	VDS_VOLUMES_NODE	((xmlChar *)"vds_volumes")
#define	VDS_INSTANCE_NODE	((xmlChar *)"vds_instance")
#define	VDS_NODE		((xmlChar *)"vds")
#define	VNET_INSTANCE_NODE	((xmlChar *)"network_instance")
#define	VNET_NODE		((xmlChar *)"network")
#define	VNET_NAME_NODE		((xmlChar *)"vnet_name")
#define	VNET_MODE_NODE		((xmlChar *)"mode")
#define	VNET_PORT_VLAN_NODE	((xmlChar *)"pvid")
#define	VNET_VLANS_NODE		((xmlChar *)"vid")
#define	VSW_INSTANCE_NODE	((xmlChar *)"vsw_instance")
#define	VSW_POLICIES_NODE	((xmlChar *)"policies")
#define	VSW_MODE_NODE		((xmlChar *)"mode")
#define	VSW_DEF_VLAN_NODE	((xmlChar *)"default-vlan-id")
#define	VSW_PORT_VLAN_NODE	((xmlChar *)"pvid")
#define	VSW_VLANS_NODE		((xmlChar *)"vid")
#define	VSW_MTU_NODE		((xmlChar *)"mtu")
#define	VSW_NODE		((xmlChar *)"vsw")
#define	IO_INSTANCE_NODE	((xmlChar *)"io_instance")
#define	IO_NODE			((xmlChar *)"io")
#define	IO_NAME_NODE		((xmlChar *)"iodevice")
#define	IO_BYPASS_NODE		((xmlChar *)"bypass_mode")
#define	TIMEOUT_NODE		((xmlChar *)"timeout")
#define	PCIEDEV_NODE		((xmlChar *)"pciedev")
#define	PCIEDEV_INSTANCE_NODE	((xmlChar *)"pciedev_instance")
#define	PCIEDEV_NAME_NODE	((xmlChar *)"pciedev_name")
#define	VDPCC_NODE		((xmlChar *)"vdpcc")
#define	VDPCC_INSTANCE_NODE	((xmlChar *)"vdpcc_instance")
#define	VDPCC_NAME_NODE		((xmlChar *)"vdpcc_name")
#define	VDPCS_NODE		((xmlChar *)"vdpcs")
#define	VDPCS_INSTANCE_NODE	((xmlChar *)"vdpcs_instance")
#define	HOSTID_NODE		((xmlChar *)"hostid")
#define	FAILURE_POLICY_NODE	((xmlChar *)"failure-policy")
#define	MASTER_NODE		((xmlChar *)"master")
#define	UUID_NODE		((xmlChar *)"uuid")
#define	ID_NODE			((xmlChar *)"id")
#define	POLICY_NODE		((xmlChar *)"policy")
#define	POLICY_INSTANCE_NODE	((xmlChar *)"policy_instance")
#define	POLICY_NAME_NODE	((xmlChar *)"policy_name")
#define	POLICY_ENABLE_NODE	((xmlChar *)"policy_enable")
#define	POLICY_PRIORITY_NODE	((xmlChar *)"policy_priority")
#define	POLICY_VCPU_MIN_NODE	((xmlChar *)"policy_vcpu_min")
#define	POLICY_VCPU_MAX_NODE	((xmlChar *)"policy_vcpu_max")
#define	POLICY_UTIL_LOWER_NODE	((xmlChar *)"policy_util_lower")
#define	POLICY_UTIL_UPPER_NODE	((xmlChar *)"policy_util_upper")
#define	POLICY_TOD_BEGIN_NODE	((xmlChar *)"policy_tod_begin")
#define	POLICY_TOD_END_NODE	((xmlChar *)"policy_tod_end")
#define	POLICY_SAMPLE_RATE_NODE	((xmlChar *)"policy_sample_rate")
#define	POLICY_ELASTIC_MARGIN_NODE	((xmlChar *)"policy_elastic_margin")
#define	POLICY_ATTACK_NODE	((xmlChar *)"policy_attack")
#define	POLICY_DECAY_NODE	((xmlChar *)"policy_decay")

#define	VERSION_ATTR		((xmlChar *)"version")

/*
 * OPTIONS_NODE is no longer used.  However, database code for
 * releases 1.1 and 1.2 requires the node to be present in the
 * database even if it will be empty.  We don't want to create
 * a major version change to solve this problem, so instead
 * we do two things:
 *   + New code that uses the version 3.x database format but
 *     doesn't need the OPTIONS_NODE specifically ignores it.
 *   + New code always creates an OPTIONS_NODE for any version 3.x
 *     format database, and then leaves it empty.
 */
#define	OPTIONS_NODE		((xmlChar *)"options")

bool_t validate_database(xmlDocPtr db);

#ifdef __cplusplus
}
#endif

#endif /* _DATABASE_INTERNAL_H */
