/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2009 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#include <assert.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <assert.h>

#include <libxml/tree.h>
#include <libxml/parser.h>
#include <libxml/xmlschemas.h>

#include "allocate.h"
#include "basics.h"
#include "database_internal.h"
#include "fatal.h"
#include "zprintf.h"

static void
schema_validity_error(void *ignored, const char *msg, ...)
{
	va_list vargs;

	va_start(vargs, msg);
	vfatal(true, true, (char *)msg, vargs);
	va_end(vargs);
}

/*
 * Validate XML database document 'db' against the database schema.  Returns
 * true if the database is valid, otherwise false.
 */
bool_t
validate_database(xmlDocPtr db)
{
	xmlSchemaValidCtxtPtr valctxt;
	xmlSchemaParserCtxtPtr ctxt;
	xmlSchemaPtr wxschema;
	char *schemafile;
	int r;

	schemafile = zsprintf("%s/%s", zeus.pkgdir, DATABASE_SCHEMAFILE);
	ctxt = xmlSchemaNewParserCtxt(schemafile);
	Xfree(schemafile);

	/* set up to exit if the schema itself is invalid */
	xmlSchemaSetParserErrors(ctxt,
	    schema_validity_error, schema_validity_error, NULL);

	wxschema = xmlSchemaParse(ctxt);
	xmlSchemaFreeParserCtxt(ctxt);

	assert(wxschema != NULL);

	valctxt = xmlSchemaNewValidCtxt(wxschema);
	/* print database validity errors to stderr */
	xmlSchemaSetValidErrors(valctxt, (xmlSchemaValidityErrorFunc) fprintf,
	    (xmlSchemaValidityWarningFunc) fprintf, stderr);

	r = xmlSchemaValidateDoc(valctxt, db);

	xmlSchemaFreeValidCtxt(valctxt);
	xmlSchemaFree(wxschema);

	if (r == 0) {
		DBG(zprintf("LDom database is valid\n"));
	} else if (r > 0) {
		warning("LDom database is invalid");
	} else {
		warning("internal error during LDom database verification");
	}

	return (r == 0 ? true : false);
}
