/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2009 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#ifndef	_DISPCTL_H_
#define	_DISPCTL_H_

#ifdef __cplusplus
extern "C" {
#endif

/*
 * Parseable output version history.
 *
 * 1.0	Initial version.
 * 1.1	Added "id" field for crypto units.
 * 1.2	Rearranged vdisk output, mpgroup support
 * 1.3  Added "pm" field to list-devices -a output
 * 1.4  Added "status" section to list -l output
 */

#define	PARSEABLE_OUTPUT_VERSION	"1.4"

#define	FNAME	"%-16s"		/* name formatter */
#define	FBNDNAME "%-27s"	/* bound name formatter */
#define	FDESC	"%-17s"		/* description formatter */
#define	FMAC	"%-17s"		/* MAC formatter */
#define	FNETDEV	"%-9s"		/* net device formatter */
#define	FDEVICE	"%-10s"		/* device formatter */
#define	FDDEVICE "%-7s"		/* disk device formatter */
#define	FIO	"%-16s"		/* IO device formatter */
#define	FPORT	"%-6s"		/* port formatter */
#define	FPORTd	"%-6"PRIu64""	/* port formatter (integer) */
#define	FVOL	"%-14s"		/* disk volume name formatter */
#define	FVOLOPTS "%-16s"	/* disk volume options formatter */
#define	FVOLMGP	"%-14s"		/* disk volume mpgroup formatter */
#define	FTOUT	"%-4s"		/* disk timeout option formatter */
#define	FID	"%-4s"		/* id option formatter */
#define	FSERVER	"%-14s"		/* disk server formatter */
#define	FMODE	"%-6s"		/* vnet mode */
#define	FDEFVID	"%-15s"		/* default Vlan ID formatter */
#define	FDEFVIDd	"%-15d"	/* default Vlan ID formatter */
#define	FPVID	"%-4s"		/* Port Vlan ID formatter */
#define	FPVIDd	"%-4d"		/* Port Vlan ID formatter(int) */
#define	FVID	"%-20s"		/* Vlan ID formatter */
#define	FMTU	"%-5s"		/* MTU formatter */
#define	FMTUd	"%-5d"		/* MTU formatter(int) */
#define	FLNKP	"%-10s"		/* Linkprop formatter */

typedef struct dispctl {
	int list:1;		/* true if doing ldm ls */
	int bindings:1;		/* true if doing ldm ls-bindings */
	int constraints:1;	/* true if doing ldm ls-constraints */
	int services:1;		/* true if doing ldm ls-services */
	int all:1;		/* true if listing all devices */
	int parseable:1;	/* true if parseable (-p) output */
	int longf:1;		/* true if long format (-l) output */
	int extended:1;		/* true if extended (-e) output */
	int oformat:1;		/* true if oformat (-o) output . . . */
	int oconsole:1;		/*   vcons,vcc output */
	int ocpu:1;		/*   vcpu,pcpu output */
	int ocrypto:1;		/*   mau output */
	int odisk:1;		/*   vdisk,vds output */
	int odomain:1;		/*   var,hostid,state,flags,softstate output */
	int omemory:1;		/*   memory output */
	int onetwork:1;		/*   mac,vsw,vnet output */
	int ophysio:1;		/*   pci,niu output */
	int oresmgmt:1;		/*   dynamic resource mgmt output */
	int oserial:1;		/*   vldc,vldcc,vdpcc,vdpcs output */
	int ostatus:1;		/*   migration output */
	int first:1;		/* true if first ldom not yet output */
	int server:1;		/* true if displaying server info */
	int clients:1;		/* true if displaying server client info */
	int header:1;		/* true if header needs to be output */
	int physfirst:1;	/* true if first phys binding not yet output */
} dispctl_t;

void dctl_init(dispctl_t *dctl);
char *dctl_label(char *label, dispctl_t *dctl);

#ifdef __cplusplus
}
#endif

#endif /* _DISPCTL_H_ */
