/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2009 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#ifndef _DR_H
#define	_DR_H

/*
 * Generic DR Definitions
 */

#ifdef __cplusplus
extern "C" {
#endif

/*
 * Return codes for DR operations (config and unconfig).
 *
 * These values are used by ldmd to keep track of the DR request
 * and are not part of any DS protocol, etc.
 */
#define	DR_OP_SUCC			0
#define	DR_OP_FAILED			(-1)
#define	DR_OP_PARTIAL			(-2)
#define	DR_OP_NO_RESPONSE		(-3)
#define	DR_OP_FAILED_ROLLBACK		(-4)

/*
 * This structure enumerates the different operations that are possible
 * on resources, some of which trigger DR operations (and modify MDs).
 *
 * These definitions are used to select the correct function to call if
 * a DR operation is possible. In some cases, this is a no-op as DR is not
 * applicable (e.g. changing some VIO services)
 */
typedef enum {
	DR_OP_TYPE_CPU,		/* CPU */
	DR_OP_TYPE_CRYPTO,	/* crypto (MAU/CWQ/etc.) */
	DR_OP_TYPE_DOMAIN,	/* domain state */
	DR_OP_TYPE_MEM,		/* memory */
	DR_OP_TYPE_NDPS,	/* Netra Data Plane devices */
	DR_OP_TYPE_PIO,		/* physical I/O */
	DR_OP_TYPE_VAR,		/* variable */
	DR_OP_TYPE_VIO,		/* virtual I/O - DR operations */
	DR_OP_TYPE_VIO_MD_NOP,	/* virtual I/O - MD unmodified (DR unneeded) */
	DR_OP_TYPE_VIO_SET	/* virtual I/O - set-xxx cmd (DR not impl) */
} dr_op_type_t;

#ifdef __cplusplus
}
#endif

#endif /* _DR_H */
