/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2008 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#ifndef _DR_CPU_H
#define	_DR_CPU_H

#pragma ident	"@(#)dr_cpu.h	1.11	08/10/17 SMI"

/*
 * CPU DR Control Protocol
 */

#ifdef __cplusplus
extern "C" {
#endif

#include <sys/types.h>
#include "ldom.h"

/*
 * Common message header
 */
typedef struct {
	uint64_t	req_num;
	uint32_t	msg_type;	/* CONFIG/UNCONFIG etc. */
	uint32_t	num_records;
} dr_cpu_hdr_t;

/*
 * A cpu-dr message is followed by a list of one or
 * more virtual cpu numbers, each of which is
 * represented by an unsigned int
 */
typedef uint32_t	ds_vcpu_id_t;

/*
 * dr-cpu status
 */
typedef struct {
	ds_vcpu_id_t	cpuid;
	uint32_t	result;
	uint32_t	status;
	uint32_t	string_off;
} dr_cpu_stat_t;

/*
 * cpu-dr message type
 */
typedef struct {
	dr_cpu_hdr_t		hdr;
	union {
		ds_vcpu_id_t	req_cpus[1];
		dr_cpu_stat_t	resp_cpu_stats[1];
		char		err_msg[1];
	} dr_cpu_payload;
} dr_cpu_msg_t;

#define	DR_CPU_MAX_ERR_SIZE	256

/*
 * CPU DR request mesage types
 */

#define	DR_CPU_CONFIGURE	('C')
#define	DR_CPU_UNCONFIGURE	('U')
#define	DR_CPU_FORCE_UNCONFIG	('F')
#define	DR_CPU_STATUS		('S')

/*
 * Response message types
 */
#define	DR_CPU_OK		('o')
#define	DR_CPU_ERROR		('e')

/*
 * Result codes for DR_CPU_OK
 */
#define	DR_CPU_RES_OK			0x0
#define	DR_CPU_RES_FAILURE		0x1
#define	DR_CPU_RES_BLOCKED		0x2
#define	DR_CPU_RES_CPU_NOT_RESPONDING	0x3
#define	DR_CPU_RES_NOT_IN_MD		0x4

/*
 * Status codes for DR_CPU_OK
 */
#define	DR_CPU_STAT_NOT_PRESENT		0x0
#define	DR_CPU_STAT_UNCONFIGURED	0x1
#define	DR_CPU_STAT_CONFIGURED		0x2

/*
 * Return codes for cpu config and unconfig operations
 */
#define	CPU_DR_OP_SUCC			0
#define	CPU_DR_OP_FAILED		(-1)
#define	CPU_DR_OP_PARTIAL		(-2)
#define	CPU_DR_OP_NO_RESPONSE		(-3)

/*
 * No additional payload is returned when msg_type
 * is DR_CPU_ERROR. This implies a malformed request
 * message and no cpu dr related operation was
 * attempted in the guest end
 */

extern int dr_cpu_configure(ldom_t *ldp, uint_t ncpuid, uint32_t *cpuidp,
    bool_t force, uint32_t *op_stats, response_t *resp);
extern int dr_cpu_unconfigure(ldom_t *ldp, uint_t ncpuid, uint32_t *cpuidp,
    bool_t force, uint32_t *op_stats, response_t *resp);

typedef int (*dr_cpu_cfg_f)(ldom_t *, uint_t, uint32_t *, bool_t,
		uint32_t *, response_t *resp);

#ifdef __cplusplus
}
#endif

#endif /* _DR_CPU_H */
