/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2008 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#ifndef _DR_CRYPTO_H
#define	_DR_CRYPTO_H

#pragma ident	"@(#)dr_crypto.h	1.1	08/06/10 SMI"

/*
 * Crypto DR Domain Service Protocol. The same protocol is used to
 * dynamically add or remove both Modular Arithmetic Unit (MAU) and
 * Control Word Queue (CWQ) devices.
 */

#ifdef __cplusplus
extern "C" {
#endif

#include <sys/types.h>
#include "ldom.h"

/*
 * Common message header used for both requests and responses.
 */
typedef struct {
	uint64_t	req_num;
	uint32_t	msg_type;
	uint32_t	num_records;
} dr_crypto_hdr_t;

/*
 * A dr-crypto message is followed by a list of one or more virtual
 * cpu identifiers. Each id represents a vcpu which is associated
 * with the target crypto device.
 */
typedef uint32_t	ds_crypto_id_t;

/*
 * The dr-crypto status structure. Provides information about the
 * result of the operation. One status structure is returned for
 * each of the vcpus in the request.
 */
typedef struct {
	ds_crypto_id_t	cpuid;
	uint32_t	result;
	uint32_t	status;
} dr_crypto_stat_t;

typedef struct {
	dr_crypto_hdr_t		hdr;
	union {
		ds_crypto_id_t		req_cpus[1];
		dr_crypto_stat_t	resp_cpu_stats[1];
	} dr_crypto_payload;
} dr_crypto_msg_t;

/*
 * Crypto DR request message types
 */
#define	DR_CRYPTO_CONFIGURE		('C')
#define	DR_CRYPTO_UNCONFIGURE		('U')
#define	DR_CRYPTO_FORCE_UNCONFIG	('F')
#define	DR_CRYPTO_STATUS		('S')

/*
 * Crypto DR response message types
 */
#define	DR_CRYPTO_OK			('o')
#define	DR_CRYPTO_ERROR			('e')

/*
 * Result codes for DR_CRYPTO_OK
 */
#define	DR_CRYPTO_RES_OK		0x0
#define	DR_CRYPTO_RES_FAILURE		0x1
#define	DR_CRYPTO_RES_BAD_CPU		0x2
#define	DR_CRYPTO_RES_BAD_CRYPTO	0x3

/*
 * Status codes for DR_CRYPTO_OK
 */
#define	DR_CRYPTO_STAT_NOT_PRESENT	0x0
#define	DR_CRYPTO_STAT_UNCONFIGURED	0x1
#define	DR_CRYPTO_STAT_CONFIGURED	0x2

/*
 * Internal return codes for crypto DR operations.
 */
#define	CRYPTO_DR_OP_SUCC		0
#define	CRYPTO_DR_OP_FAILED		(-1)
#define	CRYPTO_DR_OP_PARTIAL		(-2)
#define	CRYPTO_DR_OP_NO_RESPONSE	(-3)
#define	CRYPTO_DR_OP_NO_SVC		(-4)

/*
 * Generic function to perform crypto DR operations.
 */
typedef int (*dr_crypto_cfg_f)(ldom_t *ldp, uint_t ncpuid, uint32_t *cpuidp,
    bool_t force, uint32_t *op_stats, response_t *resp);

#ifdef __cplusplus
}
#endif

#endif /* _DR_CRYPTO_H */
