/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2008 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#ifndef _DR_VIO_H
#define	_DR_VIO_H

#pragma ident	"@(#)dr_vio.h	1.2	08/10/17 SMI"

/*
 * Virtual I/O Dynamic Reconfiguration (VIO DR) Protocol definitions
 */

#ifdef __cplusplus
extern "C" {
#endif

#include <sys/types.h>
#include "ldom.h"

#define	DR_DEV_TYPE_MAX_LEN		64

/*
 * VIO DR request message types
 */

#define	DR_VIO_CONFIGURE		0x494f43	/* 'IOC' */
#define	DR_VIO_UNCONFIGURE		0x494f55 	/* 'IOU' */
#define	DR_VIO_FORCE_UNCONFIG		0x494f46	/* 'IOF' */
#define	DR_VIO_STATUS			0x494f53	/* 'IOS' */

/*
 * VIO DR Request message definition.
 */
typedef struct {
	uint64_t	req_num;	/* request number */
	uint64_t	dev_id;		/* device ID */
	uint32_t	msg_type;	/* message type */
	char		name[1];	/* device name (null-terminated) */
} dr_vio_req_t;


/*
 * Result codes for DR_CPU_OK
 */
#define	DR_VIO_RES_OK			0x0
#define	DR_VIO_RES_FAILURE		0x1
#define	DR_VIO_RES_BLOCKED		0x2
#define	DR_VIO_RES_NOT_IN_MD		0x3

/*
 * Status codes for DR_VIO_OK
 */
#define	DR_VIO_STAT_NOT_PRESENT		0x0
#define	DR_VIO_STAT_UNCONFIGURED	0x1
#define	DR_VIO_STAT_CONFIGURED		0x2

#define	DR_VIO_MAX_ERR_SIZE		256

/*
 * Status/Response from VIO DR operation
 */
typedef struct {
	uint64_t	req_num;	/* request number */
	uint32_t	result;		/* result code */
	uint32_t	status;		/* device status */
	char		reason[1];	/* err description (null-terminated) */
} dr_vio_res_t;


extern int dr_vio_configure(ldom_t *ldp, uint64_t dev_id, char *name,
		bool_t force, response_t *resp);
extern int dr_vio_unconfigure(ldom_t *ldp, uint64_t dev_id, char *name,
		bool_t force, response_t *resp);

typedef int (*dr_vio_cfg_f)(ldom_t *, uint64_t, char *, bool_t, response_t *);

#ifdef __cplusplus
}
#endif

#endif /* _DR_VIO_H */
