/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2010 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#ifndef	_ERRORS_H
#define	_ERRORS_H

#ifdef __cplusplus
extern "C" {
#endif

enum {
	ERR_GENERIC 			= -1,
	ERR_MD_UPDATE_NOTSUP 		= -2,
	ERR_LDOM_IS_BOUND		= -3,
	ERR_NO_SERVICE			= -4,
	ERR_SERVICE_EXISTS		= -5,
	ERR_NO_CLIENT			= -6,
	ERR_CLIENT_EXISTS		= -7,
	ERR_SERVICE_BOUND_CLIENTS	= -8,
	ERR_LDOM_LOCKED			= -9,
	ERR_NO_CHANGES			= -10,

	ERR_CPU_DR_BOUND_MAUS 		= -100,
	ERR_CPU_DR_NOTSUP 		= -101,
	ERR_CPU_DELETE_ALL 		= -102,
	ERR_CPU_DELETE_NUM_BOUND	= -103,
	ERR_CPU_NOT_BOUND		= -104,
	ERR_CPU_DR_OP_FAILED		= -105,
	ERR_CPU_DR_OP_NO_RESPONSE	= -106,
	ERR_CPU_DR_PARTIAL		= -107,
	ERR_CPU_DR_MIG_DISABLED		= -108,

	ERR_DR_VIO_NOTSUP 		= -120,
	ERR_DR_VIO_IN_USE 		= -121,
	ERR_DR_VIO_FAILED		= -122,
	ERR_DR_VIO_NO_RESPONSE		= -123, /* timeout waiting for resp */
	ERR_DR_VIO_UNK_RESPONSE		= -124,	/* couldn't parse DS resp */
	ERR_DR_VIO_ROLLBACK		= -125,	/* add failed, do rollback */
	ERR_DR_VIO_INVALID_ARG		= -126,
	ERR_DR_VIO_NOT_IN_MD		= -127,
	ERR_DR_VIO_STILL_ALLOCATED	= -128, /* used to keep MAC nodes */

	ERR_CRYPTO_DR_NOTSUP		= -140,
	ERR_CRYPTO_DR_OP_FAILED		= -141,
	ERR_CRYPTO_DR_OP_NO_RESPONSE	= -142,
	ERR_CRYPTO_DR_PARTIAL		= -143,

	ERR_MEM_NO_MBLOCK_IDS		= -200,
	ERR_MEM_BIND_REAL		= -201,
	ERR_MEM_DR_UNALIGN_SIZE		= -202,
	ERR_MEM_DR_DELAYED_RECONF	= -203,
	ERR_MEM_DR_OP_PARTIAL		= -204,
	ERR_MEM_DR_OP_NO_RESPONSE	= -205,
	ERR_MEM_DR_OP_FAILED		= -206,
	ERR_MEM_DR_OP_CANCELLED		= -207,
	ERR_MEM_DR_OP_LOCKED		= -208,
	ERR_MEM_DR_NOTSUP		= -209,
	ERR_MEM_DR_VERIFY_FAILED	= -210,
	ERR_MEM_DR_OP_DOMAIN_FAILED	= -211,
	ERR_MEM_DR_HV_NOTSUP		= -212,
	ERR_MEM_DR_SPARSE_MEM		= -213,
	ERR_MEM_DR_SCRUB_FAILED		= -214,

	ERR_LDC_NO_IDS			= -300,
	ERR_LDC_NO_CHANS		= -301,
	ERR_LDC_CHAN_ALLOCATED		= -302,

	ERR_NO_DELRECONF_ADD		= -400,
	ERR_NO_DELRECONF_OTHER		= -401,
	ERR_NO_DELRECONF_REMOVE		= -402,
	ERR_NO_DELRECONF_REMOVE_2PHASE	= -403,

	ERR_VDS_VOLUME_EXISTS		= -500,
	ERR_VDS_VOLUME_NOEXISTS		= -501,
	ERR_VDS_VOLUME_BUSY		= -502,
	ERR_VDS_BLKDEV_EXISTS		= -503,
	ERR_VDS_ARGS_INVALID		= -504,

	ERR_VSW_DEV_PATH_EXISTS		= -600,
	ERR_VSW_LINKPROP_VSW		= -601,
	ERR_VSW_LINKPROP_VNET		= -602,

	ERR_VNET_NO_MODIFY_BOUND	= -700,
	ERR_VNET_MODE_INVALID		= -701,

	ERR_NET_VID_INVALID		= -750,

	ERR_VCC_PORT_RANGE_INVALID	= -800,
	ERR_VCC_PORT_OUT_OF_RANGE	= -801,

	ERR_CONSOLE_NO_PORTS		= -900,
	ERR_CONSOLE_PORT_EXISTS		= -901,
	ERR_CONSOLE_NO_GROUP		= -902,
	ERR_CONSOLE_DUP_ID		= -903,

	ERR_VDISK_NO_MODIFY_BOUND	= -1000,

	ERR_DUP_ID			= -1100,
};

/*
 * In general parsable output should not be localized, i.e. the syntax is
 * locale-independent. Therefore we create i18n and non-i18n versions of
 * these responses
 */
#define	MSG_YES		"yes"
#define	MSG_NO		"no"
#define	MSG_YES_I18N	gettext("yes")
#define	MSG_NO_I18N	gettext("no")

/*
 * Error messages for object names
 */

#define	MSG_ERR_NAME_TOO_SHORT \
gettext("Name cannot be empty\n")

#define	MSG_ERR_NAME_TOO_LONG \
gettext("Name cannot be longer than %d characters\n")

#define	MSG_ERR_NAME_ALPHANUM \
gettext("Name must start with an alpha-numberic character\n")

#define	MSG_ERR_VAR_VALUE_TOO_LONG \
gettext("Variables cannot be longer than %d characters\n")

/*
 * Error messages for generic operations
 */

#define	MSG_ERR_SERVICE_BOUND_CLIENTS \
gettext("Service is active and being used by bound client(s)\n")

#define	MSG_ERR_MD_NOTSUP \
gettext("LDom %s does not support runtime reconfiguration\n")

#define	MSG_ERR_NO_LDOM_SPECIFIED \
gettext("No LDom specified\n")

#define	MSG_ERR_ILLEGAL_LDOM_NAME \
gettext("Illegal LDom name %s\n")

#define	MSG_ERR_FAIL_SEND_CMD \
gettext("Failed to send '%s' command to ldmd(1m)\n")

#define	MSG_ERR_OP_NOT_ALLOWED \
gettext("The operation on LDom %s is not allowed at this time\n")

#define	MSG_ERR_ID_EXISTS \
gettext("Id already exists\n")

#define	MSG_ERR_PRIV \
gettext("Cannot setup privileges\n")

#define	MSG_ERR_PRIV_SET \
gettext("Cannot %s %s privilege\n")

/*
 * Error messages for VIO operations
 */
#define	MSG_ERR_DR_VIO_IN_USE \
gettext("VIO operation failed because device is being used in LDom %s\n")

#define	MSG_ERR_DR_VIO_NOTSUP \
gettext("LDom %s does not support dynamic reconfiguration of IO devices\n")

#define	MSG_ERR_VIO_DR_CONFIG_OP_FAILED \
gettext("Unable to configure VIO device into LDom %s\n")

#define	MSG_ERR_VIO_DR_CONFIG_OP_NO_RESP \
gettext("LDom %s did not respond to request to configure VIO device\n")

#define	MSG_ERR_VIO_DR_CONFIG_UNK_STATE \
gettext("VIO device is considered to be allocated to LDom, but might not\n" \
	"be available to the guest OS\n")

#define	MSG_ERR_DR_VIO_INVALID_ARG \
gettext("Internal error (%d) during DR operation on VIO device in LDom %s\n")

#define	MSG_ERR_VIO_DR_UNCONFIG_OP_FAILED \
gettext("Unable to unconfigure VIO device in LDom %s\n")

#define	MSG_ERR_VIO_DR_UNCONFIG_OP_NO_RESP \
gettext("LDom %s did not respond to request to unconfigure VIO device\n")

#define	MSG_ERR_VIO_UNKNOWN_FAILURE \
gettext("Unknown failure (%d) occurred during VIO operation on LDom %s\n")

#define	MSG_ERR_VIO_DR_REASON \
gettext("Guest LDom returned the " \
	"following reason for failing the operation:\n")

/*
 * Generic Delayed Reconfiguration messages
 */
#define	MSG_ERR_DEL_RECONF_NOTSUP \
gettext("Delayed reconfiguration of LDom %s is not permitted\n")

#define	MSG_ERR_DO_OP_BOUND_OR_INACTIVE \
gettext("Please perform the operation while the LDom is bound or inactive\n")

#define	MSG_ERR_COMPLETE_RECONFIG \
gettext("Please complete the delayed reconfiguration of LDom %s first\n")


/*
 * Domain Migration error messages
 */
#define	MSG_ERR_MIG_SUCCESS \
gettext("Domain Migration of LDom %s succeeded\n")

#define	MSG_ERR_MIG_FAILED \
gettext("Domain Migration of LDom %s failed\n")

#define	MSG_ERR_MIG_AUTH_FAILED \
gettext("Authorization failed on target LDoms Manager\n")

#define	MSG_ERR_MIG_DRY_RUN_FAILURE \
gettext("Domain Migration of LDom %s would fail if attempted\n")

#define	MSG_ERR_MIG_INVALID_STATE \
gettext("LDom %s is currently %s and cannot be migrated\n")

#define	MSG_ERR_MIG_NOT_IN_PROGRESS \
gettext("LDom %s is not in the process of a Domain Migration\n")

#define	MSG_ERR_MIG_DELAYED_RECONF_IN_PROGRESS \
gettext("LDom %s is in the process of a delayed reconfiguration on %s.\n")

#define	MSG_ERR_MIG_CANNOT_CANCEL_ON_TARGET \
gettext("Domain Migration of LDom %s cannot be cancelled from target machine\n")

#define	MSG_ERR_MIG_IDENTICAL_TARGET_SOURCE \
gettext("Source and target hosts (%s) are identical\n")

#define	MSG_ERR_MIG_IN_PROGRESS \
gettext("A Domain Migration is already in progress\n")

#define	MSG_ERR_MIG_IN_PROGRESS_ON_LDOM \
gettext("A Domain Migration is already in progress on LDom %s\n")

#define	MSG_ERR_MIG_SOURCE_DESTROY \
gettext("LDom is running on target but failed to destroy instance on source\n")

#define	MSG_ERR_MIG_FAILED_SRC_HOSTNAME \
gettext("Failed to get hostname of source host\n")

#define	MSG_ERR_MIG_CANCEL_COMPLETED \
gettext("Domain Migration of LDom %s has been cancelled\n")

#define	MSG_ERR_MIG_UNEXPECTED_ERROR \
gettext("Domain Migration of LDom %s failed, unexpected error: %d\n")

#define	MSG_ERR_MIG_CPU_SPEED_MISMATCH \
gettext("The CPU speeds on the source and target machines do not match\n")

#define	MSG_ERR_MIG_CPU_TYPE_MISMATCH \
gettext("The CPU types on the source and target machines do not match\n")

#define	MSG_ERR_MIG_CPU_CORE_UNAVAIL \
gettext("Not enough full CPU cores available for migration\n")

#define	MSG_ERR_MIG_CPU_REDUCED \
gettext("LDom may have reduced number of VCPUs following aborted migration\n")

#define	MSG_ERR_MIG_NO_MIGRATE_PRIMARY \
gettext("Control LDom cannot be migrated\n")

#define	MSG_ERR_MIG_CONNECTION_LOST \
gettext("Lost connection to target machine %s\n")

#define	MSG_ERR_MIG_PHYS_DEVS_BOUND \
gettext("Physical devices are bound to LDom %s\n")

#define	MSG_ERR_MIG_EXPORTING_SERVICES \
gettext("LDom %s is bound and exporting services\n")

#define	MSG_ERR_MIG_NO_TGT_VDS \
gettext("Failed to find required vds %s on target machine\n")

#define	MSG_ERR_MIG_NO_TGT_VOL \
gettext("Failed to find required volume %s@%s on target machine\n")

#define	MSG_ERR_MIG_NO_TGT_VCC \
gettext("Failed to find any virtual console concentrator (vcc) on " \
"target machine\n")

#define	MSG_ERR_MIG_NO_CONS_PORT \
gettext("Unable to bind console for LDom %s. No console tcp ports available\n")

#define	MSG_ERR_MIG_VOL_IN_MPGROUP \
gettext("Failed because volume %s@%s is part of a multipath group" \
" (mpgroup) %s\n")

#define	MSG_ERR_MIG_VCPCS_PRESENT \
gettext("Failed because LDom contains one or more virtual data plane " \
"channel server(s) (vdpcs)\n")

#define	MSG_ERR_MIG_VCPCC_PRESENT \
gettext("Failed because LDom contains one or more virtual data plane " \
"channel client(s) (vdpcc)\n")

#define	MSG_ERR_MIG_SLAVE_PRESENT \
gettext("Failed because LDom is a dependency for one or more LDoms\n")

#define	MSG_ERR_MIG_NO_TGT_VSW \
gettext("Failed to find required vsw %s on target machine\n")

#define	MSG_ERR_MIG_CANNOT_SAVE_STATE \
gettext("Unable to save %s resource state\n")

#define	MSG_ERR_MIG_HV_DOWNREV \
gettext("System Firmware version on %s does not support Domain Migration\n")

#define	MSG_ERR_MIG_NO_BOUND_CPUS \
gettext("LDom %s does not have any bound vcpus\n")

#define	MSG_ERR_MIG_MD_PARSE_FAIL \
gettext("Unable to parse guest machine description (MD) on target machine\n")

#define	MSG_ERR_MIG_OUTGOING_MIGRATION_DISABLED \
gettext("Migrating a LDom to another machine is disabled. It can be\n" \
"enabled by setting the 'outgoing_migration_enabled' property\n" \
"of the 'ldmd' service to 'true' using svccfg(1M).\n")

#define	MSG_ERR_MIG_CONN_FAILED \
gettext("Failed to establish connection with ldmd(1m) on target: %s\n" \
"Check that the 'ldmd' service is enabled on the target machine and\n"	\
"that the version supports Domain Migration. Check that the 'xmpp_enabled'\n" \
"and 'incoming_migration_enabled' properties of the 'ldmd' service on\n" \
"the target machine are set to 'true' using svccfg(1M).")

#define	MSG_ERR_MIG_PM \
gettext("Migration is disabled because Power Management is active.\n" \
"Set PM policy to performance to enable migration.\n")

#define	MSG_ERR_MIG_COMPRESSION \
gettext("Failed to compress memory contents for transfer\n")

#define	MSG_ERR_MIG_DECOMPRESSION \
gettext("Failed to decompress received memory contents\n")

#define	MSG_ERR_MIG_COMPRESSION_SIZE \
gettext("Failed to initialize compression for memory transfer\n")

/*
 * vdsdev error messages
 */
#define	MSG_ERR_ADD_DEV_FAILED \
gettext("Failed to add device %s\nas %s@%s because this device is already " \
"exported on LDom %s.\n")

#define	MSG_ERR_VDSDEV_ONLY_PERMITTED \
gettext("You can only add multiple instances of the same device when\n%s.\n")

/*
 * XML error messages
 */
#define	MSG_ERR_TAG_MISSING_DATA \
gettext("Malformed XML Document: %s tag missing data")

#define	MSG_ERR_SECTION_MISSING_CHILDREN \
gettext("Malformed XML Document: %s Section contains no children tags")

#define	MSG_ERR_SECTION_HAS_NO_PROPERTY \
gettext("Malformed XML Document: %s Section contains no property")

#define	MSG_ERR_VDSDEV_MISSING_TAG \
gettext("Malformed XML Document: %s tag missing one of %s or %s tag")

#define	MSG_ERR_MISSING_PROPERTY \
gettext("Malformed XML Document: %s tag missing %s property value")

#define	MSG_ERR_INVALID_FILE_SIZE \
gettext("Invalid file size for %s")

#define	MSG_ERR_UNABLE_TO_OPEN \
gettext("Unable to open %s for creation: %s")

#define	MSG_ERR_UNABLE_TO_CREATE \
gettext("Unable to create %s file")

#define	MSG_ERR_ZVOL_CREATE \
gettext("ZFS pool location incorrect")

#define	MSG_ERR_MKFILE_AND_ZVOL \
gettext("mkfile and zvol properties must not be used at the same time")

#define	MSG_ERR_VDISK_MISSING_ALL \
gettext("vdisk node missing all properties")

#define	MSG_ERR_VSW_MUST_HAVE_DATA \
gettext("%s must contain data for add-vsw")

#define	MSG_ERR_VNET_MUST_HAVE_DATA \
gettext("%s must contain data for add-vnet")

#define	MSG_ERR_LINKPROP_INVALID_VAL \
gettext("Invalid linkprop value\n")

#define	MSG_ERR_LINKPROP_NO_NET_DEV \
gettext("Failed because linkprop requires the virtual switch \"%s\"\n" \
"must first have a physical network device assigned\n")

#define	MSG_ERR_VSW_LINKPROP \
gettext("Failed to modify virtual switch because the linkprop of the " \
"virtual\nswitch requires that it has a physical network device assigned\n")

#define	MSG_ERR_VNET_LINKPROP \
gettext("Failed to modify virtual switch because the linkprop of at least\n" \
"one virtual network device requires that the virtual switch has\n" \
"a physical network device assigned\n")

#define	MSG_ERR_BIND_VSW_NO_NET_DEV \
gettext("Failed to bind virtual switch because the linkprop of the " \
"virtual\nswitch does not have a physical network device assigned\n")

#define	MSG_ERR_BIND_VSW_NO_NET_DEV_VNET \
gettext("Failed to bind virtual switch because the linkprop of at least\n" \
"one virtual network device requires that the virtual switch has\n" \
"a physical network device assigned\n")

#define	MSG_ERR_BIND_VNET_NO_NET_DEV \
gettext("Failed to bind virtual network device because the linkprop of the " \
"virtual\nswitch \"%s\" does not have a physical network device assigned\n")

#define	MSG_ERR_REBOOT_BIND \
gettext("Failed Reboot Bind Command of %s: %s\n")

#define	MSG_ERR_REBOOT_START_ALL \
gettext("Failed Reboot Start Command for all Domains: %s\n")

#define	MSG_ERR_REBOOT_START \
gettext("Failed Reboot Start Command of %s:\n")

#define	MSG_ERR_REBOOT_SPCONFIG \
gettext("Failed Reboot Add-spconfig Command for %s:%s\n")

#define	MSG_ERR_REBOOT_FILE_CREATION \
gettext("Unable to create reboot file")

#define	MSG_ERR_REBOOT_SYSTEM \
gettext("Unable to reboot system: %s")

#define	MSG_ERR_REBOOT_FILE_REMOVAL \
gettext("Failed to remove reboot file. Remove %s for proper behavior the " \
"next time LDMD is restarted")

/* Autosave error messages */
#define	MSG_ERR_AUTOSAVE_NOTIFY \
gettext("Autorecovery Policy is NOTIFY. Autosave configuration, '%s', is\n" \
	"newer than the configuration stored in the SP. Use either the\n" \
	"ldm add-spconfig -r <cfg> or ldm add-spconfig -r <cfg> <newcfg>\n" \
	"command to save the autosave configuration to the SP, and " \
	"power cycle\n" \
	"the system for the updated SP configuration to take effect. The\n" \
	"requested command was aborted, reissue your command to perform the\n" \
	"operation you intended. No more autosave warning messages will be " \
	"issued.\n")

#define	MSG_ERR_AUTOSAVE_CONFIG_NAME_INVALID \
gettext("%s is not a valid autosave configuration name\n")

#define	MSG_ERR_AUTOSAVE_CONFIG_INVALID \
gettext("Autosave configuration '%s' is invalid\n")

#define	MSG_ERR_AUTOSAVE_NO_SUCH_CONFIG \
gettext("Autosave configuration '%s' does not exist\n")

#define	MSG_ERR_AUTOSAVE_NO_CONFIGS \
gettext("No autosave configurations\n")

#define	MSG_ERR_AUTOSAVE_MDSTORE_OPERATION \
gettext("The requested operation could not be performed because the " \
	"communication\n" \
	"channel between the LDom Manager and the system controller is " \
	"down.\n" \
	"To recover, restart the LDom Manager.\n")

#define	MSG_ERR_AUTOSAVE_CANT_REMOVE \
gettext("Can't remove autosave configuration '%s'\n")

#define	MSG_ERR_AUTOSAVE_CANT_RENAME \
gettext("Can't rename autosave configuration '%s' to '%s', autosave\n" \
	"configuration '%s' will be unchanged.\n")

#define	MSG_ERR_AUTOSAVE_CANT_COPY \
gettext("SP configuration '%s' was successfully downloaded, but can't\n" \
	"copy autosave configuration '%s' to '%s'\n")

#define	MSG_ERR_DRM_DISABLED_BY_PM \
gettext("Resource Management is disabled while Power Management is enabled\n")

#define	MSG_ERR_GUEST_DEP_BAD_FIRMWARE \
gettext("System Firmware version does not support setting guest dependencies\n")

#define	MSG_ERR_MIG_LDOM_EXISTS_ON_TGT \
gettext("LDom %s already exists on the target machine\n")

/*
 * Memory DR messages
 */
#define	MSG_ERR_NO_ENOUGH_MEM	\
gettext("Not enough free memory present to meet this request\n")

#define	MSG_ERR_SEGMENT_LIMIT	\
gettext("Unable to bind memory: limit of %"PRId16" segments reached\n")

#define	MSG_ERR_BINDING_LIMIT	\
gettext("Unable to bind memory: limit of %"PRId16" bindings reached\n")

#define	MSG_WARN_MEMORY_CONSTRAINS  \
gettext("Warning: The LDom %s has been allocated more memory (%"PRId64 \
	"MB)\nthan requested because of memory alignment constraints.\n")

#define	MSG_WARN_INC_MEM_ALLOC \
gettext("Warning: memory allocation of LDom %s is below minimum " \
	"needed for boot\nloader. Increasing allocation to %s\n")

#define	MSG_ERR_MEM_DELAYED_RECONF_IN_PROGRESS \
gettext("LDom %s is in the process of a delayed reconfiguration\n" \
	"Memory DR is disabled until %s reboots\n")

#define	MSG_ERR_MEM_LDOM_LOCK \
gettext("Memory DR is disabled because LDom %s is locked\n")

#define	MSG_ERR_MEM_DR_NOTSUP \
gettext("LDom %s does not support dynamic reconfiguration of memory\n")

#define	MSG_ERR_MEM_SIZE_UNALIGN \
gettext("Size of memory must be multiple of %"PRId16"MB\n")

#define	MSG_ERR_MEM_DR_NOT_IN_PROGRESS \
gettext("LDom %s has no memory removal in progress\n")

#define	MSG_ERR_MEM_DR_CANNOT_CANCEL_ADD \
gettext("Cancel memory add is not supported\n")

#define	MSG_ERR_MEM_DR_DONE_CANNOT_CANCEL \
gettext("Memory removal is done: cannot cancel\n")

#define	MSG_ERR_MEM_DR_CANNOT_DEL_ALL_MEM \
gettext("%s equals memory that is currently allocated\n")

#define	MSG_ERR_MEM_DR_CANNOT_DEL_MORE \
gettext("%s is more memory than is currently allocated (%s)\n")

#define	MSG_ERR_MEM_DR_SET_MEM_ALIGN	\
gettext("set-mem %s would %s %"PRId64"MB, which is not multiple of %"PRId16 \
	"MB, suggest to %s %"PRId64"MB for %"PRId16"MB alignment\n")

#define	MSG_ERR_MEM_DR_REMOVE_FAILED \
gettext("Resource removal failed\n")

#define	MSG_ERR_MEM_DR_ADD_FAILED \
gettext("Resource addition failed\n")

#define	MSG_ERR_MEM_DR_MODIFY_FAILED \
gettext("Resource modification failed\n")

#define	MSG_ERR_MEM_DR_DOMAIN_FAILED \
gettext("Memory DR aborted (error %d)\n")

#define	MSG_ERR_MEM_DR_CANCELLED \
gettext("Memory DR canceled\n")

#define	MSG_ERR_MEM_DR_PARTIAL_CANCEL \
gettext("But %"PRId64"%c already removed from LDom %s\n")

#define	MSG_ERR_MEM_DR_NO_RESP \
gettext("Request sent to LDom %s but no valid response received\n")

#define	MSG_ERR_MEM_DR_VERIFY_FAILED \
gettext("HV failed to verify unconfigured memory."\
	"LDom %s has been rebooted\n")

#define	MSG_ERR_MEM_DR_SCRUB_FAILED \
gettext("HV failed to scrub unconfigured memory")

#define	MSG_ERR_MEM_DR_PARTIAL_ADD \
gettext("Only %"PRId64"%c could be added to LDom %s\n")

#define	MSG_ERR_MEM_DR_PARTIAL_DEL \
gettext("Only %"PRId64"%c could be removed from LDom %s " \
	"because the rest of the memory is in use\n")

#define	MSG_ERR_MEM_DR_OP_FAILED \
gettext("LDom %s: %s of memory %"PRId64"%c failed\n")

#define	MSG_ERR_MEM_DR_HV_DOWNREV \
gettext("System Firmware version does not support memory DR\n")

#define	MSG_ERR_MEM_DR_SPARSE_MEM \
gettext("Sparse memory configuration does not support memory DR\n")

#define	MSG_ERR_MEM_ASSIGN_UNALIGN \
gettext("Assign unalign memory is disabled because LDom %s is in the process" \
	"of memory DR. \n")

#define	MSG_ERR_MEM_NO_UNALIGN_MEM \
gettext("No unaligned memory.\n")

#define	MSG_ERR_MEM_UNALIGN_MEM_CONTROL_DOMAIN \
gettext("Unaligned memory can only be allocated to the control domain.\n")

/*
 * MAC address collision detection error messages
 */
#define	MSG_ERR_MAC_SOCKET_CREATION_FAILED \
gettext("Multicast socket creation failed.\n")

#define	MSG_ERR_MAC_SET_MANUALLY \
gettext("Please set domain MAC addresses manually.\n")

#define	MSG_ERR_MAC_REMOTE_COLLISION \
gettext("Cannot check remote MAC address collision.\n")

#define	MSG_ERR_CHECK_NETWORK \
gettext("Please check your network, automatic MAC allocation not available.\n")

#ifdef __cplusplus
}
#endif

#endif	/* _ERRORS_H */
