/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2009 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#pragma ident	"@(#)fma_pri_svc.c	1.9	09/02/09 SMI"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <alloca.h>
#include <errno.h>

#include "ldom.h"
#include "zpri.h"
#include "allocate.h"
#include "zprintf.h"
#include "ds.h"
#include "ds_impl.h"
#include "ds_registry.h"
#include "support.h"

typedef struct {
	uint64_t	req_num;
} fma_req_pri_t;

typedef struct {
	uint64_t	req_num;
	uint8_t		pri[];
} fma_pri_resp_t;

void
pri_data_handler(ds_svc_t *svc, void *buf, size_t buflen)
{
	fma_req_pri_t	*pri_req;
	void		*resp;
	fma_pri_resp_t	*pri_resp;
	size_t		resp_len, data_len;
	int		md_size;
	uint8_t		*mdbuf;

	if (buflen != sizeof (ds_data_handle_t) + sizeof (fma_req_pri_t)) {
		Xfree(buf);
		return;
	}

	pri_req = (fma_req_pri_t *)((char *)buf + sizeof (ds_data_handle_t));

	md_size = md_gen_bin(pri.core_mdp, &mdbuf);

	data_len = sizeof (fma_pri_resp_t) + md_size;
	resp_len = sizeof (ds_hdr_t) + sizeof (ds_data_handle_t) + data_len;
	resp = Xmalloc(resp_len);
	ds_fill_data_header(resp, svc->hdl, data_len);
	pri_resp = (fma_pri_resp_t *)((char *)resp + sizeof (ds_hdr_t) +
	    sizeof (ds_data_handle_t));
	pri_resp->req_num = pri_req->req_num;
	memcpy(pri_resp->pri, mdbuf, md_size);

	ds_chan_send_msg(svc->chan, resp, resp_len);

	Xfree(mdbuf);
	Xfree(resp);
	Xfree(buf);
}

static prot_version_t versions[] = {
	{ 1, 0 }
};

static svc_ops_t fma_pri_service_ops = {
	NULL,				/* reg_handler */
	NULL,				/* unreg_handler */
	pri_data_handler,		/* data_handler */
	NULL,				/* seq_handler */
	NULL				/* data */
};

svc_info_t *
info(void)
{
	svc_info_t *info;

	info = Xcalloc(1, svc_info_t);
	info->svcs_ver.major_number = SVCS_VER_MAJOR;
	info->svcs_ver.minor_number = SVCS_VER_MINOR;
	info->num_chans = 1;
	info->chan_names = Xmalloc(info->num_chans * sizeof (char *));
	info->chan_names[0] = Xstrdup(FMA_CHAN_NAME);
	info->num_svcs = 1;
	info->svcs = Xmalloc(info->num_svcs * sizeof (char *));
	info->svcs[0] = Xstrdup(FMA_PRI_SVC);
	info->props = Xmalloc(info->num_svcs * sizeof (int));
	info->props[0] = SVC_OUTGOING;
	info->num_vers = NUM_VERS(versions);
	info->versions = Xmalloc(info->num_vers * sizeof (prot_version_t));
	memcpy(info->versions, versions,
	    info->num_vers * sizeof (prot_version_t));

	return (info);
}

svc_ops_t *
ops(char *svc_name)
{
	if (strcmp(svc_name, FMA_PRI_SVC) == 0)
		return (&fma_pri_service_ops);
	else
		return (NULL);
}
