/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2010 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#ifndef	_HVCTL_H
#define	_HVCTL_H

#ifdef __cplusplus
extern "C" {
#endif

/*
 * From hypervisor.h.
 */
#define	SIS_NORMAL		0x1
#define	SIS_TRANSITION		0x2
#define	SOFT_STATE_SIZE		32

/*
 * From guest.h.
 */
#define	GUEST_STATE_STOPPED		0x0	/* dead pending restart */
#define	GUEST_STATE_RESETTING		0x1	/* in process of resetting */
#define	GUEST_STATE_NORMAL		0x2	/* running normally */
#define	GUEST_STATE_SUSPENDED		0x3	/* suspended for migrating */
#define	GUEST_STATE_EXITING		0x4	/* in process of exiting */
#define	GUEST_STATE_UNCONFIGURED	0xff	/* unused */

/*
 * HV MD version info
 */
#define	HVMD_VERSION_MAJOR_NUMBER 1ULL
#define	HVMD_VERSION_MINOR_NUMBER 3

/* HV MD version number that uses Direct I/O HV MD format */
#define	HVMD_DIO_VERSION_MAJOR_NUMBER 1ULL
#define	HVMD_DIO_VERSION_MINOR_NUMBER 3

/* migration information */
#define	HVMD_PROP_MIG_TGT		"mig_target"
#define	HVMD_PROP_MIG_SRC		"mig_source"
#define	HVMD_PROP_TICK_SYNC		"tick_sync_mode"
#define	HVMD_PROP_MIG_RSVD		"mig_rsvd"
#define	HVMD_PROP_MIG_MDPA		"mig_mdpa"

/* guest dependency information */
#define	HVMD_NODE_GUESTDEPENDENCY	"guestdependency"
#define	HVMD_NODE_GUESTCONTROL		"guestcontrol"
#define	HVMD_PROP_DEPENDENCY_GID	"guest_id"

/*  memory dr property */
#define	HVMD_PROP_MEMDR_IN_PROGRESS		"memdr_in_progress"


typedef enum {
	HVCTL_down,
	HVCTL_challenge,
	HVCTL_up
} hvctl_chan_state_t;

struct HVCTL_STATE {
	int			fd;
	hvctl_chan_state_t	hvctl_chan_state;
	bool_t			version_set;
	int			major_ver;
	int			minor_ver;
	int			zeus_seqn;
	int			hv_seqn;
} hvctl_state;

/*
 * Rest is from hvctl.h.
 */

#define	HVCTL_MSG_MAXLEN	64

#define	HVCTL_STATE_UNCONNECTED	0
#define	HVCTL_STATE_CHALLENGED	1
#define	HVCTL_STATE_CONNECTED	2

/*
 * 1.5		Memory Utilization
 * 1.4		Memory DR Verification + Multiple segments per mblock support
 * 1.3		Domain Migration
 * 1.2		Power Management
 * 1.1
 * 1.0		Initial version
 */
#define	HVCTL_VERSION_MAJOR_NUMBER	1LL
#define	HVCTL_VERSION_MINOR_NUMBER	5LL

/*
 * HVCTL version that supports PM
 */
#define	HVCTL_PM_MAJOR_VERSION	1LL
#define	HVCTL_PM_MINOR_VERSION	2LL

/*
 * HVCTL version that supports Migration
 */
#define	HVCTL_MIG_MAJOR_VERSION	1LL
#define	HVCTL_MIG_MINOR_VERSION	3LL

/*
 * HVCTL version that supports Memory DR
 */
#define	HVCTL_MEM_DR_MAJOR_VERSION	1LL
#define	HVCTL_MEM_DR_MINOR_VERSION	4LL

/*
 * HVCTL version that supports multiple mblocks per segment
 */
#define	HVCTL_MULT_SEG_MAJOR_VERSION	1LL
#define	HVCTL_MULT_SEG_MINOR_VERSION	4LL

/*
 * HVCTL version that supports scrubbing memory
 */
#define	HVCTL_SCRUB_MEM_MAJOR_VERSION	1LL
#define	HVCTL_SCRUB_MEM_MINOR_VERSION	4LL

/*
 * HVCTL version that supports memory utilization
 */
#define	HVCTL_MEM_UTIL_MAJOR_VERSION	1LL
#define	HVCTL_MEM_UTIL_MINOR_VERSION	5LL


#define	HVCTL_HV_CHALLENGE_K	0xbadbeef20	/* to go away */
#define	HVCTL_ZEUS_CHALLENGE_K	0x12cafe42a	/* to go away */

#define	HVCTL_PM_COOKIE_MAXLEN		128
#define	HVCTL_RES_STATUS_DATA_SIZE	40

#define	HVPM_BUF_COUNT		2		/* toggle physmem buffer */
#define	HVPM_BUF_SIZE		(FRAG_SIZE / 2)

/*
 * HV PM Async Timeout
 */
#define	HV_PM_ASYNC_TIMEOUT	5

/*
 * HV Async Timeout
 */
#define	HV_ASYNC_TIMEOUT	60

#if !defined(_ASM)

/*
 * Hypervisor control message definitions
 */


/*
 * Type codes for HV control messages
 */


/*
 * The operation code
 */
typedef enum {
	HVctl_op_hello = 0,	/* Initial request to open hvctl channel */
				/* yields a response of the same if fails */
	HVctl_op_challenge,	/* challenge returned from HV to Zeus */
	HVctl_op_response,	/* Response from Zeus */
	HVctl_op_get_hvconfig,	/* Get the HV config pointers */
	HVctl_op_reconfigure,	/* Reconfigure request */
	HVctl_op_guest_start,	/* Start a guest */
	HVctl_op_guest_stop,	/* Stop a guest */
	HVctl_op_guest_delayed_reconf,	/* Delayed reconfigure on guest exit */
	HVctl_op_guest_suspend,	/* Suspend a guest */
	HVctl_op_guest_resume,	/* Resume a guest */
	HVctl_op_guest_panic,	/* Panic a guest */
	HVctl_op_get_res_stat,	/* Get resource status if supported */
	HVctl_op_new_res_stat,	/* Aync resource status update if supported */
	HVctl_op_cancel_reconf,	/* Cancel any pending delayed reconfigure */
	HVctl_op_guest_only_reconf,	/* Reconfigure just the guest MDs */
	HVctl_op_guest_only_delayed_reconf,	/* Delayed reconf just guests */
	HVctl_op_pm,		/* Power Management */
	HVctl_op_state_save,	/* Save guest state information */
	HVctl_op_state_restore,	/* Restore guest state information */
	HVctl_op_api_group_query,	/* Query API group feasability */
	HVctl_op_scrub_mem,	/* Scrub physical memory */
	HVctl_op_mem_dr_verify,	/* Verify guest has released memory */
} hvctl_op_t;

/*
 * Response codes.
 */
typedef enum {
	HVctl_st_ok	= 0,
	HVctl_st_bad_seqn,	/* Bad sequence number (dropped packet) */
	HVctl_st_eauth,		/* Not authorised */
	HVctl_st_enotsupp,	/* OP is not supported */
	HVctl_st_badmd,		/* Broken MD */
	HVctl_st_mdnotsupp,	/* Unsupported MD format */
	HVctl_st_rc_failed,	/* Reconfig failed */
	HVctl_st_einval,	/* Invalid argument specified */
	HVctl_st_eillegal,	/* Illegal operation requested */
	HVctl_st_stop_failed,	/* Stop request failed */
	HVctl_st_timedout,	/* OP has timed out */
	HVctl_st_async,		/* Status is delivered asynchronously */
	HVctl_st_apis_not_supported,	/* Listed API groups aren't supported */
	HVctl_st_again,		/* OP is not yet complete. Try again */
} hvctl_status_t;


/*
 * Resource codes ... in parse process order
 */
typedef enum {
	HVctl_res_guest = 0,	/* guest resource */
	HVctl_res_vcpu,		/* virtual cpu resource */
	HVctl_res_memory,	/* memory resource */
	HVctl_res_mau,		/* N1 crypto unit */
	HVctl_res_cwq,		/* Niagara II crypto unit */
	HVctl_res_ldc,		/* LDC */
	HVctl_res_console,	/* Console nodes */
	HVctl_res_hv_ldc,	/* HV LDC - to be deleted */
	HVctl_res_pcie_bus,	/* PCIE bus */
	HVctl_res_guestmd,	/* guest md */
	HVctl_res_network_device,	/* Network */
} hvctl_res_t;

/*
 * Format for initial hello.
 * Used for both init handshake, and for a nack response.
 */
typedef struct hvctl_hello {
	uint16_t	major;
	uint16_t	minor;
} hvctl_hello_t;

/*
 * Format used for an ack response to a handshake
 * holds the challenge code required by the HV and the HV's version numbers.
 */
typedef struct hvctl_challenge {
	uint64_t	code;
	uint16_t	major;
	uint16_t	minor;
} hvctl_challenge_t;

/*
 * Request to return the HVs current config info
 */
typedef struct hvctl_hvconfig {
	uint64_t	hv_membase;
	uint64_t	hv_memsize;
	uint64_t	hvmdp;
	uint64_t	del_reconf_hvmdp;
	uint32_t	del_reconf_gid;
} hvctl_hvconfig_t;

/*
 * Payload and response packet used for operation HVctl_op_scrub_mem.
 * If the operation was successful, hv_memsize will contain the actual
 * number of bytes that were scrubbed and hv_membase will be zero. If
 * the operation failed, hv_membase of the response packet will contain
 * the error code.
 */
typedef struct hvctl_scrub_mem {
	uint64_t	hv_membase;
	uint64_t	hv_memsize;
} hvctl_scrub_mem_t;

/*
 * Payload and response packet used for operation HVctl_op_mem_dr_verify.
 * Zeus calls HV with a PA, RA and size of the memory that a guest has just
 * released.  In addition, zeus also passes in a cookie which is initialized
 * to 0.
 * If the HV responds with a status of HVctl_st_again with a cookie value,
 * Zeus will call the hvctl op again using the same PA, RA and size of
 * the memory with the cookie value returned by HV.
 * The HV will respond with HVctl_st_ok when the verification succeeds and
 * HVctl_st_rc_failed if the guest did not properly release the memory.
 */
typedef struct hvctl_mem_dr_verify {
	uint64_t    paddr;		    /* physical address */
	uint64_t    size;		    /* memory size */
	uint64_t    raddr;		    /* real address */
	uint32_t    guestid;		    /* guest domain id */
	uint32_t    cookie;		    /* HV cookie */

} hvctl_mem_dr_verify_t;

/*
 * Configuration change request.
 *
 * HV figures out the rest and lets us know what if anything can't be done.
 *
 * guestid is invalid if this is not a delayed config request
 *
 * The command is acked if the MD parses OK, but the actual reconfigure
 * is pended until the guest defined by guestid requests an exit or a reboot.
 */
typedef struct hvctl_reconfig {
	uint64_t	hvmdp;
	uint32_t	guestid;
} hvctl_reconfig_t;

/*
 * If the new config fails for some reason this is the response packet.
 *
 * The HV stops on first failure.
 *
 * The packet contains the node index of the resource that could not be
 * configured. The resource type (see above) and a resource type specific
 * failure code as to why. The remainder of the packet may (at some point)
 * contain further resource specific information.
 */
typedef struct hvctl_rc_fail {
	uint64_t	hvmdp;	/* the one that failed */
	uint32_t	res;	/* code of the resource the failed */
	uint32_t	code;	/* resource specific failure code */
	uint32_t	nodeidx;	/* idx of node in given HVMD */
	uint32_t	resid;	/* resource id of resource */
} hvctl_rc_fail_t;

/*
 * Payload and response packet used for other guest operations:
 * HVctl_op_guest_start
 * HVctl_op_guest_stop
 * HVctl_op_guest_suspend
 * HVctl_op_guest_resume
 * HVctl_op_guest_panic
 *
 * code is unused (set 0) in request from Zeus, but may contain a guest
 * resource specific error code from the HV in the event that the op fails
 * and more info than just the hvctl_status_t in the pkt header is needed.
 */
typedef struct hvctl_guest_op {
	uint32_t	guestid;
	uint32_t	code;
} hvctl_guest_op_t;

/*
 * State save/restore resource types.
 */
typedef enum state_restype {
	STATE_RES_GUEST,
	STATE_RES_VCPU,
	STATE_RES_LDC
} state_restype_t;

/*
 * Payload for state save operations.
 */
typedef struct hvctl_state_save_op {
	uint64_t	statepa;
	uint32_t	guestid;
	uint32_t	restype;
	uint32_t	resid;
} hvctl_state_save_op_t;

/*
 * Resource status/statistics request and async notice packet. The payload
 * is resource specific, and so left in a general form here.
 *
 * In the get request payload the data field is ignored.
 *
 * For example, this is used for a guest status update notification
 *	The data payload holds the guest state info.
 *	The side-effect of the get form in this case is to clear
 *	any no-notify on change state.
 */
typedef struct hvctl_res_status {
	uint32_t	res;	/* code of the resource type */
	uint32_t	resid;	/* ID of the instance of resource type */
	uint32_t	infoid;	/* ID of the info within the resource */
	uint32_t	code;	/* resource specific failure code */
	uint8_t		data[HVCTL_RES_STATUS_DATA_SIZE];
} hvctl_res_status_t;

/*
 * Info identifiers for guest resource.
 */
typedef enum {
	HVctl_info_guest_state,
	HVctl_info_guest_soft_state,
	HVctl_info_guest_tod,
	HVctl_info_guest_utilisation,
	HVctl_info_guest_api_groups,
	HVctl_info_guest_tick_sync,
	HVctl_info_guest_max
} hvctl_guest_info_t;

/*
 * Resource state returned in data for the
 * HVctl_info_guest_state infoid.
 */
typedef struct rs_guest_state {
	uint64_t	state;
} rs_guest_state_t;

/*
 * Resource state returned in data for the
 * HVctl_info_guest_soft_state infoid.
 */
typedef struct rs_guest_soft_state {
	uint8_t		soft_state;
	char		soft_state_str[SOFT_STATE_SIZE];
} rs_guest_soft_state_t;

/*
 * Resource state returned in data for the
 * HVctl_info_guest_tod infoid.
 */
typedef struct rs_guest_tod {
	uint64_t	tod;
} rs_guest_tod_t;

/*
 * Rsource state returned in data for the
 * HVctl_info_guest_utilisation infoid.
 * (this structure is full)
 */
typedef struct rs_guest_util {
	uint64_t	lifespan;
	uint64_t	wallclock_delta;
	uint64_t	active_delta;
	uint64_t	stopped_cycles;
	uint64_t	yielded_cycles;
} rs_guest_util_t;

/*
 * Info identifiers for vcpu resource.
 */
typedef enum {
	HVctl_info_vcpu_state,
	HVctl_info_vcpu_max
} hvctl_vcpu_info_t;

/*
 * Resource state returned in data for the
 * HVctl_info_vcpu_state infoid.
 */
typedef struct rs_vcpu_state {
	uint8_t		state;
	uint64_t	lifespan;
	uint64_t	wallclock_delta;
	uint64_t	active_delta;
	uint64_t	yielded_cycles;
} rs_vcpu_state_t;

/*
 * Info identifiers for memory resource.
 */
typedef enum {
	HVctl_info_mem_util,
} hvctl_mem_info_t;

/*
 * Resource state returned in data for the
 * HVctl_info_mem_util infoid.
 *
 */
typedef struct rs_mem_state {
	uint64_t	perf_cntr;	/* value of performance counter */
	uint64_t	sample_time;	/* time (nsec) when counter was read */
} rs_mem_state_t;

/*
 * Resource state returned in data for the
 * HVctl_info_guest_api_groups infoid.
 */
typedef struct rs_guest_api_groups {
	uint64_t	group_table;	/* PA of the group table */
	uint32_t	group_size;	/* Number of API groups */
} rs_guest_api_groups_t;

typedef struct hv_api_group_entry {
	uint32_t	major_ver;
	uint32_t	minor_ver;
} hv_api_group_entry_t;

/*
 * Individual entries in 'group_table' field
 */
typedef struct hv_api_group_version {
	uint64_t		api_group;
	hv_api_group_entry_t	version;
} hv_api_group_version_t;

typedef rs_guest_api_groups_t	hvctl_group_query_t;

/*
 * Methods used by the guest to synchronize tick.
 */
#define	GUEST_TICK_SYNC_LOCAL	0x1	/* using its own local tick */
#define	GUEST_TICK_SYNC_GLOBAL	0x2	/* using the machine global tick */

/*
 * Resource state returned in data for the
 * HVctl_info_guest_tick_sync infoid.
 */
typedef struct rs_guest_tick_sync {
	uint32_t	tick_sync_mode;
} rs_guest_tick_sync_t;

/*
 * Power Management operations
 */
typedef struct hvctl_pm_version {
	uint16_t pm_version_major;	/* PRI pm-version major number */
	uint16_t pm_version_minor;	/* PRI pm-version minor number */
} hvctl_pm_version_t;

typedef struct hvctl_pm_cookie_info {
	char	pm_cookie[HVCTL_PM_COOKIE_MAXLEN]; /* pm_cookie string */
	uint16_t	pm_res_state;	/* resource power state */
	hvctl_status_t	pm_res_status;	/* return resource status */
} hvctl_pm_cookie_info_t;

typedef enum {
	HVctl_pm_state_get	= 0,	/* get cuurent pm_res_state */
	HVctl_pm_state_set		/* set new pm_res_state */
} hvctl_pm_state_dir_t;

typedef struct hvctl_pm_state {
	hvctl_pm_state_dir_t	pm_state_dir;	/* direction of pm_state op */
	uint64_t	pm_cookie_addr;	/* paddr of the first pm_cookie */
	uint16_t	pm_num_cookies;	/* number of pm_cookies */
} hvctl_pm_state_t;

typedef struct hvctl_pm_async_resp {
	uint16_t	pm_aresp_seqnum;	/* original seq num */
	hvctl_status_t	pm_aresp_status;	/* status of operation */
} hvctl_pm_async_resp_t;

typedef enum {
	HVctl_pm_op_version	= 0,	/* get pm_version op */
	HVctl_pm_op_state,		/* get or set pm_res_state */
	HVctl_pm_op_async_resp,		/* async response op */
} hvctl_pm_op_t;

typedef struct hvctl_pm {
	hvctl_pm_op_t	pm_op;
	union {
		hvctl_pm_version_t	pm_version;	/* get pm_version */
		hvctl_pm_state_t	pm_state;	/* get/set pm_state */
		hvctl_pm_async_resp_t	pm_async_resp;	/* send async resp */
	} pm_op_data;
} hvctl_pm_t;

/*
 * Standard message header
 *
 * Applies to both requests and responses.
 *
 * The op field corresponds to an operation code or a reply code.
 * The sequence number is specific to the sender and is merely
 * used to detect dropped message packets.
 *
 * The status response accompanies a op field when in response to
 * a request to do something.
 */

/*
 * Basic header
 *
 * In requests to the HV the status field is ignored, in responses to
 * requests from Zeus, status corresponds to the relevent error code for
 * the request (op) field.
 *
 * For all messages (except the initial hello messages) the sequence number
 * is tracked to determine dropped requests.
 *
 * In the event of a dropped request, all future messages are dumped until
 * Zeus re-negotiates the HV control channel connection.
 */

typedef struct hvctl_header {
	uint16_t	op;
	uint16_t	seqn;
	uint16_t	chksum;
	uint16_t	status;	/* = 0 for commands, status for responses */
} hvctl_header_t;


typedef struct hvctl_msg {
	hvctl_header_t	hdr;
	union {
		hvctl_hello_t		hello;
		hvctl_challenge_t	clnge;
		hvctl_hvconfig_t	hvcnf;
		hvctl_reconfig_t	reconfig;
		hvctl_rc_fail_t		rcfail;
		hvctl_guest_op_t	guestop;
		hvctl_res_status_t	resstat;
		hvctl_pm_t		pm;
		hvctl_state_save_op_t	save;
		hvctl_group_query_t	api_query;
		hvctl_scrub_mem_t	scrub_mem;
		hvctl_mem_dr_verify_t	mem_dr_verify;
	} msg;
} hvctl_msg_t;

#endif	/* !defined(_ASM) */

#ifdef __cplusplus
}
#endif

#endif	/* _HVCTL_H */
