/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2008 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#ifndef	_LDCS_H
#define	_LDCS_H

#pragma ident	"@(#)ldcs.h	1.3	08/11/04 SMI"

#ifdef __cplusplus
extern "C" {
#endif

#include "ldmd_types.h"
#include "chain.h"
#include "mdesc_impl.h"
#include "resource_id.h"
#include "response.h"

/*
 * Values for target_type property - spcecifies connection point of other
 * end of channel.
 */
enum {
	TYPE_GUEST = 0ull,
	TYPE_HV,
	TYPE_SP
};

#define	LDCDB_HVGID		(-1)
#define	LDCDB_SPGID		(-2)
#define	LDCDB_IS_GUESTGID(_g)	((_g != LDCDB_HVGID) && (_g != LDCDB_SPGID))

/* HV LDC endpoint service types */
#define	LDC_HVCTL_SVC	1
#define	LDC_CONSOLE_SVC	2

/*
 * A LDC channel.
 */
struct ldc_chan {
	resid_t *chan;			/* channel number within guest */
	resid_t *id;			/* HV resource ID (globally unique) */
	CHAIN_LINK(struct ldc_chan, link);
};

struct ldc_list {
	CHAIN(struct ldc_chan, ldcs);
};

/*
 * Holds SP->guest LDC endpoints read out of HV MD.
 * See tgt_niagara1.c for usage.
 */
struct sp_guest_ldc {
	uint64_t	target_type;
	ldc_chan_t	*channel;
	uint64_t	target_guest;
	uint64_t	target_channel;
	CHAIN_LINK(sp_guest_ldc_t, link);
};

/*
 * Holds guest->SP LDC endpoints read out of HV MD.
 * See tgt_niagara1.c for usage.
 */
struct guest_sp_ldc {
	ldc_chan_t	*channel;
	uint64_t	target_channel;
	CHAIN_LINK(guest_sp_ldc_t, link);
};

/*
 * For processing the ldc endpoints from HV.
 */
struct ldc_epdb {
	uint64_t	gid;
	uint64_t	ldc;
	resid_t		*id;
	ldom_t		*ldp;
	struct ldc_epdb	*peer;		/* NULL if hypervisor channel */
	int		count;		/* for debug */
	CHAIN_LINK(struct ldc_epdb, link);
};


int ldcs_chan_alloc(ldcs_t *ldcs, res_pool_t *ldc_pool);
ldc_chan_t *ldcs_search(ldcs_t *ldcs, uint64_t ldc_id);
void get_ldc_ids(md_t *md, md_node_t *node, int *num_ldc_ids,
    uint64_t **ldc_ids);
sp_guest_ldc_t *sp_guest_ldc_find_by_id(uint64_t ldcid);
sp_guest_ldc_t *sp_guest_ldc_find_by_target(uint64_t target_guest,
    uint64_t target_channel);
void ldc_pull_ids(ldom_t *ldp, res_pool_t *ldc_pool, int num_ldc_ids,
    uint64_t *ldc_ids, ldcs_t *ldcs);
void ldc_get_and_pull_ids(ldom_t *ldp, res_pool_t *ldc_pool,
    md_t *md, md_node_t *node, ldcs_t *ldcs);
int ldc_mig_get_and_pull_ids(ldom_t *ldp, md_t *hvmdp, res_pool_t *ldc_pool,
    md_t *md, md_node_t *node, ldcs_t *ldcs);
int ldc_mig_pull_ids(ldom_t *ldp, md_t *hvmdp, res_pool_t *ldc_pool,
    int num_ldc_ids, uint64_t *ldc_ids, ldcs_t *ldcs);
int ldc_mig_get_and_pull_peer_ids(ldom_t *ldp, md_t *hvmdp, ldcs_t *src_ldcs,
    ldcs_t *peer_ldcs, res_pool_t *peer_pool);

void ldcs_free(ldcs_t *ldcs);
void ldcs_stage(ldcs_t *ldcs);
void ldcs_print(const ldcs_t *ldcs, response_t *resp);

#ifdef __cplusplus
}
#endif

#endif	/* _LDCS_H */
