/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2010 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#ifndef	_LDM_PROTOCOL_H
#define	_LDM_PROTOCOL_H

#ifdef __cplusplus
extern "C" {
#endif

#include <inttypes.h>

/*
 * Protocol version supported.
 */
#define	LDM_MAJOR_VER	1
#define	LDM_MINOR_VER	0

/* per class version numbers */
#define	LDM_CLI_MAJOR_VER	2
#define	LDM_CLI_MINOR_VER	0

#define	LDM_CONTROL_MAJOR_VER	1
#define	LDM_CONTROL_MINOR_VER	0

#define	LDM_SUNMC_MAJOR_VER	1
#define	LDM_SUNMC_MINOR_VER	0

#define	LDM_MIG_MAJOR_VER	1
#define	LDM_MIG_MINOR_VER	2

enum {
	LDM_INIT_REQ	= 0x0,
	LDM_INIT_ACK	= 0x1,
	LDM_INIT_NACK	= 0x2,
	LDM_ERROR	= 0x3,
	LDM_OP_REQ	= 0x4,
	LDM_OP_REPLY	= 0x5,
	LDM_MIG_MSG	= 0x6,		/* domain migration messages */
};

typedef struct {
	uint32_t	msg_type;
	uint32_t	payload_len;
} ldm_hdr_t;

enum {
	LDM_CLASS_CLI		= 0x1,
	LDM_CLASS_CONTROL	= 0x2,
	LDM_CLASS_SUNMC		= 0x3,
	LDM_CLASS_MIG		= 0x4,
};

typedef struct {
	uint32_t	client_class;
	uint16_t	major_vers;
	uint16_t	minor_vers;
} ldm_init_req_t;

typedef struct {
	uint16_t	minor_vers;
} ldm_init_ack_t;

typedef struct {
	uint16_t	major_vers;
} ldm_init_nack_t;

enum {
	LDM_ERROR_UNKNOWN	= 0x1,
	LDM_ERROR_UNEXPECTED	= 0x2,
	LDM_ERROR_INVALID	= 0x3
};

typedef struct {
	uint32_t	error_code;
} ldm_error_t;

typedef struct {
	uint32_t	rq_len;
	uint32_t	rq_id;
	uint8_t		rq_data[];
} ldm_op_req_t;

typedef struct {
	uint32_t	rp_len;
	uint32_t	rp_reqid;
	uint8_t		rp_data[];
} ldm_op_reply_t;

#ifdef __cplusplus
}
#endif

#endif	/* _LDM_PROTOCOL_H */
