/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2009 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#include <stdio.h>
#include <strings.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <unistd.h>

#include "basics.h"
#include "ldom.h"
#include "mac_multicast.h"
#include "ldmd_discovery.h"

#define	LDMD_DISC_VER	1
#define	MANAGER_NAME	"LDMD"

/*ARGSUSED0*/
void
handle_ldmd_discovery(ldmd_discovery_t *multicast_msg, int sock_out,
    struct sockaddr_in sock_w)
{
	multicast_msg_t		resp_msg;
	int			sock_w_len = sizeof (sock_w);
	int			error;
	struct hostent		*hp;

	resp_msg.version_no = LDMD_DISC_VER;
	resp_msg.magic_no = MAC_MULTI_MAGIC_NO;
	resp_msg.msg_type = LDMD_DISC_RESP;
#if defined(lint)
	resp_msg.payload.Ldmd_discovery = 0;
#endif

	(void) snprintf(resp_msg.discovery.zeus_version, ZEUS_VERSION_LEN,
	    "%s:%s", MANAGER_NAME, LDM_VERSION);
	resp_msg.discovery.port_no = XMPP_DEFAULT_PORT;
	(void) gethostname((char *)&resp_msg.discovery.hostname,
	    MAXHOSTNAMELEN);
	hp = getipnodebyname(resp_msg.discovery.hostname, AF_INET, AI_DEFAULT,
	    &error);
	if (hp != NULL) {
		(void) memcpy(&resp_msg.discovery.ip_address.s_addr,
		    hp->h_addr, sizeof (resp_msg.discovery.ip_address.s_addr));
	}

	(void) sendto(sock_out, &resp_msg, sizeof (resp_msg), 0,
	    (struct sockaddr *)&sock_w, sock_w_len);
}
