/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2009 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#ifndef	_CLI_UTIL_H_
#define	_CLI_UTIL_H_

#ifdef __cplusplus
extern "C" {
#endif

#include <stdint.h>

#include "basics.h"
#include "cli_conn.h"
#include "dispctl.h"
#include "ldmd_types.h"
#include "ldom.h"
#include "response.h"


#define	LDM_SEPARATOR	\
"------------------------------------------------------------------------------"


/* config_util.c */
extern void config_mode_change_warning(response_t *resp);
extern void config_mode_info_warning(response_t *resp, dispctl_t *dctl);

/* parse_util.c */
extern bool_t is_valid_variable(response_t *resp, char *name);
extern bool_t is_valid_device_name(response_t *resp, char *namep);
extern bool_t is_valid_object_name(response_t *resp, char *namep);

extern ldom_t *ldm_get_ldp(response_t *resp, char *name);

extern bool_t parse_num(response_t *resp, uint64_t *num, char *value);
extern bool_t add_res(uint64_t curr, uint64_t *amt);
extern int get_comma_sep_list_len(char *slist);
extern int get_comma_sep_list(char *slist, int *list, int n);

/* print_util.c */
extern void output_version(response_t *resp, dispctl_t *dctl);

extern void print_ldom_variables(ldom_t *ldp, response_t *resp,
    dispctl_t *dctl);
extern bool_t print_ldom_variable_list(ldom_t *ldp, response_t *resp,
    int nvars, char **listvars);
extern void printldom(ldom_t *ldp, response_t *resp, dispctl_t *dctl);
extern void print_bsc(ldom_t *ldp, response_t *resp, dispctl_t *dctl);

/* debug.c */
#if ZEUS_DEBUG_CLI
extern bool_t ldm_debug(int argc, void **argv, cli_conn_t *client);
#endif


#ifdef __cplusplus
}
#endif

#endif /* _CLI_UTIL_H_ */
