/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2009 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <strings.h>

#include "dispctl.h"

/*
 * Initialize a display control descriptor.
 */
void
dctl_init(dispctl_t *dctl)
{
	dctl->all = 0;
	dctl->bindings = 0;
	dctl->constraints = 0;
	dctl->services = 0;
	dctl->parseable = 0;
	dctl->longf = 0;
	dctl->extended = 0;
	dctl->oformat = 0;
	dctl->oconsole = 0;
	dctl->ocpu = 0;
	dctl->ocrypto = 0;
	dctl->odisk = 0;
	dctl->odomain = 0;
	dctl->omemory = 0;
	dctl->onetwork = 0;
	dctl->ophysio = 0;
	dctl->oresmgmt = 0;
	dctl->oserial = 0;
	dctl->ostatus = 0;
	dctl->list = 0;
	dctl->first = 1;
	dctl->server = 1;
	dctl->clients = 1;
	dctl->header = 0;
	dctl->physfirst = 1;
}

/*
 * Format an output section label. Labels are all upper-case.
 * This routine uses static storage hence it is not reentrant.
 */
/*ARGSUSED*/
char *
dctl_label(char *label, dispctl_t *dctl)
{
	static char dlabel[32];
	int i;

	snprintf(dlabel, sizeof (dlabel), "%s", label);
	for (i = strlen(dlabel) - 1; i >= 0; i--)
		dlabel[i] = toupper(dlabel[i]);

	return (dlabel);
}
