/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2010 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#ifndef	_LDOMCONFIGURE_H_
#define	_LDOMCONFIGURE_H_

#ifdef __cplusplus
extern "C" {
#endif

#include "cli_conn.h"
#include "clp.h"
#include "mac_addrs.h"
#include "response.h"

enum {
	ACT_ADD,
	ACT_REMOVE,
	ACT_SET
};

typedef enum {
	CLI_RESULT_OK = 0,
	CLI_RESULT_FAIL,
	CLI_RESULT_USAGE
} cli_result_t;

cli_result_t cli_cancel_reconf(int, char **, cli_conn_t *);
cli_result_t cli_cancel_operation(int argc, char **argv, cli_conn_t *client);

cli_result_t cli_ldom_start(int, char **, cli_conn_t *);
cli_result_t cli_ldom_stop(int, char **, cli_conn_t *);
cli_result_t cli_ldom_bind(int, char **, cli_conn_t *);
cli_result_t cli_ldom_unbind(int, char **, cli_conn_t *);
cli_result_t cli_ldom_create(int argc, char **argvp, cli_conn_t *clientp);
cli_result_t cli_ldom_set(int argc, char **argvp, cli_conn_t *clientp);
cli_result_t cli_ldom_destroy(int argc, char **argvp, cli_conn_t *clientp);
cli_result_t cli_ldom_panic(int argc, char **argvp, cli_conn_t *clientp);
cli_result_t cli_ldom_migrate(int argc, char **argvp, cli_conn_t *clientp);
cli_result_t cli_ldom_list(int argc, char **argvp, cli_conn_t *clientp);

cli_result_t cli_vds_add(int argc, char **argvp, cli_conn_t *clientp);
cli_result_t cli_vds_remove(int argc, char **argvp, cli_conn_t *client);

cli_result_t cli_vdisk_add(int argc, char **argvp, cli_conn_t *clientp);
cli_result_t cli_vdisk_set(int argc, char **argvp, cli_conn_t *clientp);
cli_result_t cli_vdisk_remove(int argc, char **argvp, cli_conn_t *clientp);

cli_result_t cli_vdsdev_add(int argc, char **argvp, cli_conn_t *clientp);
cli_result_t cli_vdsdev_set(int argc, char **argvp, cli_conn_t *clientp);
cli_result_t cli_vdsdev_remove(int argc, char **argvp, cli_conn_t *client);

cli_result_t cli_vswitch_add(int argc, char **argvp, cli_conn_t *clientp);
cli_result_t cli_vswitch_remove(int argc, char **argvp, cli_conn_t *clientp);
cli_result_t cli_vswitch_set(int argc, char **argvp, cli_conn_t *clientp);

cli_result_t cli_vnet_add(int argc, char **argvp, cli_conn_t *clientp);
cli_result_t cli_vnet_remove(int argc, char **argvp, cli_conn_t *clientp);
cli_result_t cli_vnet_set(int argc, char **argvp, cli_conn_t *clientp);

cli_result_t cli_vcons_set(int argc, char **argvp, cli_conn_t *clientp);

cli_result_t cli_vcc_add(int argc, char **argvp, cli_conn_t *clientp);
cli_result_t cli_vcc_remove(int argc, char **argvp, cli_conn_t *clientp);
cli_result_t cli_vcc_set(int argc, char **argvp, cli_conn_t *clientp);

cli_result_t cli_var_remove(int argc, char **argvp, cli_conn_t *clientp);
cli_result_t cli_var_set(int argc, char **argvp, cli_conn_t *clientp);
cli_result_t cli_var_list(int argc, char **argvp, cli_conn_t *clientp);

cli_result_t cli_option_remove(int argc, char **argvp, cli_conn_t *clientp);
cli_result_t cli_option_set(int argc, char **argvp, cli_conn_t *clientp);
cli_result_t cli_option_list(int argc, char **argvp, cli_conn_t *clientp);

cli_result_t cli_vdpcs_add(int argc, char **argvp, cli_conn_t *clientp);
cli_result_t cli_vdpcs_remove(int argc, char **argvp, cli_conn_t *client);

cli_result_t cli_vdpcc_add(int argc, char **argvp, cli_conn_t *clientp);
cli_result_t cli_vdpcc_remove(int argc, char **argvp, cli_conn_t *clientp);

cli_result_t cli_vcpu_add(int argc, char **argvp, cli_conn_t *clientp);
cli_result_t cli_vcpu_remove(int argc, char **argvp, cli_conn_t *clientp);
cli_result_t cli_vcpu_set(int argc, char **argvp, cli_conn_t *clientp);

cli_result_t cli_mem_add(int argc, char **argvp, cli_conn_t *clientp);
cli_result_t cli_mem_remove(int argc, char **argvp, cli_conn_t *clientp);
cli_result_t cli_mem_set(int argc, char **argvp, cli_conn_t *clientp);

cli_result_t cli_mau_add(int argc, char **argvp, cli_conn_t *clientp);
cli_result_t cli_mau_remove(int argc, char **argvp, cli_conn_t *clientp);
cli_result_t cli_mau_set(int argc, char **argvp, cli_conn_t *clientp);

cli_result_t cli_io_add(int argc, char **argvp, cli_conn_t *clientp);
cli_result_t cli_io_remove(int argc, char **argvp, cli_conn_t *clientp);
cli_result_t cli_io_list(int argc, char **argvp, cli_conn_t *clientp);

cli_result_t cli_spconfig_add(int argc, char **argvp, cli_conn_t *clientp);
cli_result_t cli_spconfig_remove(int argc, char **argvp, cli_conn_t *clientp);
cli_result_t cli_spconfig_set(int argc, char **argvp, cli_conn_t *clientp);
cli_result_t cli_spconfig_list(int argc, char **argvp, cli_conn_t *clientp);

cli_result_t cli_bindings_list(int argc, char **argvp, cli_conn_t *clientp);
cli_result_t cli_constraints_list(int argc, char **argvp, cli_conn_t *clientp);
cli_result_t cli_devices_list(int argc, char **argvp, cli_conn_t *clientp);
cli_result_t cli_services_list(int argc, char **argvp, cli_conn_t *clientp);

cli_result_t cli_policy_add(int argc, char **argv, cli_conn_t *client);
cli_result_t cli_policy_set(int argc, char **argv, cli_conn_t *client);
cli_result_t cli_policy_remove(int argc, char **argv, cli_conn_t *client);
cli_result_t cli_policy_list(int argc, char **argv, cli_conn_t *client);

bool_t valid_name(response_t *resp, char *name);
ldom_t *ldom_create(char *name, mac_addr_t mac_address, uint64_t hostid,
    uint64_t failure_policy, mlist_t *mlist);
bool_t bind_boot_options(ldom_t *ldp, response_t *resp);
void undo_partial_binding(ldom_t *ldp);
bool_t check_locked_ldom(ldom_t *ldp, response_t *resp);
bool_t destroy_one_ldom(ldom_t *ldp, response_t *resp,
    cli_conn_t *client);
void cleanse_progress_chain(void);
char *ldom_flags(ldom_t *ldp);
bool_t parse_mem_size(response_t *resp, uint64_t *mem_size, char *value,
    char *unit);
void update_statistics(ldom_t *ldp, bool_t do_cpus);

#ifdef __cplusplus
}
#endif

#endif /* _LDOMCONFIGURE_H_ */
