/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2008 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#ifndef	_MAC_ADDRS_H_
#define	_MAC_ADDRS_H_

#pragma ident	"@(#)mac_addrs.h	1.12	08/10/01 SMI"

#ifdef __cplusplus
extern "C" {
#endif

#include <libxml/tree.h>
#include <libxml/parser.h>

#include "chain.h"
#include "resource_id.h"

#define	PUBLIC_MAC_START	0x00144ff80000
#define	PUBLIC_MAC_END		0x00144ffbffff

#define	PRIVATE_MAC_START	0x02144ff80000
#define	PRIVATE_MAC_END		0x02144fffffff

#define	IS_PUBLIC_MAC_ADDR(mac)	((mac >= PUBLIC_MAC_START) && \
	    (mac <= PUBLIC_MAC_END))

typedef	uint64_t	mac_addr_t;

#define	ILLEGAL_MAC_ADDR	((mac_addr_t)-1LL)

/* big enough for MAC representation as octets or as a single hex number */
#define	MAC_STR_LEN		32

#define	NO_OF_MACS		0x3ffff

#define	OBTAIN_MAC_TIMEOUT	30

#define	MAC_PUBLIC		"public"
#define	MAC_PRIVATE		"private"
#define	MAC_INDEX_RANGE		((xmlChar *)"mac_index_range")
#define	MIN_MAC_INDEX		((xmlChar *)"min_mac_index")
#define	MAX_MAC_INDEX		((xmlChar *)"max_mac_index")
#define	USED_MAC_LIST_NODE	((xmlChar *)"used_mac_list")

typedef	struct VNET_MAC		vnet_mac_t;
typedef struct VSW_MAC		vsw_mac_t;
typedef struct VSW_LIST		vsw_list_t;

typedef struct ADDR_OBJ		addr_obj_t;
typedef struct MAC_LIST		mac_list_t;

enum {
	ROUTED_MODE,
	NON_ROUTED_MODE,
	DB_ROUTED_MODE,
	DB_NON_ROUTED_MODE,
	UNKNOWN,
};

enum {
	ALLOCATING,
	NON_ALLOCATING,
};

struct VNET_MAC {
	int		mode;
	mac_addr_t	mac_addr;
	resid_t		*macaddr_res;
	char		*ldm_name;
	char		*service_name;
	char		*vnet_name;
	CHAIN_LINK(vnet_mac_t, link);
};

struct VSW_MAC {
	int		mode;
	mac_addr_t	mac_addr;
	char		*ldm_name;
	char		*service_name;
	CHAIN(vnet_mac_t, vnet_list);
	CHAIN_LINK(vsw_mac_t, link);
};

struct VSW_LIST {
	CHAIN(vsw_mac_t, vsw_list);
};

vsw_list_t	mac_list;
res_pool_t	private_mac_pool;
bool_t		mac_socket_up;

typedef struct USED_MAC		used_mac_t;
typedef struct USED_MAC_LIST	used_mac_list_t;

struct USED_MAC {
	mac_addr_t	mac_addr;
	CHAIN_LINK(used_mac_t, link);
};

struct USED_MAC_LIST {
	CHAIN(used_mac_t, mac_list);
};

used_mac_list_t		used_list;

/* mac_addrs.c */
void mac_init(void);
vsw_mac_t *get_vsw_node(char *service_name);
vnet_mac_t *get_vnet_node(char *ldm_name, char *vnet_name);
bool_t is_mac_allocated(mac_addr_t mac_address);
mac_addr_t get_usable_mac();
vnet_mac_t *private_macs_allocated(vsw_mac_t *vsw_node);
void print_mac_list(void);
void remove_ldom_macs(char *ldm_name);
void add_used_mac(mac_addr_t mac_addr);
void free_used_mac(used_mac_t *used_mac);
used_mac_t *is_mac_used(mac_addr_t mac_addr);

	/* buf has to be MAC_STR_LEN bytes long */
void mactoa(mac_addr_t mac_addr, char *bufp);
	/* returns false if unable to convert to mac addr */
bool_t atomac(char *bufp, mac_addr_t *mac_addrp);

/* mac_vsw.c */
int create_vsw_mac(vsw_mac_t **sent_vsw, char *ldm_name, char *service_name,
	char *mode, mac_addr_t mac_address);
int update_vsw_mac(vsw_mac_t *vsw_node, char *ldm_name, mac_addr_t mac_address,
	char *mode);
void remove_vsw_mac(char *service_name);
vsw_mac_t *create_basic_vsw_node(char *ldm_name, char *service_name, int mode);
void populate_vsw(xmlNodePtr ldom);
void delete_vsw(vsw_mac_t *vsw_node);
void blank_out_vsw(vsw_mac_t *vsw_node);

/* mac_vnet.c */
int create_vnet_node(vnet_mac_t **sent_vnet, vsw_mac_t **vsw, char *ldm_name,
	char *service_name, char *vnet_name, char *mode, mac_addr_t mac_addr);
void remove_vnet_mac(char *ldm_name, char *vnet_name);
int update_vnet_node(vnet_mac_t **sent_vnet, char *new_service,
	mac_addr_t mac_addr, char *mode);
vnet_mac_t *create_vnet(vsw_mac_t *vsw_node, char *ldm_name, char *vnet_name,
	int mode);
void delete_vnet(vsw_mac_t *vsw_node, vnet_mac_t *vnet_node);
void remove_empty_vsw(vsw_mac_t *vsw_node);
void populate_vnet(xmlNodePtr ldom);

#ifdef __cplusplus
}
#endif

#endif /* _MAC_ADDRS_H_ */
