/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */


/*
 * Copyright 2010 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#ifndef	_MAC_MULTICAST_H_
#define	_MAC_MULTICAST_H_

#ifdef __cplusplus
extern "C" {
#endif

#include "response.h"
#include "mac_addrs.h"
#include "ldmd_discovery.h"

#define	MAC_MULTI_PORT		64535
#define	MAC_MULTI_GROUP		"239.129.9.27"
#define	MAC_MULTI_MAGIC_NO	92792004
#define	MAC_MULTI_TIMEOUT	1	/* Time to wait for response */
#define	MAC_MULTI_TTL		1	/* Time to live for socket creation */
#define	MAC_MULTI_VERSION	1

#define	SKIP_LOCAL_MAC_CHECK	false   /* skip local MAC collision check */
#define	PERFORM_LOCAL_MAC_CHECK	true

enum {
	SEND_MSG = 0,
	RESPONSE_MSG,
	LDMD_DISC_SEND,
	LDMD_DISC_RESP,
};

typedef struct {
	uint64_t	mac_addr;
	char		source_ip[INET_ADDRSTRLEN];
	bool_t		local_mac_collision_check;
} mac_lookup_t;

typedef struct {
	uint32_t	version_no;
	uint32_t	magic_no;
	uint32_t	msg_type;
	uint32_t	resv;
	union {
		mac_lookup_t		Mac_lookup;
		ldmd_discovery_t	Ldmd_discovery;
	} payload;
#define	lookup		payload.Mac_lookup
#define	discovery	payload.Ldmd_discovery
} multicast_msg_t;

/*
 * check the condition that the local mac collision detection is performed
 * and a given mac address is found to be already allocated on the local
 * system. The caller function needs to have a local variable called
 * 'local_mac_collision_check'.
 */
#define	LOCAL_MAC_ALLOCATED(mac_addr)\
	(local_mac_collision_check && is_macaddr_allocated_on_system(mac_addr))

bool_t is_macaddr_free(uint64_t mac_addr, bool_t local_mac_collision_check);
bool_t macaddr_sock_create(void);
void *macaddr_loop(void *unused);
bool_t start_mac_auto_allocate(void);
bool_t is_macaddr_in_use(mac_addr_t mac_addr, bool_t local_mac_collision_check,
    response_t *resp);

#ifdef __cplusplus
}
#endif

#endif /* _MAC_MULTICAST_H_ */
