/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2009 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#ifndef	_MASTER_H_
#define	_MASTER_H_

#ifdef __cplusplus
extern "C" {
#endif


/*
 * Supported failure policies.
 */
#define	INVALID_FAILURE_POLICY	0xFFFFFFFFFFFFFFFF
#define	FAILURE_POLICY_IGNORE	0			/* default policy */
#define	FAILURE_POLICY_STOP	1
#define	FAILURE_POLICY_RESET	2
#define	FAILURE_POLICY_PANIC	3

#define	VALID_FAILURE_POLICY(_p)	(((_p) >= FAILURE_POLICY_IGNORE) && \
					((_p) <= FAILURE_POLICY_PANIC))

/*
 * Dependency macros.
 */
#define	NUM_DEPENDENCIES(_ldp)		(CHAIN_LENGTH((_ldp)->props.masters))
#define	DEPENDENCY_LIMIT_REACHED(_ldp)	(NUM_DEPENDENCIES((_ldp)) >= \
					pri.max_guest_dependencies)
#define	DEPENDENCY_LIMIT_OVERFLOW(_ldp)	(NUM_DEPENDENCIES((_ldp)) > \
					pri.max_guest_dependencies)
#define	LDOM_DEPENDENCIES_SUPPORTED()	(pri.max_guest_dependencies > 0)

typedef struct master {
	ldom_t		*ldp;			/* master domain */
	CHAIN_LINK(struct master, link);	/* next master */
} master_t;

typedef struct mlist {
	char		*namep;
	ldom_t		*master;
	struct mlist	*next;
} mlist_t;

bool_t get_failure_policy(char *str, uint64_t *policy);
char *get_failure_policy_string(uint64_t policy);
char *get_masters_string(ldom_t *ldp);
bool_t is_valid_master(master_t *mp);
bool_t has_bound_slaves(ldom_t *ldp, response_t *resp);
void invalidate_slaves(ldom_t *master);
void print_failure_policy(ldom_t *ldp, response_t *resp, dispctl_t *dctl);
void print_master(ldom_t *ldp, response_t *resp, dispctl_t *dctl);
bool_t bind_dependency(ldom_t *ldp, response_t *resp);
bool_t dependency_cycle(ldom_t *ldp, mlist_t *mlist, response_t *resp);
bool_t validate_masters(ldom_t *ldp);
bool_t unique_masters(mlist_t *mlist, response_t *resp);
void mlist2master(ldom_t *ldp, mlist_t *mlist);
void free_mlist(mlist_t *mlist);
void free_masters(ldom_t *ldp);


#ifdef __cplusplus
}
#endif

#endif /* _MASTER_H_ */
