/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2008 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#ifndef	_MDESC_H_
#define	_MDESC_H_

#pragma ident	"@(#)mdesc.h	1.4	08/02/15 SMI"

#ifdef __cplusplus
extern "C" {
#endif

#include <sys/byteorder.h>

/*
 * Each logical domain is detailed via a (Virtual) Machine Description
 * available to each guest Operating System courtesy of a
 * Hypervisor service.
 *
 * A complete Machine Description (MD) is built from a very
 * simple list of machine descriptor elements (MDE)s.
 */

/* the version this library understands */

#define	MD_HEADER_VERS_OFF	0x0
#define	MD_HEADER_NODE_OFF	0x4
#define	MD_HEADER_NAME_OFF	0x8
#define	MD_HEADER_DATA_OFF	0xc

#define	MD_HEADER_SIZE	0x10

#define	MD_TRANSPORT_VERSION	(uint32_t)(0x10000)

#define	MD_ELEMENT_SIZE		0x10
#define	MD_ALIGNMENT_SIZE	0x10

#define	MDE_ILLEGAL_IDX		(uint64_t)(-1)

#define	MDET_LIST_END	(uint8_t)(0x0)
#define	MDET_NULL	(uint8_t)(' ')
#define	MDET_NODE	(uint8_t)('N')
#define	MDET_NODE_END	(uint8_t)('E')
#define	MDET_PROP_ARC	(uint8_t)('a')
#define	MDET_PROP_VAL	(uint8_t)('v')
#define	MDET_PROP_STR	(uint8_t)('s')
#define	MDET_PROP_DAT	(uint8_t)('d')

#ifndef _ASM	/* { */

/*
 * Each MD has the following header to
 * provide information about each section of the MD.
 *
 * There are 3 sections:
 * The description list, the name table and the data block.
 *
 * All values are stored in network byte order.
 *
 * Elements in the first (description list) section are defined by their
 * index location within the node block. An index is simply the byte offset
 * within the block / element size (16bytes). All elements are refered to
 * by their index, to avoid bugs related to alignment etc.
 *
 * The name_len field holds the storage length of an ASCII name, NOT the strlen.
 * The header fields are written in network
 * byte order.
 */

struct MD_HEADER {
	uint32_t	transport_version;
	uint32_t	node_blk_sz;	/* size in bytes of the node block */
	uint32_t	name_blk_sz;	/* size in bytes of the name block */
	uint32_t	data_blk_sz;	/* size in bytes of the data block */
};

typedef struct MD_HEADER md_header_t;

#if defined(_BIG_ENDIAN)
#define	mdtoh8(x)	((uint8_t)(x))
#define	mdtoh16(x)	((uint16_t)(x))
#define	mdtoh32(x)	((uint32_t)(x))
#define	mdtoh64(x)	((uint64_t)(x))
#define	htomd8(x)	(x)
#define	htomd16(x)	(x)
#define	htomd32(x)	(x)
#define	htomd64(x)	(x)
#else
#define	mdtoh8(x)	((uint8_t)(x))
#define	mdtoh16(x)	ntohs(x)
#define	mdtoh32(x)	ntohl(x)
#define	mdtoh64(x)	BSWAP_64((uint64_t)(x))
#define	htomd8(x)	((uint8_t)(x))
#define	htomd16(x)	htons(x)
#define	htomd32(x)	htonl(x)
#define	htomd64(x)	BSWAP_64((uint64_t)(x))
#endif

struct MD_ELEMENT {
	uint8_t		tag;
	uint8_t		name_len;
	uint16_t	_reserved;
	uint32_t	name_offset;	/* mde_str_cookie_t */
	union {
		struct {
			uint32_t	len;
			uint32_t	offset;
		} prop_data;			/* for PROP_DATA and PROP_STR */
		uint64_t	prop_val;	/* for PROP_VAL */
		uint64_t	prop_idx;	/* for PROP_ARC and NODE */
	} d;
};

typedef struct MD_ELEMENT md_element_t;

#endif	/* } _ASM */

#ifdef __cplusplus
}
#endif

#endif	/* _MDESC_H_ */
