/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2009 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#ifndef	_MDESC_IMPL_H_
#define	_MDESC_IMPL_H_

#pragma ident	"@(#)mdesc_impl.h	1.17	09/05/20 SMI"

#ifdef __cplusplus
extern "C" {
#endif

#define	LIBMD_MAGIC	0x4d61636844657363	/* MachDesc */

#include <synch.h>
#ifndef _ASM

#include "basics.h"
#include "chain.h"
#include "mdesc.h"

/*
 * Internal definitions.
 */

typedef struct MD_STRING md_string_t;
typedef struct MD_DATA_BLOCK md_data_block_t;
typedef struct MD_PROP md_prop_t;
typedef struct MD_NODE md_node_t;
typedef struct MD md_t;

struct MD_STRING {
	CHAIN_LINK(md_string_t, link);
	char		*strp;
	int		size;	/* strlen()+1 */
	uint32_t	hash;
	int		ref_cnt;
	uint32_t	build_offset;
};

struct MD_DATA_BLOCK {
	CHAIN_LINK(md_data_block_t, link);
	uint8_t		*datap;
	uint32_t	size;
	uint32_t	hash;
	int		ref_cnt;
	uint32_t	build_offset;
};

struct MD_PROP {
	uint8_t		type;
	md_string_t	*sp;
	union {
		uint64_t	value;
		struct {
			bool_t	is_ptr;
			union {
				uint64_t	index;
				md_node_t	*nodep;
			} val;
		} arc;
		md_data_block_t *dbp;
	} d;
	CHAIN_LINK(md_prop_t, link);
};

struct MD_NODE {
	md_string_t	*typep;
	CHAIN(md_prop_t, prop_list);
	CHAIN_LINK(md_node_t, link);
	CHAIN_LINK(md_node_t, ref);	/* referenced nodes (md_scour) */
	int		build_index;	/* for building a binary md & cloning */
	int		next_index;	/* for building a binary md */
	char		seen;		/* seen flag (md_scan_dag/md_scour) */
	char		deleted;	/* pending deletion flag */
};

struct MD {
	mutex_t md_lock;		/* to serialize cloning & gen. bin */
	CHAIN(md_node_t, node_list);
	CHAIN(md_string_t, string_list);
	CHAIN(md_data_block_t, data_block_list);
};

#define	MDE_TAG(_p)			mdtoh8((_p)->tag)
#define	MDE_NAME(_p)			mdtoh32((_p)->name_offset)
#define	MDE_NAME_LEN(_p)		mdtoh32((_p)->name_len)
#define	MDE_PROP_DATA_OFFSET(_p)	mdtoh32((_p)->d.prop_data.offset)
#define	MDE_PROP_DATA_LEN(_p)		mdtoh32((_p)->d.prop_data.len)
#define	MDE_PROP_VALUE(_p)		mdtoh64((_p)->d.prop_val)
#define	MDE_PROP_INDEX(_p)		mdtoh64((_p)->d.prop_idx)

md_prop_t *md_find_node_prop(md_t *, md_node_t *, md_string_t *, int);

md_node_t *md_find_node_index(md_t *mdp, int index);

md_string_t *md_find_string(md_t *mdp, char *strp, uint32_t *hashp);

md_node_t *md_new_node(md_t *mdp, char *sp);

md_prop_t *md_new_property(md_t *mdp, md_node_t *nodep, uint8_t type, char *sp);

void md_add_value_property(md_t *mdp, md_node_t *nodep, char *sp,
    uint64_t value);

void md_add_string_property(md_t *mdp, md_node_t *nodep, char *sp, char *bufp);

void md_add_data_property(md_t *mdp, md_node_t *nodep, char *sp, int len,
    uint8_t *bufp);

void md_add_string_as_data_property(md_t *mdp, md_node_t *nodep, char *sp,
    char *bufp);

void md_add_arc_property(md_t *mdp, md_node_t *nodep, char *arcnamep,
    md_node_t *tgtnodep);

void md_update_value_property(md_t *md, md_node_t *nodep, char *prop_namep,
    uint64_t val);

void md_update_string_property(md_t *mdp, md_node_t *nodep, char *prop_namep,
    char *bufp);

void md_update_data_property(md_t *mdp, md_node_t *nodep, char *prop_namep,
    int len, uint8_t *bufp);

md_data_block_t *md_new_data_block(md_t *mdp, uint8_t *bufp, int len);

void md_free_node(md_t *mdp, md_node_t *nodep);

void md_free_prop(md_t *mdp, md_prop_t *propp);

void md_free_string(md_t *mdp, md_string_t *msp);

void md_free_data_block(md_t *mdp, md_data_block_t *mdbp);

md_t *md_scan_bin(uint8_t *bufp, int size);

int md_scan_bin_size(md_header_t *mdhp);

int md_gen_bin(md_t *mdp, uint8_t **bufpp);

md_node_t *md_link_new_node(md_t *mdp, char *nodenamep, md_node_t *parentnodep,
    char *linktonewp, char *linkbackp);

void md_delete_node(md_t *mdp, md_node_t *nodep);

void md_mark_node_deleted(md_node_t *nodep);

void md_delete_prop(md_t *mdp, md_node_t *nodep, md_prop_t *propp);

md_node_t *md_find_node(md_t *md, char *node_name);
md_node_t *md_find_subnode(md_t *md, md_node_t *node, char *node_name);

bool_t md_find_delete_node(md_t *md, char *node_name);

void md_scour(md_t *mdp, md_string_t *exclude_arcp);

md_t *md_clone(md_t *mdp);

void md_add(md_t *mda, md_t *mdb);

md_t *md_new_md(void);

void md_copy_properties(md_t *md, md_node_t *to, md_node_t *from);

md_t *md_init(void *);

int md_fini(md_t *);

int md_node_count(md_t *);

md_string_t *md_find_name(md_t *, char *namep);

md_node_t *md_root_node(md_t *);

int md_scan_nodes(md_t *, md_string_t *, md_node_t **);

int md_scan_dag(md_t *, md_node_t *, md_string_t *, md_string_t *,
    md_node_t **);

int md_get_prop_val(md_t *, md_node_t *, char *, uint64_t *);

int md_get_prop_str(md_t *, md_node_t *, char *, char **);

int md_get_prop_data(md_t *, md_node_t *, char *, uint8_t **, int *);

int md_get_prop_arc(md_t *, md_node_t *, char *, md_node_t **);

#endif	/* _ASM */

#ifdef __cplusplus
}
#endif

#endif	/* _MDESC_IMPL_H_ */
