/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2007 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#ifndef _MDSTORE_H
#define	_MDSTORE_H

#pragma ident	"@(#)mdstore.h	1.4	07/02/09 SMI"

/*
 * Machine Description Store Protocol
 */

#ifdef __cplusplus
extern "C" {
#endif

#include <sys/types.h>

#include "ldom.h"

#define	MDSET_BEGIN_REQUEST	0x0001
#define	MDSET_END_REQUEST	0x0002
#define	MD_TRANSFER_REQUEST	0x0003
#define	MDSET_LIST_REQUEST	0x0004
#define	MDSET_SELECT_REQUEST	0x0005
#define	MDSET_DELETE_REQUEST	0x0006
#define	MDSET_RETRIEVE_EQUEST	0x0007
#define	MDSET_UPDATES_REQUEST	0x0008
#define	MDSET_FLUSH_REQUEST	0x0009

#define	MDSET_BEGIN_REPLY	0x0101
#define	MDSET_END_REPLY		0x0102
#define	MD_TRANSFER_REPLY	0x0103
#define	MDSET_LIST_REPLY	0x0104
#define	MDSET_SELECT_REPLY	0x0105
#define	MDSET_DELETE_REPLY	0x0106

/*
 * mdstore set begin/end message
 */
typedef struct mds_begin_end_req {
	uint64_t	reqnum;
	uint16_t	command;
	uint16_t	nmds;
	uint32_t	namelen;
	char		name[];
} mds_begin_end_req_t;

/*
 * mdstore transfer message
 */
typedef struct mds_transfer_req {
	uint64_t	reqnum;
	uint16_t	command;
	uint16_t	type;
	uint32_t	size;
	uint64_t	offset;
	char		md[];
} mds_transfer_req_t;

#define	PRI_TYPE		0x01
#define	HV_MD_TYPE		0x02
#define	CTL_DOM_MD_TYPE		0x04
#define	SVC_DOM_MD_TYPE		0x08

/*
 * mdstore select/delete message
 */
typedef struct mds_sel_del_req {
	uint64_t	reqnum;
	uint16_t	command;
	uint16_t	_rsvd;
	uint32_t	namelen;
	char		name[];
} mds_sel_del_req_t;

/*
 * mdstore list request
 */
typedef struct mds_list_req {
	uint64_t	reqnum;
	uint16_t	command;
	uint8_t		_rsvd[6];
} mds_list_req_t;

/*
 * variable update request
 */
typedef struct mds_update_req {
	uint64_t	reqnum;
	uint16_t	command;
	uint8_t		rsvd[6];
} mds_update_req_t;

/*
 * variable flush request
 */
typedef struct mds_flush_req {
	uint64_t	reqnum;
	uint16_t	command;
	uint8_t		rsvd[6];
} mds_flush_req_t;

/*
 * mdstore list response message
 */
typedef struct mds_list_resp {
	uint64_t	reqnum;
	uint32_t	result;
	uint16_t	booted_set;
	uint16_t	boot_set;
	char		sets[];
} mds_list_resp_t;

/*
 * mdstore generic response message
 */
typedef struct mds_resp {
	uint64_t	reqnum;
	uint32_t	result;
	uint32_t	_rsvd;
	char		data[];
} mds_resp_t;

#define	MDST_SUCCESS		0x0	/* successful operation */
#define	MDST_FAILURE		0x1	/* unspecified error */
#define	MDST_INVALID_MSG	0x2	/* unknown command */
#define	MDST_MAX_MDS_ERR	0x3	/* request to store > supported # mds */
#define	MDST_BAD_NAME_ERR	0x4	/* null name or mdset name too long */
#define	MDST_SET_EXISTS_ERR	0x5	/* mdset with name exists */
#define	MDST_ALLOC_SET_ERR	0x6	/* out of storage space for mdset */
#define	MDST_ALLOC_MD_ERR	0x7	/* out of storage space for md */
#define	MDST_MD_COUNT_ERR	0x8	/* unexpected number of mds */
#define	MDST_MD_SIZE_ERR	0x9	/* md exceeded supported size */
#define	MDST_MD_TYPE_ERR	0xa	/* unknown md type transfered */
#define	MDST_NOT_EXIST_ERR	0xb	/* unknown mdset requested */

#ifdef __cplusplus
}
#endif

#endif /* _MDSTORE_H */
