/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2010 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#ifndef	_MEMORY_H
#define	_MEMORY_H

#ifdef __cplusplus
extern "C" {
#endif

#include "basics.h"
#include "ldom.h"
#include "mdesc_impl.h"
#include "resource_id.h"
#include "response.h"
#include "zpri.h"

/*
 * For a bound_mblock the pblk link is used for lists of bound_mblocks that
 * belong to a given pri_mblock.  For the pri_mblock there are two lists,
 * free_mem and bound_mem - the pblk link is used for both.
 */

typedef struct mblock mblock_t;

struct mblock {
	uint64_t	paddr;
	uint64_t	raddr;
	uint64_t	size;
	ident_val_t	id;
	resid_t		*mblk_resid;
	CHAIN_LINK(mblock_t, pblk);
	CHAIN_LINK(mblock_t, ldom); /* bound to same ldom or frag chain */
	CHAIN_LINK(mblock_t, sys);  /* memory used by the system */
	CHAIN_LINK(mblock_t, blk);  /* for temporary work chains */
	pri_mblock_t	*pmblk;
	bool_t		flag_isconfigd;
	ldom_t		*ldp;
	/* for display purposes */
	bool_t		is_free;		/* on the free list  */
};


typedef struct mem_blocks {
	CHAIN(mblock_t, blocks);
} mem_blocks_t;

extern mem_blocks_t free_memory;
extern mem_blocks_t system_memory;

/*
 * Definitions used by migration code when sending memory information
 * to the target machine. Three pieces of info (fields) are sent per
 * memory binding (raddr, paddr, size).
 */
#define	MBLOCK_NUM_BINDING_INFO_FIELDS		3
#define	MBLOCK_BINDING_INFO_SIZE	\
	(MBLOCK_NUM_BINDING_INFO_FIELDS * (int)sizeof (uint64_t))

/*
 * Function prototypes
 */
bool_t mem_gen_hvmd_nodes(ldom_t *ldp, md_t *mdp, md_node_t *memsp,
    md_node_t *guestp);
bool_t mem_add(ldom_t *ldp, uint64_t size, void *clientp);
bool_t mem_del(ldom_t *ldp, uint64_t size, void *clientp);
bool_t mem_cancel_dr(ldom_t *ldp, response_t *resp);
mblock_t *mem_new_mblock(pri_mblock_t *pmb, uint64_t paddr, uint64_t size);
void mem_free_mblock(mblock_t *mbp);
uint64_t ldom_size_bound_mem(ldom_t *ldp);
mblock_t *mem_bind_phys_explicit(uint64_t basepa, uint64_t size);
bool_t mem_bind_virt_explicit(ldom_t *ldp, mem_blocks_t *bindings);
mblock_t *mem_bind_phys(uint64_t size);
bool_t mem_bind_virt(ldom_t *ldp, uint64_t size, response_t *resp);
void mem_unbind_all(ldom_t *ldp);
void mem_unbind(mblock_t *mblk);
void mem_clone_bindings(ldom_t *ldp, mem_blocks_t *clones,
    mem_blocks_t *committed);
void mem_free_cloned_bindings(mem_blocks_t *clones);
bool_t mem_bind_real(ldom_t *ldp, mem_blocks_t *bindings, response_t *resp);
bool_t mem_bind_real_check(ldom_t *ldp, mem_blocks_t *bindings,
    response_t *resp);
void mem_unbind_unconfigured(ldom_t *ldp, int n_fail, void *failed_blks);
bool_t scrub_mblk(uint64_t paddr, uint64_t size);
uint64_t unbind_span(ldom_t *ldp, uint64_t addr, uint64_t size,
    bool_t do_scrub);
bool_t mem_reassign_unaligned(response_t *resp);
void mem_alloc_scrub_mblks(ldom_t *ldp, int nmblks);
int cli_check_mem_dr_capable(ldom_t *ldp, response_t *resp);
void mem_sync_with_domain(void *arg);
void mem_dr_cleanup(ldom_t *ldp);

#ifdef __cplusplus
}
#endif

#endif	/* _MEMORY_H */
