/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2010 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#ifndef	_MIG_BE_H
#define	_MIG_BE_H

#ifdef __cplusplus
extern "C" {
#endif

#include "basics.h"
#include "ldom.h"
#include "mig.h"

/*
 * Ops vector for source
 */
typedef struct mig_src_ops {
	bool_t	(*mig_src_init)(mig_seq_args_t *args);
	bool_t	(*mig_src_send_config)(mig_seq_args_t *args);
	bool_t	(*mig_src_send_bindings)(mig_seq_args_t *args);
	bool_t	(*mig_src_feasibility_check)(mig_seq_args_t *args);
	bool_t	(*mig_src_bind_check)(mig_seq_args_t *args);
	bool_t	(*mig_src_send_state)(mig_seq_args_t *args);
} mig_src_ops_t;

extern mig_src_ops_t mig_src_cold_ops;
extern mig_src_ops_t mig_src_warm_ops;

/*
 * Common source functions used by multiple backends.
 */
int migrate_config(mig_conn_t *mconn, ldom_t *ldp);
bool_t mig_src_local_feasibility_check(mig_seq_args_t *args);
bool_t mig_src_bound_feasibility_check(mig_seq_args_t *args);


/*
 * Ops vector for target
 */
typedef struct mig_tgt_ops {
	bool_t	(*mig_tgt_init)(mig_seq_args_t *args);
	bool_t	(*mig_tgt_recv_config)(mig_seq_args_t *args);
	bool_t	(*mig_tgt_recv_bindings)(mig_seq_args_t *args);
	bool_t	(*mig_tgt_feasibility_check)(mig_seq_args_t *args);
	bool_t	(*mig_tgt_bind)(mig_seq_args_t *args);
	bool_t	(*mig_tgt_recv_state)(mig_seq_args_t *args);
} mig_tgt_ops_t;

extern mig_tgt_ops_t mig_tgt_cold_ops;
extern mig_tgt_ops_t mig_tgt_warm_ops;

/*
 * Common target functions used by multiple backends.
 */
ldom_t *receive_config(mig_seq_args_t *args);
int send_final_handoff(mig_seq_args_t *args);
bool_t mig_tgt_local_feasibility_check(mig_seq_args_t *args);
bool_t mig_tgt_bound_feasibility_check(mig_seq_args_t *args);
bool_t is_mig_init_extended(mig_conn_t *mconn);

#ifdef __cplusplus
}
#endif

#endif	/* _MIG_BE_H */
