/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2010 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#ifndef	_MIG_COMMON_H
#define	_MIG_COMMON_H

/* MIGXXX could this file be combined with mig.h ? */

#ifdef __cplusplus
extern "C" {
#endif

#include "hvctl.h"

#define	MIG_COOKIE_LEN		UINT64_STR_LEN

/* Maximum size of a domain migration message payload */
#define	MIG_MAX_MSG_LEN		(5 * 1024 * 1024)

/* Size of the non-payload part of the domain migration message */
#define	MIG_MSG_HDR_LEN		((int)(sizeof (ldm_hdr_t) + sizeof (mig_msg_t)))

/*
 * Storage for all the migration command line arguments.
 */
typedef struct {
	bool_t	dry_run;
	char	*passfile;
	char	*src_ldom;
	char	*user;
	char	*host;
	char	*tgt_ldom;
	bool_t	force;
} mig_arg_t;

/* maximum password length allowed in password file */
#define	MAXPASSWD	256

int migrate_parse_options(int argc, char **argv, mig_arg_t *args);
int migrate_connect_to_host(char *host, int port);
int connect_to_host_port(char *host, int port, int timeout);

/* calls to the HV backend */
extern int call_hv_query_api_groups(int, uint32_t, hv_api_group_version_t **);
extern int call_hv_get_api_groups(int, hv_api_group_version_t **);

/* message processing */
uint8_t *init_mig_msg(int msg_type, uint32_t vlen, uint32_t status);

#if ZDBG
/* Macro to display API Group versioning info for a specific entry */
#define	DUMP_API_GROUP_ENTRY(entry)					     \
{									     \
	DBG5(zprintf("Group: 0x%.3" PRIx64 "\tVer: (%d.%d)\n",		     \
	    hton64(((hv_api_group_version_t *)(entry))->api_group),	     \
	    hton32(((hv_api_group_version_t *)(entry))->version.major_ver),  \
	    hton32(((hv_api_group_version_t *)(entry))->version.minor_ver)));\
}

/* Macro to display API Group versioning info for all entries in array */
#define	DUMP_API_GROUPS(num, entry)					\
{									\
	for (int _i = 0; _i < (num); _i++)				\
		DUMP_API_GROUP_ENTRY((entry) + _i);			\
}

#define	DUMP_MIG_MSG(msg, dir)						\
{									\
	DBG4(zprintf("MIGMSG %s: type:0x%x len:%d status:%d magic:0x%x\n", \
	    dir, (msg)->type, (msg)->len, (msg)->status, (msg)->magic)); \
}

#else	/* ZDBG */
#define	DUMP_API_GROUP_ENTRY(entry)
#define	DUMP_API_GROUPS(num, entry)
#define	DUMP_MIG_MSG(msg, dir)
#endif	/* ZDBG */

#define	RESTYPE_STR(rt)		(((rt) == STATE_RES_GUEST) ? "guest" :	\
				((rt) == STATE_RES_VCPU) ? "vcpu" :	\
				((rt) == STATE_RES_LDC) ? "ldc" :	\
				"unknown")

#ifdef __cplusplus
}
#endif

#endif	/* _MIG_COMMON_H */
