/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2008 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#ifndef	_NVLIST_H
#define	_NVLIST_H

#pragma ident	"@(#)nvlist.h	1.1	08/09/19 SMI"

#ifdef __cplusplus
extern "C" {
#endif

#include "basics.h"
#include "response.h"

typedef struct nvlist nvlist_t;
typedef struct nvpair nvpair_t;

struct nvpair {
	char	*name;
	char	*value;
};

struct nvlist {
	int	size;			/* sum of lengths of names and values */
	int	len;			/* number of variables in array  */
	nvpair_t *pairs;		/* array of <name.value> pairs */
	bool_t	ordered;		/* true if pairs are ordered */
};

void nvlist_init(nvlist_t *nvl);
void nvlist_clear(nvlist_t *nvl);
void nvlist_sort(nvlist_t *nvl);
nvpair_t *nvlist_find(nvlist_t *nvl, char *name);
bool_t nvlist_add_pair(nvlist_t *nvl, char *name, char *value);
bool_t nvlist_delete_pair(nvlist_t *nvl, char *name);
void nvlist_print(nvlist_t *nvl, char *fmt, response_t *resp);

#ifdef __cplusplus
}
#endif

#endif	/* _NVLIST_H */
