/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2009 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#pragma ident	"@(#)ldm.h	1.1	09/06/01 SMI"

#ifndef _P2V_LDM_H
#define	_P2V_LDM_H

#ifdef __cplusplus
extern "C" {
#endif

#include "ldmp2v.h"

typedef struct vdisk_binding {
	char		*name;	/* disk name 'disk0' */
	char		*dev;	/* dev@n */
	char		*vol;	/* normaal-vol0@primary-vds0 */
	struct vdisk_binding	*next;
} vdisk_binding_t;

typedef struct vdsdev_binding {
	char	*vol;		/* vol= */
	char	*dev;		/* dev= */
	struct vdsdev_binding	*next;
} vdsdev_binding_t;

typedef struct binding {
	char	*domain_state;
	vdsdev_binding_t	*vdsdevs;
	vdisk_binding_t		*vdisks;
} binding_t;

binding_t *ldm_get_bindings(const char *, const char *);
void ldm_free_bindings(binding_t *);
const char *ldm_get_vdisk_dev_by_vol(binding_t *bnd, const char *dev);
const char *ldm_get_vdisk_name_by_dev(binding_t *bnd, const char *dev);
const char *ldm_get_vdisk_name_by_vol(binding_t *bnd, const char *dev);

int ldm_vcpu_avail();
long ldm_mem_avail();
int ldm_domain_exists(const char *);
int ldm_stop_domain(const char *);
int ldm_unbind_domain(const char *);

int ldm_set_vcpu(const char *, int);
int ldm_set_memory(const char *, long);
int ldm_add_domain(const char *, const char *);
int ldm_rm_domain(const char *);
int ldm_add_vnet(const char *, const char *, const char *, const char *);
int ldm_rm_vnet(const char *, const char *);
int ldm_add_vdsdev(const char *, const char *);
int ldm_rm_vdsdev(const char *);
int ldm_add_vdisk(const char *, const char *, const char *);
int ldm_rm_vdisk(const char *, const char *);

#ifdef __cplusplus
}
#endif

#endif	/* _P2V_LDM_H */
