/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2009 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#pragma ident	"@(#)ldmp2v.h	1.1	09/06/01 SMI"

#ifndef _P2V_LDMP2V_H
#define	_P2V_LDMP2V_H

#ifdef __cplusplus
extern "C" {
#endif

#include <sys/types.h>

#define	MAX_LINE_LENGTH		256

#define	BACKEND_TYPE_ZVOL	1
#define	BACKEND_TYPE_FILE	2

#define	SLICE_TYPE_UNUSED	0
#define	SLICE_TYPE_UFS		1
#define	SLICE_TYPE_SWAP		2

/*
 * Resize request list element.
 */
typedef struct resize_info {
	char		*mntpt;	/* mountpoint */
	long		blks;	/* new number of blocks */
	int		valid;	/* valid mountpoint? */
	struct resize_info	*next;
} resize_info_t;

typedef struct slice_info {
	int		nr;		/* index nr */
	char		*special;	/* special device in physical system */
	int		slice;		/* slice no */
	ushort_t	tag;		/* partition tag */
	ushort_t	flag;		/* partition flag */
	long		start;		/* partition start block */
	long		blks;		/* partition block count */
	long		end;		/* partition end block */
	int		type;		/* SLICE_TYPE_* */
} slice_info_t;

typedef struct disk_info_t {
	int		nr;		/* index nr */
	char		*name;		/* /dev/dsk/cXtYdZ */
	char		*pri_name;	/* name in control domain */
	char		*virt_name;	/* name in guest domain */
	char		*vdisk;		/* name of the vdisk */
	long		blks;		/* number of blocks */
	int		resized;	/* 1 = a slice has been resized */
} disk_info_t;

/*
 * UFS file system data.
 */
typedef struct ufs_info {
	int		nr;		/* index nr */
	char		*mntpt;		/* mountpoint */
	char		*special;	/* /dev/dsk/cXtYdZZsN */
	char		*pri_special;	/* name in control domain */
	char		*virt_special;	/* name in guest domain */
	long		blks;		/* total blocks */
	long		free;		/* free blocks */
	long		avail;		/* available blocks */
} ufs_info_t;

/*
 * Swap partiton data.
 */
typedef struct swap_info {
	int		nr;		/* index number */
	char		*special;	/* /dev/dsk/cXtYdZsN */
	long		blks;		/* size in blocks */
} swap_info_t;

/*
 * Solaris Volume Manager data.
 */
typedef struct  svm_info {
	char		*md;		/* SVM metadevice */
	char		*special;	/* underlying special device */
} svm_info_t;

/*
 * Physical network interface
 */
typedef struct net_info {
	int		nr;		/* index number */
	char		*name;		/* bge0, e1000g0 */
	char		*ipaddr;	/* 192.168.1.1 */
	char		*mac;		/* 0:3:ba:3d:90:1a */
} net_info_t;

/*
 * Physical system configuration data.
 */
typedef struct phys {
	char		*os_rel;	/* uname -r */
	char		*method;	/* archive method */
	int		md_cnt;
	svm_info_t	*mds;
} phys_t;

/*
 * Virtual system configuration data.
 */
typedef struct virt {
	char		*name;		/* LDom name */
	char		*root;		/* mountpoint for guest / */
	char		*backend_path;	/* dataset or directoryname */
	int		backend_type;	/* ZVOL or file */
	int		sparse;		/* create sparse backend devices */
	resize_info_t	*resize_list;	/* resize requests */
	char		*vsw;		/* virtual switch */
	char		*vds;		/* virtual disk server */
} virt_t;

/*
 * Common system configuration data.
 */
typedef struct config {
	int		cpu;		/* number of CPUs */
	long		memsize;	/* memory in Mbytes */
	char		*nodename;	/* original system's nodename */
	char		*hostid;	/* NULL = auto assign */
	disk_info_t	*disks;
	int		disk_cnt;	/* number of entries in disks */
	slice_info_t	*slices;
	int		slice_cnt;	/* number of entries in slices */
	ufs_info_t	*ufss;
	int		ufs_cnt;	/* number of entries in uffs */
	net_info_t	*nets;
	int		net_cnt;	/* number of entries in nets */
	swap_info_t	*swaps;
	int		swap_cnt;	/* number of entries in swaps */
	union  {
		phys_t	phys;		/* physical system config */
		virt_t	virt;		/* virtual system config */
	} config;
#define	phys config.phys
#define	virt config.virt
} config_t;

#ifdef __cplusplus
}
#endif

#endif	/* _P2V_LDMP2V_H */
