/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2009 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

/*
 * Internal calls for manipulating resource power states
 * zpm_xxx calls are testing code
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include "response.h"
#include "frags.h"
#include "pm_shared.h"
#include "pm_caps.h"

extern int epm_get_pwr(pm_res_type_t res_type, int count, pm_node_t **nodepp);
extern int epm_build_parent_list(int count, pm_node_t **nodepp,
	pm_node_t ***parentnp);

/*
 * File contains functions for PM debug testing
 */

int
cli_set_cpu_states(int count, uint32_t *cpuids, int dir, int do_rcfg)
{
	int 			i;
	pm_resid_t		*ids;
	pmi_pwr_state_t		pmi_pwr;
	pm_config_op_t		rcfg_op;
	int			*status = NULL;
	int			ret = PWR_OP_OK;

	if (dir == 1)
		pmi_pwr = PPS_FULL_POWER;
	else if (dir == 0)
		pmi_pwr = PPS_LOWEST_POWER;
	else {
		DBG(zprintf("%s: unrecognized dir\n", __func__));
		return (PWR_OP_EINVAL);
	}

	if (do_rcfg)
		rcfg_op = (pmi_pwr == PPS_FULL_POWER) ? PCO_ADD_RESOURCE :
		    PCO_REMOVE_RESOURCE;
	else
		rcfg_op = PCO_NONE;

	/* convert cpuids to resource ids */
	ids = Xmalloc(count * sizeof (pm_resid_t));
	for (i = 0; i < count; i++) {
		ids[i].pi_res_type = PM_RES_CPU;
		ids[i].pi_res_name = Xmalloc(strlen("cpu") + 1);
		strcpy(ids[i].pi_res_name, "cpu");
		ids[i].u.cpu.pi_id_val = cpuids[i];
		ids[i].pi_fma_status = FMA_CPU_STATUS_ONLINE;
	}

	ret = pmi_adjust_power(PM_RES_CPU, rcfg_op, pmi_pwr, count, ids,
	    &status, 0x0);
	if (ret != PWR_OP_OK)
		DBG(zprintf("%s: pm_adjust_power failed, ret(%d)\n",
		    __func__, ret));

	Xfree(ids);
	Xfree(status);
	return (ret);
}

int
cli_get_cpu_states(int count, uint32_t *cpuids)
{
	int 		i, head;
	pm_resid_t	*ids;
	pm_node_t	**npp;
	pm_node_t	*commpa = NULL;
	int		ret = PWR_OP_OK;

	/* convert cpuids to pm resids */
	ids = Xcalloc(count, pm_resid_t);
	for (i = 0; i < count; i++) {
		ids[i].pi_res_type = PM_RES_CPU;
		ids[i].pi_res_name = strdup("cpu");
		ids[i].u.cpu.pi_id_val = cpuids[i];
	}

	/*
	 * get resource power state by common parent,
	 * get common parent's power state
	 */
	npp = Xmalloc(count * sizeof (pm_node_t *));
	for (i = 0; i < count; i++) {
		if (!(npp[i] = epm_find_node(&ids[i], NULL, NULL)))
			goto cli_get_done;
		if (!commpa) {
			head = i;
			commpa = npp[i]->pn_parent;
		}
		if (npp[i]->pn_parent != commpa) {
			ret |= epm_get_pwr(PM_RES_CPU, (i - head), &npp[head]);
			ret |= epm_get_pwr(PM_RES_CPU, 1, &commpa);
			commpa = NULL;
			i--;
		}
	}
	ret |= epm_get_pwr(PM_RES_CPU, (count - head), &npp[head]);
	ret |= epm_get_pwr(PM_RES_CPU, 1, &commpa);

cli_get_done:
	for (i = 0; i < count; i++)
		Xfree(ids[i].pi_res_name);
	Xfree(ids);
	Xfree(npp);
	return (ret);
}

int
cli_get_power_all(void)
{
	return (pmi_get_power_all(PM_RES_CPU, NULL));
}
