/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2010 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#ifndef	_PM_EVENTS_H_
#define	_PM_EVENTS_H_

#pragma ident	"@(#)pm_events.h	1.3	09/02/18 SMI"

#ifdef __cplusplus
extern "C" {
#endif

#include "pm/pm_shared.h"

typedef enum {
	PM_DOMAIN_CONFIG_EVENT = 1,
	PM_RES_EVENT,
	PM_FMA_EVENT,
	PM_DOMAIN_DR_REG_EVENT,
	PM_DOMAIN_DOWN_EVENT
} pm_event_type_t;

typedef struct pm_domain_event_info {
	pm_event_type_t	event_type;
	pm_res_type_t	res_type;
	void		*handle;
	CHAIN_LINK(struct pm_domain_event_info, link);
} pm_domain_event_info_t;

typedef struct pm_domain_event_list {
	CHAIN(pm_domain_event_info_t, domain_event_list);
} pm_domain_event_list_t;

typedef struct pm_util_event_info {
	pm_res_type_t	res_type;
	double		low_wm;
	double		high_wm;
	uint32_t	time_low_ms;
	uint32_t	time_high_ms;
	void		*handle;
	bool_t		active;
	CHAIN_LINK(struct pm_util_event_info, link);
} pm_util_event_info_t;

typedef struct pm_util_event_list {
	CHAIN(pm_util_event_info_t, util_event_list);
} pm_util_event_list_t;

/* Global definitions used throughout LDMD */
pm_util_event_list_t	pm_util_list;
pm_domain_event_list_t	pm_domain_list;
mutex_t			pm_util_list_lock;

bool_t	pm_domain_events_on;
bool_t	pm_domain_dr_reg_event_on;
bool_t	pm_cpu_events_on;
bool_t	pm_fma_events_on;

void init_pm_events(void);
void reset_cpu_util_data(pm_util_event_info_t *ep);
void send_pm_dom_event(ldom_t *ldp, uint_t flags, ldm_err_t *result);
void send_pm_cpu_event(ldom_t *ldp, uint_t flags, pm_resid_t *resids,
    int nresids, ldm_err_t *result);

#ifdef __cplusplus
}
#endif

#endif /* _PM_EVENTS_H_ */
