/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2008 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#ifndef	_RESOURCE_ID_H_
#define	_RESOURCE_ID_H_

#pragma ident	"@(#)resource_id.h	1.7	08/02/21 SMI"

#ifdef __cplusplus
extern "C" {
#endif

/*
 * Resource ID allocation.
 */

#include "ldmd_types.h"
#include "chain.h"

typedef struct resource_id	resid_t;
typedef struct resource_pool	res_pool_t;

typedef uint64_t ident_val_t;

#define	IDENT_MIN	((ident_val_t)0)
#define	IDENT_MAX	((ident_val_t)(1ULL<<62))

typedef enum { Res_none = 0x0, Res_sorted = 0x1 } res_flags_t;

struct resource_id {
	ident_val_t	id;
	res_pool_t	*pool;
	CHAIN_LINK(resid_t, link);
	CHAIN_LINK(resid_t, stage_link);
	CHAIN_LINK(resid_t, trans_link);
};

struct resource_pool {
	ident_val_t	minid;
	ident_val_t	maxid;
	res_flags_t	flags;
	CHAIN(resid_t, free_staging); /* staging area to delay reallocation */
	CHAIN(resid_t, free_list);
	ident_val_t	nextid;
	char		name[64];	/* for debug */
};

void create_res_pool(res_pool_t *resp, uint64_t minid, uint64_t maxid,
    res_flags_t flags);
void name_res_pool(res_pool_t *resp, char *name);
void name_ldom_res_pool(res_pool_t *resp, ldom_t *ldp, char *name);
void name_ldom_vdev_res_pool(res_pool_t *resp, ldom_t *ldp, char *vdev,
    char *vdev_name, char *name);
resid_t	*alloc_res(res_pool_t *resp);
void stage_res(resid_t *rp);
void free_from_stage(res_pool_t *resp);
void free_res(resid_t *rp);
resid_t	*pull_res_id(res_pool_t *resp, ident_val_t id);
void destroy_res_pool(res_pool_t *resp);
ident_val_t available_res_ids(res_pool_t *resp);
ident_val_t used_res_ids(res_pool_t *resp);
int is_res_available(res_pool_t *resp, ident_val_t id);

#ifdef __cplusplus
}
#endif

#endif	/* _RESOURCE_ID_H_ */
