/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2009 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#ifndef	_RESPONSE_H_
#define	_RESPONSE_H_

#pragma ident	"@(#)response.h	1.11	09/03/05 SMI"

#ifdef __cplusplus
extern "C" {
#endif

#ifndef __GNUC__
#define	__attribute__(x)
#endif

#include <locale.h>	/* gettext() - i18n */

typedef struct RESPONSE response_t;

struct RESPONSE {
	/* error code if any */
	int	ret;
	/* text response formulated */
	int	marker;			/* marks point for reset */
	int	space;
	int	length;
	char	*bufp;
};

/*
 * Offset to the location in the 'bufp' where the content starts.
 *
 * Leaves space at beginning of the response for the LDM protocol
 * headers and the response length, condition code and text block length.
 */
#define	RESPONSE_CONTENT_OFFSET	\
	(sizeof (ldm_hdr_t) + sizeof (ldm_op_reply_t) + (3 * sizeof (uint32_t)))

/*PRINTFLIKE2*/
int rprintf(response_t *resp, char *fmt, ...) __attribute__(
	(format(printf, 2, 3)));

int rputs_safe(response_t *resp, char *s, size_t maxlen);

void init_response(response_t **resp);
void fini_response(response_t *resp);
void mark_response(response_t *resp);
void reset_response(response_t *resp);
char *get_response_text(response_t *resp);

#ifdef __cplusplus
}
#endif

#endif	/* _RESPONSE_H_ */
