/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2009 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#ifndef	_SSL_CONNECTION_H
#define	_SSL_CONNECTION_H

#pragma ident	"@(#)ssl_connection.h	1.3	09/04/01 SMI"

#ifdef __cplusplus
extern "C" {
#endif

#include <openssl/ssl.h>
#include <openssl/rand.h>
#include <openssl/err.h>
#include <openssl/engine.h>

/*
 * Prototypes and pointers to functions needed from libssl.
 */
typedef void (*SSL_load_error_strings_pt)(void);
typedef SSL_METHOD *(*SSLv23_server_method_pt)(void);
typedef int (*SSL_CTX_use_PrivateKey_file_pt)(SSL_CTX *ctx, const char *file,
    int type);
typedef int (*SSL_get_error_pt)(const SSL *ssl, int ret);
typedef long (*ERR_get_error_pt)(void);
typedef char *(*ERR_error_string_pt)(unsigned long e, char *buf);
typedef int (*SSL_accept_pt)(SSL *ssl);
typedef int (*SSL_CTX_use_certificate_chain_file_pt)(SSL_CTX *ctx,
    const char *file);
typedef int (*SSL_library_init_pt)(void);
typedef SSL_CTX *(*SSL_CTX_new_pt)(const SSL_METHOD *method);
typedef SSL_METHOD *(*SSLv23_client_method_pt)(void);
typedef int (*SSL_write_pt)(SSL *ssl, const void *buf, int num);
typedef void (*RAND_seed_pt)(const void *buf, int num);
typedef int (*SSL_connect_pt)(SSL *ssl);
typedef int (*SSL_set_fd_pt)(SSL *ssl, int fd);
typedef int (*SSL_get_fd_pt)(SSL *ssl);
typedef void (*SSL_free_pt)(SSL *ssl);
typedef int (*SSL_read_pt)(SSL *ssl, void *buf, int num);
typedef SSL *(*SSL_new_pt)(SSL_CTX *ctx);
typedef SSL_CTX *(*SSL_get_SSL_CTX_pt)(const SSL *ssl);
typedef void (*SSL_CTX_free_pt)(SSL_CTX *ctx);
typedef void (*CRYPTO_set_locking_callback_pt)(void (*locking_function)
	(int mode, int n, const char *file, int line));
typedef int (*CRYPTO_num_locks_pt)(void);
typedef void (*CRYPTO_set_id_callback_pt)(unsigned long (*id_function)(void));
typedef ENGINE *(*ENGINE_by_id_pt)(const char *id);
typedef int (*ENGINE_init_pt)(ENGINE *e);
typedef void (*ENGINE_load_pk11_pt)(void);
typedef int (*ENGINE_set_default_ciphers_pt)(ENGINE *e);


/*
 * function prototypes of SASL functions loaded in load_SSL_lib()
 */
SSL_new_pt			SSL_new_f;
SSL_free_pt			SSL_free_f;
SSL_read_pt			SSL_read_f;
SSL_write_pt			SSL_write_f;
SSL_connect_pt			SSL_connect_f;
SSL_set_fd_pt			SSL_set_fd_f;
SSL_get_fd_pt			SSL_get_fd_f;
SSL_load_error_strings_pt	SSL_load_error_strings_f;
SSLv23_server_method_pt		SSLv23_server_method_f;
SSL_get_error_pt		SSL_get_error_f;
ERR_get_error_pt		ERR_get_error_f;
ERR_error_string_pt		ERR_error_string_f;
SSL_accept_pt			SSL_accept_f;
SSL_library_init_pt		SSL_library_init_f;
SSLv23_client_method_pt		SSLv23_client_method_f;
RAND_seed_pt			RAND_seed_f;
SSL_CTX_new_pt			SSL_CTX_new_f;
SSL_CTX_free_pt			SSL_CTX_free_f;
SSL_get_SSL_CTX_pt		SSL_get_SSL_CTX_f;
SSL_CTX_use_PrivateKey_file_pt	SSL_CTX_use_PrivateKey_file_f;
SSL_CTX_use_certificate_chain_file_pt	SSL_CTX_use_certificate_chain_file_f;
CRYPTO_set_locking_callback_pt	CRYPTO_set_locking_callback_f;
CRYPTO_num_locks_pt		CRYPTO_num_locks_f;
CRYPTO_set_id_callback_pt	CRYPTO_set_id_callback_f;
ENGINE_set_default_ciphers_pt   ENGINE_set_default_ciphers_f;
ENGINE_init_pt			ENGINE_init_f;
ENGINE_by_id_pt			ENGINE_by_id_f;
ENGINE_load_pk11_pt		ENGINE_load_pk11_f;

#define	FUNCTION_ADD(_function, _pointer, _lib, _func_name, _ret)	\
	_function = (_pointer)dlsym(_lib, _func_name);			\
	if (_function == NULL) {					\
		DBG(zprintf("%s dlsym failed\n", _func_name));		\
		_ret += -1;						\
	}
#define	RAND_BUF_SIZE	1024
#define	SERVER_KEY	"/var/opt/SUNWldm/server.key"
#define	SERVER_CRT	"/var/opt/SUNWldm/server.crt"
#define	PKCS11_ENGINE	"pkcs11"

/* defined in ssl_connnection.c */
extern SSL_CTX *client_ssl_ctx;

/* defined in ssl_connnection.c */
extern SSL_CTX *server_ssl_ctx;

int	load_SSL_lib();
int	ldmd_setup_SSL();


#ifdef __cplusplus
}
#endif

#endif	/* _SSL_CONNECTION_H */
