/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2008 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#pragma ident	"@(#)sunmc_client.c	1.7	08/02/21 SMI"

#include <string.h>

#include "basics.h"
#include "cli_conn.h"
#include "sunmc_client.h"
#include "allocate.h"
#include "zprintf.h"
#include "fma_cpu.h"

static void sunmc_write_response(cli_conn_t *conn, sunmc_resp_op_t resp);

static void
sunmc_client_op(sunmc_msg_op_t *incoming, sunmc_resp_op_t *outgoing)
{
	int			rv;
	fma_cpu_service_req_t	cpu_msg;
	fma_cpu_resp_t		cpu_resp;

	cpu_msg.req_num = hton64(0);
	cpu_msg.msg_type = hton32(FMA_CPU_REQ_STATUS);
	cpu_msg.cpu_id = incoming->cpuid;

	rv = fma_cpu_svc_get_p_status(&cpu_msg, &cpu_resp);
	if (rv != 0) {
		DBG(zprintf("sunmc_client_op: Failed status call\n"));
		outgoing->status = hton32(-1);
		return;
	}
	if (ntoh32(cpu_resp.result) != FMA_CPU_RESP_FAILURE) {
		switch (ntoh32(cpu_resp.status)) {
		case FMA_CPU_STAT_ONLINE:
			outgoing->status = hton32(P_ONLINE);
			break;
		case FMA_CPU_STAT_OFFLINE:
			outgoing->status = hton32(P_OFFLINE);
			break;
		default:
			outgoing->status = hton32(-1);
		}
	} else {
		outgoing->status = hton32(-1);
	}
}

void
handle_sunmc_client(cli_conn_t *conn)
{
	sunmc_msg_op_t		*conn_msg;
	sunmc_msg_op_t		op;
	sunmc_resp_op_t		out_op;

	conn_msg = (sunmc_msg_op_t *)conn->bufp;
	op.cpuid = conn_msg->cpuid;

	sunmc_client_op(&op, &out_op);
	sunmc_write_response(conn, out_op);
}

static void
sunmc_write_response(cli_conn_t *conn, sunmc_resp_op_t resp)
{
	char			*bufp;
	ldm_hdr_t		*hdr;
	ldm_op_reply_t		*reply;
	int			reply_len, msg_len;

	reply_len = sizeof (sunmc_resp_op_t);
	msg_len = sizeof (ldm_hdr_t) + sizeof (ldm_op_reply_t) + reply_len;

	bufp = Xmalloc(msg_len);

	/* set LDM protocol headers */
	hdr = (ldm_hdr_t *)bufp;
	hdr->msg_type = hton32(LDM_OP_REPLY);
	hdr->payload_len = hton32(sizeof (ldm_op_reply_t) + reply_len);

	reply = (ldm_op_reply_t *)(bufp + sizeof (ldm_hdr_t));
	reply->rp_len = hton32(reply_len);
	reply->rp_reqid = conn->reqid;

	memcpy(reply->rp_data, &resp, sizeof (sunmc_resp_op_t));

	client_write_buffer(conn, bufp);
}
