/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2010 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#ifndef	_SUPPORT_H
#define	_SUPPORT_H

#ifdef __cplusplus
extern "C" {
#endif

#include <inttypes.h>
#include "ldom.h"
#include "mdesc_impl.h"

/*
 * I/O support
 */
int full_read(int fd, uint8_t *bufp, int size);
int full_write(int fd, uint8_t *bufp, int size);
int timed_SSL_read(mig_conn_t *mconn, void *buf, size_t size, int timeout);
int timed_SSL_write(mig_conn_t *mconn, void *buf, size_t size, int timeout);
int timed_read(mig_conn_t *mconn, void *buf, size_t size, int timeout);
int timed_write(mig_conn_t *mconn, void *buf, size_t size, int timeout);

/*
 * Version Negotiation
 */
typedef struct {
	uint16_t major_number;
	uint16_t minor_number;
} prot_version_t;

#define	NUM_VERS(v)	(sizeof (v) / sizeof (prot_version_t))

/*
 * Test if prot_version_t is == a given { major, minor }
 */
#define	VERSION_EQ(v, _major, _minor)					\
	((v.major_number == _major) && (v.minor_number == _minor))

/*
 * Test if prot_version_t is >= a given { major, minor }
 */
#define	VERSION_GTEQ(v, _major, _minor)					\
	((v.major_number > _major) ||					\
	((v.major_number == _major) &&					\
	(v.minor_number >= _minor)))

#define	INVALID_VERSION	0xff

bool_t supported_version(int num_versions, prot_version_t *sup_versionsp,
    uint16_t major, uint16_t minor);
bool_t negotiate_version(int num_versions, prot_version_t *sup_versionsp,
    uint16_t req_major, uint16_t *new_majorp, uint16_t *new_minorp);

/*
 * Error injection
 */
#if ZDBG
#define	INJECT_ERROR(e)	inject_error(e)
#else
#define	INJECT_ERROR(e)	0
#endif

int inject_error(uint64_t error);

/*
 * Miscellaneous
 */
#define	BYTES2KB(x)	((x) / 1024)
#define	BYTES2MB(x)	((BYTES2KB(x)) / 1024)
#define	BYTES2GB(x)	((BYTES2MB(x)) / 1024)

void dump_msg(uint8_t *buf, int len);
bool_t dump_to_file(void *buf, int size, char *ftemplate, ...);
void dump_md_to_file(md_t *md, char *filename);
bool_t do_read_mem_to_file(char *file, uint64_t addr, uint64_t len);
bool_t do_write_file_to_addr(char *file, uint64_t addr);
md_t *read_md_from_file(char *filename);
md_t *get_md_from_file(char *mdname);
char *conv_size(uint64_t size);
void sort_int_array(int *array, int nelem);
md_node_t *find_guest_md_node(md_t *hvmdp, ldom_t *ldp);
int64_t scrub_mem(uint64_t pa, uint64_t len);

#ifdef __cplusplus
}
#endif

#endif	/* _SUPPORT_H */
