/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2009 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#ifndef	_SVCS_H
#define	_SVCS_H

#pragma ident	"@(#)svcs.h	1.4	09/05/07 SMI"

#ifdef __cplusplus
extern "C" {
#endif

#include "chain.h"
#include "support.h"
#include "libds.h"

typedef int (*ds_init_f)(void);
typedef void (*ds_fini_f)(void);
typedef int (*ds_svc_reg_f)(ds_capability_t *cap, ds_ops_t *ops);
typedef int (*ds_clnt_reg_f)(ds_capability_t *cap, ds_ops_t *ops);
typedef int (*ds_isready_f)(ds_hdl_t hdl, boolean_t *is_ready);
typedef int (*ds_domain_lookup_f)(ds_hdl_t hdl, ds_domain_hdl_t *dhdlp);
typedef int (*ds_hdl_lookup_f)(char *service, boolean_t is_client,
    ds_hdl_t *hdlsp, uint_t maxhdls, uint_t *nhdlsp);
typedef int (*ds_unreg_hdl_f)(ds_hdl_t hdl);
typedef int (*ds_send_msg_f)(ds_hdl_t hdl, void *buf, size_t buflen);
typedef int (*ds_recv_msg_f)(ds_hdl_t hdl, void *buf, size_t buflen,
    size_t *msglen);
typedef void (*ds_unreg_svc_f)(char *service, boolean_t is_client);
typedef int (*ds_dom_name_to_hdl_f)(char *domain_name, ds_domain_hdl_t *dhdlp);
typedef int (*ds_dom_hdl_to_name_f)(ds_domain_hdl_t dhdl, char *domain_name,
    uint_t maxnamlen);

typedef struct libds_ops {
	ds_init_f	ds_init;
	ds_fini_f	ds_fini;
	ds_svc_reg_f	ds_svc_reg;
	ds_clnt_reg_f	ds_clnt_reg;
	ds_isready_f	ds_isready;
	ds_hdl_lookup_f	ds_hdl_lookup;
	ds_send_msg_f	ds_send_msg;
	ds_unreg_svc_f	ds_unreg_svc;
	ds_unreg_hdl_f	ds_unreg_hdl;
	ds_recv_msg_f	ds_recv_msg;
	ds_dom_name_to_hdl_f	ds_dom_name_to_hdl;
	ds_dom_hdl_to_name_f	ds_dom_hdl_to_name;
	ds_domain_lookup_f	ds_domain_lookup;
} libds_ops_t;

extern libds_ops_t libds_ops;

/*
 * Service properties.
 */
#define	SVC_OUTGOING	0x1

enum {
	DS_SERVICE = 0,
	DS_CLIENT = 1,
} ds_type_t;

/*
 * info structure for userland ds communication
 */
typedef struct libds_info {
	int ds_type; /* whether ds client or service */
	ds_capability_t *cap;
	ds_ops_t *ds_ops;
} libds_info_t;

libds_info_t *cp_dsinfo(libds_info_t *dsinfo, char *svc_id);

/*
 * Service operations vector.
 */
typedef struct svc_ops {
	void (*reg_handler)(struct ds_svc *svc);
	void (*unreg_handler)(struct ds_svc *svc);
	void (*data_handler)(struct ds_svc *svc, void *buf, size_t buflen);
	void (*seq_handler)(struct ds_svc *svc, void *buf, size_t buflen);
	void *data;
} svc_ops_t;

/*
 * Returned by the info() interface which must be exported by all plugin
 * files.  Holds information about the services the plugin file provides. A
 * plugin file may provide multiple services but they must all be at the same
 * version(s) and on the same channel.
 */
typedef struct svc_info {
	prot_version_t svcs_ver;	/* services framework version */
	int num_chans;		/* number of channels service works on */
	char **chan_names;	/* names of channel services provided on */
	int num_svcs;			/* number of services provided */
	char **svcs;		/* array of names of services provided */
	int *props;			/* array of service properties */
	int num_vers;			/* number of versions supported */
	prot_version_t *versions;	/* array of versions supported */

	libds_info_t **libds_svcs;		/* array of services info */
} svc_info_t;

#define	SVCS_VER_MAJOR	1		/* all services use ver 1.0 for now */
#define	SVCS_VER_MINOR	0

/*
 * For a given service used to hold information specific to each plugin file
 * which supports versions of the service.
 */
typedef struct svc_plugin_info {
	char *filename;			/* name of plugin file */
	int num_vers;			/* number of versions supported */
	prot_version_t *versions;	/* versions supported by file */
	CHAIN_LINK(struct svc_plugin_info, link); /* next file */
} svc_plugin_info_t;

/*
 * Holds information about a service which is provided by one or more
 * plugin files. Each plugin file which provides the service will support
 * different major versions but the channel the service is on and its
 * properties must be the same.
 */
typedef struct svc {
	char *name;			/* name of service */
	char *chan_name;		/* name of channel service is on */
	uint64_t hdl_base;		/* base for service handles */
	int props;			/* service properties */
	int num_vers;			/* number of versions supported */
	prot_version_t **versions;	/* versions supported */
	CHAIN(struct svc_plugin_info, plugins);	/* plugins providing service */
	CHAIN_LINK(struct svc, link);	/* next service */
	CHAIN_LINK(struct svc, chanlink); /* links services on channel */
	CHAIN_LINK(struct svc, outlink); /* links outgoing services on chan */
} svc_t;

/*
 * Used to point to the services plugged in on each channel.
 */
typedef struct channel_plugins {
	char *chan_name;		/* channel name */
	CHAIN(struct svc, plugged_svcs); /* services plugged in on channel */
	CHAIN(struct svc, outgoing); 	/* outgoing services */
	CHAIN_LINK(struct channel_plugins, link); /* next channel */
} channel_plugins_t;

svc_t *find_plugged_svc_by_name(channel_plugins_t *channel_plugin, char *name);
svc_t *find_plugged_svc_on_chan_by_name(char *chan_name, char *svc_name);
channel_plugins_t *find_channel_plugins_by_name(char *name);
int get_plugged_services(void);
int load_svc_plugin(struct ds_svc *svc);
void check_libds_support(void);
void register_libds_plugins(void);

#ifdef __cplusplus
}
#endif

#endif	/* _SVCS_H */
