/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2006 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#pragma ident	"@(#)libmgr.c	1.2	06/11/14 SMI"

#include <dlfcn.h>
#include <thread.h>
#include <synch.h>
#include <stdio.h>
#include <stdlib.h>
#include <strings.h>

#include "allocate.h"
#include "chain.h"
#include "fatal.h"
#include "ldom.h"
#include "libmgr.h"

static mutex_t load_lock = DEFAULTMUTEX;

typedef struct loadlib {
	char *name;
	void *dl_hdl;
	int refcnt;
	CHAIN_LINK(struct loadlib, link);
} loadlib_t;

CHAIN(loadlib_t, loaded_libs) = DEFAULTCHAIN;

void *
load_library(char *name)
{
	loadlib_t *lib;
	char *libpath;
	void *dl_hdl = NULL;

	mutex_lock(&load_lock);
	CHAIN_ITER(loaded_libs, lib, link) {
		if (strcmp(lib->name, name) == 0)
			break;
	}
	if (lib == NULL) {
		libpath = zsprintf("%s/%s", zeus.pkgdir, name);
		dl_hdl = dlopen(libpath, RTLD_NOW);
		if (dl_hdl == NULL) {
			free(libpath);
		} else {
			free(libpath);
			lib = Xmalloc(sizeof (*lib));
			lib->name = Xstrdup(name);
			lib->dl_hdl = dl_hdl;
			lib->refcnt = 1;
			CHAIN_ADD(loaded_libs, lib, link);
		}
	} else {
		dl_hdl = lib->dl_hdl;
		lib->refcnt++;
	}
	mutex_unlock(&load_lock);

	return (dl_hdl);
}

int
unload_library(void *dl_hdl)
{
	loadlib_t *lib;

	mutex_lock(&load_lock);
	CHAIN_ITER(loaded_libs, lib, link) {
		if (lib->dl_hdl == dl_hdl)
			break;
	}
	if (lib != NULL) {
		lib->refcnt--;
		ASSERT(lib->refcnt >= 0);
		if (lib->refcnt == 0) {
			CHAIN_UNLINK(loaded_libs, lib, link);
			dlclose(lib->dl_hdl);
			Xfree(lib->name);
			Xfree(lib);
		}
	}
	mutex_unlock(&load_lock);

	return (lib == NULL ? -1 : 0);
}
