/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2009 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#include <stdio.h>
#include <stdlib.h>

#include "ptrvec.h"
#include "allocate.h"

typedef void *anyptr_t;

typedef struct ptrvec {
	int		max_size;
	int		size;
	anyptr_t	*vector;
} ptrvec_s;

#define	PTRVEC(p)	((ptrvec_s *)(p))

#define	INITIAL_SIZE	40

ptrvec_t
ptrvec_new(void)
{
	ptrvec_s *pvp;

	pvp = Xmalloc(sizeof (ptrvec_s));

	pvp->vector	= Xmalloc(INITIAL_SIZE * sizeof (anyptr_t));
	pvp->max_size	= INITIAL_SIZE;
	pvp->size	= 0;

	return (pvp);
}


void
ptrvec_free(ptrvec_t handle)
{
	ptrvec_s *pvp = PTRVEC(handle);

	free(pvp->vector);
	free(pvp);
}


void
ptrvec_append(ptrvec_t handle, void *p)
{
	ptrvec_s *pvp = PTRVEC(handle);

	if (pvp->size == pvp->max_size) {
		anyptr_t	*new_vector;
		size_t		new_max_size = pvp->max_size * 2;

		new_vector = (anyptr_t *)realloc(pvp->vector,
		    new_max_size * sizeof (anyptr_t));
		if (new_vector == NULL)
			return;		/* XXX */

		pvp->max_size	= new_max_size;
		pvp->vector	= new_vector;
	}
	pvp->vector[pvp->size++] = p;
}


void
ptrvec_clear(ptrvec_t handle)
{
	ptrvec_s *pvp = PTRVEC(handle);

	pvp->size = 0;
}

void
ptrvec_delete(ptrvec_t handle)
{
	int i;

	for (i = 0; i < ptrvec_count(handle); i++)
		Xfree(ptrvec_index(handle, i));
	ptrvec_free(handle);
}

int
ptrvec_count(ptrvec_t handle)
{
	ptrvec_s *pvp = PTRVEC(handle);

	return (pvp->size);
}


void **
ptrvec_vector(ptrvec_t handle)
{
	ptrvec_s *pvp = PTRVEC(handle);

	return (pvp->vector);
}


void *
ptrvec_index(ptrvec_t handle, int index)
{
	ptrvec_s *pvp = PTRVEC(handle);

	if (index < ptrvec_count(handle))
		return (pvp->vector[index]);

	return (NULL);
}
