/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2010 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <libintl.h>
#include <tzfile.h>

#include "policy.h"
#include "response.h"

char *
tod2str(int tod)
{
	char	hour[3];
	char	min[3];
	char	sec[3];
	char	str[9] = { NULL };
	int	units;

	if (tod < SECSPERHOUR) {
		strcat(str, "00");
	} else {
		units = tod / SECSPERHOUR;
		tod -= units * SECSPERHOUR;
		sprintf(hour, "%02d", units);
		strcat(str, hour);
	}

	strcat(str, ":");
	if (tod < SECSPERMIN) {
		strcat(str, "00");
	} else {
		units = tod / SECSPERMIN;
		tod -= units * SECSPERMIN;
		sprintf(min, "%02d", units);
		strcat(str, min);
	}

	strcat(str, ":");
	if (tod > 0) {
		sprintf(sec, "%02d", tod);
		strcat(str, sec);
	} else {
		strcat(str, "00");
	}

	return (str);
}

int
str2tod(char *str, response_t *resp)
{
	char	*tm = NULL;
	char	*eos = NULL;
	int	hour = 0;
	int	min = 0;
	int	sec = 0;

	if ((tm = strtok(str, ":")) == NULL)
		hour = (int)strtol(str, &eos, 10);
	else
		hour = (int)strtol(tm, &eos, 10);

	if (eos == NULL || *eos != '\0') {
		rprintf(resp, gettext("hours must be an integer\n"));
		return (INVALID_POLICY_TOD);
	}
	if (hour < 0 || hour > 23) {
		rprintf(resp, gettext("Policy time of day requires "
		    "a 24-hour specification [00,...,23] for hour\n"));
		return (INVALID_POLICY_TOD);
	}

	if (tm == NULL || (tm = strtok(NULL, ":")) == NULL)
		return (hour * SECSPERHOUR);

	min = (int)strtol(tm, &eos, 10);
	if (eos == NULL || *eos != '\0') {
		rprintf(resp, gettext("minutes must be an integer\n"));
		return (INVALID_POLICY_TOD);
	}
	if (min < 0 || min > 59) {
		rprintf(resp, gettext("Policy time of day requires "
		    "a specification of [00,...,59] for minutes\n"));
		return (INVALID_POLICY_TOD);
	}

	if (tm == NULL || (tm = strtok(NULL, ":")) == NULL)
		return ((hour * SECSPERHOUR) + (min * SECSPERMIN));

	sec = (int)strtol(tm, &eos, 10);
	if (eos == NULL || *eos != '\0') {
		rprintf(resp, gettext("seconds must be an integer\n"));
		return (INVALID_POLICY_TOD);
	}
	if (sec < 0 || sec > 59) {
		rprintf(resp, gettext("Policy time of day requires "
		    "a specification of [00,...,59] for seconds\n"));
		return (INVALID_POLICY_TOD);
	}

	return ((hour * SECSPERHOUR) + (min * SECSPERMIN) + sec);
}

int
current_tod(void)
{
	time_t		now;
	struct tm	*tm;
	int		tod;

	time(&now);
	tm = localtime(&now);

	tod = (tm->tm_hour * SECSPERHOUR)
	    + (tm->tm_min * SECSPERMIN)
	    + tm->tm_sec;

	return (tod);
}
