/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2006 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#pragma ident	"@(#)vfatal.c	1.7	06/10/30 SMI"

#include <stdarg.h>
#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <stdlib.h>
#include <syslog.h>

#include "basics.h"
#include "allocate.h"
#include "zprintf.h"


/* caller responsible for freeing buf */
char *
zvsprintf(char *formatp, va_list vargs)
{
	int len, bufsize;
	char *buf;

	/* let vsnprintf() tell us how big to make the buffer */
	len = vsnprintf(NULL, 0, formatp, vargs);
	bufsize = len + 1;
	buf = Xmalloc(bufsize);
	vsnprintf(buf, bufsize, formatp, vargs);

	return (buf);
}

/* caller responsible for freeing buf */
char *
zsprintf(char *formatp, ...)
{
	char *buf;
	va_list vargs;

	va_start(vargs, formatp);
	buf = zvsprintf(formatp, vargs);
	va_end(vargs);
	return (buf);
}

void
vfatal(bool_t really_quit, bool_t ign_errno, char *fmt, va_list args)
{
	char *buf1, *buf2;
	char *quit = really_quit ? "fatal error" : "warning";

	if (errno != 0 && ign_errno == false) {
		buf1 = zsprintf("%s: %s: ", quit, strerror(errno));
	} else {
		buf1 = zsprintf("%s: ", quit);
	}
	buf2 = zvsprintf(fmt, args);
	zprintf("%s%s\n", buf1, buf2);

	Xfree(buf1);
	Xfree(buf2);

	if (really_quit)
		abort();
}
