/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2006 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#pragma ident	"@(#)zprintf.c	1.3	06/10/30 SMI"


#include <stdarg.h>
#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <stdlib.h>
#include <syslog.h>
#include <thread.h>
#include <synch.h>

#include "basics.h"
#include "allocate.h"
#include "option.h"
#include "zprintf.h"

static mutex_t zplock = RECURSIVEMUTEX;

void
zprintf_lock(void)
{
	mutex_lock(&zplock);
}

void
zprintf_unlock(void)
{
	mutex_unlock(&zplock);
}

#pragma inline(zprintf_lock, zprintf_unlock)

void
zprintf(char *formatp, ...)
{
	va_list vargs;

	va_start(vargs, formatp);
	zvprintf(formatp, vargs);
	va_end(vargs);
}

void
zvprintf(char *formatp, va_list args)
{
	zprintf_lock();
	if (option.flag_daemonise && USE_SYSLOG)
		vsyslog(LOG_NOTICE, formatp, args);
	else
		vfprintf(stderr, formatp, args);
	zprintf_unlock();
}
