/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2008 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#ifndef	_VCC_H
#define	_VCC_H

#ifdef __cplusplus
extern "C" {
#endif

#include "ldmd_types.h"
#include "ldom.h"
#include "resource_id.h"
#include "response.h"
#include "dispctl.h"

#define	NEW_CONSOLE_PORT	(-1)

#define	AUTO_CONSOLE_TCP_PORT	0	/* denotes auto TCP port selection */
#define	MIN_CONSOLE_TCP_PORT	1
#define	MAX_CONSOLE_TCP_PORT	65535

#define	DEFAULT_MIN_VCC_PORT	5000
#define	DEFAULT_MAX_VCC_PORT	65535

/*
 * Properties in MD node of vcc server.
 */
#define	VCC_MIN_PORT	"vcc-min-tcp-port"
#define	VCC_MAX_PORT	"vcc-max-tcp-port"

/*
 * Properties in MD node of console port.
 */
#define	VCONS_GROUP	"vcc-group-name"
#define	VCONS_TCP_PORT	"vcc-tcp-port"
#define	VCONS_DOMAIN	"vcc-domain-name"

struct vcc_attributes {
	char		*name;
	char		*compatible;
	char		*md_name;
	char		*cli_name;
	char		*device_type;
	char		*port_name;
};

struct vcc {
	ldom_t 		*ldp;		/* owner ldom */
	char		*service;	/* service name */
	int 		min_tcp_port;	/* lower end of port range */
	int 		max_tcp_port;	/* upper end of port range */
	vcc_binding_t	*binding;
	CHAIN_LINK(vcc_t, link);	/* all vccs */
	CHAIN_LINK(vcc_t, ldom);	/* next vcc on domain */
};
#define	PORT_IN_RANGE(port, vcc) \
	((port) >= (vcc)->min_tcp_port && (port) <= (vcc)->max_tcp_port)

struct vcc_binding {
	resid_t		*cfg_handle;
	res_pool_t	free_ports;	/* free console port ids */
	res_pool_t	tcp_pool;	/* free ports in the TCP port range */
	CHAIN(vcc_console_t, consoles);
	CHAIN(console_group_t, groups);	/* groups in use */
};

struct vcc_console {
	ldcs_t		ldcs;
	resid_t		*port;		/* port id */
	console_t	*console;	/* client console */
	console_group_t *group;		/* console's group */
	vcc_t		*vcc;		/* parent concentrator */
	CHAIN_LINK(vcc_console_t, link);
};

struct console_group {
	char		*group_name;
	resid_t		*tcp_port; 	/* TCP port allocated to this group */
	int		refcnt;
	vcc_t 		*vcc; 		/* link to server */
	CHAIN_LINK(console_group_t, link);
	/* scratch work area used in updating a TCP port range */
	struct {
		int	tcp_port;
	} work;
};

extern struct vcc_attributes vcc_attrs;

vcc_console_t *vcc_new_console(vcc_t *vcc, console_group_t *group, int port);
vcc_t *vcc_new(ldom_t *ldp, char *service, int min_tcp_port, int max_tcp_port);
int vcc_bind_all(ldom_t *ldp, response_t *resp);
int vcc_unbind_all(ldom_t *ldp, response_t *resp);
void vcc_delete_all(ldom_t *ldp);
int vcc_add(ldom_t *ldp, char *service_name, int min_port, int max_port,
    response_t *resp);
int vcc_modify(vcc_t *vcc, int min_port, int max_port, response_t *resp);
int vcc_remove(vcc_t *vcc, bool_t force, response_t *resp);
void vcc_add_from_mdnode(ldom_t *ldp, md_t *guest_md, md_node_t *vdev);
vcc_t *vcc_find_by_name(char *service_name);
console_group_t *console_group_find_by_name(vcc_t *vcc, char *name);
console_group_t *console_group_find_by_port(vcc_t *vcc, int tcp_port);
vcc_console_t *vcc_console_find_by_ldcid(ldom_t *ldp, uint64_t ldcid);
void vcc_list(ldom_t *ldp, response_t *resp, dispctl_t *dctl);
void vcc_gen_ldc_endpoints(ldom_t *ldp, md_t *hv_md, md_node_t *hv_guest,
    md_node_t *ldc_endpoints);
void vcc_gen_guest_md(ldom_t *ldp, md_t *md, md_node_t *cdevs,
    md_node_t *chan_endpoints);
int vcc_unbind_console(vcc_console_t *vcons);
void vcc_destory_console(vcc_console_t *vcons);
console_group_t *vcc_get_console_group(vcc_t *vcc, char *gname, int tcp_port);
void release_console_group(console_group_t *group);

#ifdef __cplusplus
}
#endif

#endif	/* _VCC_H */
