/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2009 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#ifndef	_VCPU_H
#define	_VCPU_H

#ifdef __cplusplus
extern "C" {
#endif

#include <sys/time.h>
#include "basics.h"
#include "ldom.h"
#include "mdesc_impl.h"
#include "resource_id.h"
#include "response.h"
#include "zpri.h"

#define	NEW_VCPUID	(-1)
#define	NEW_MMU_PARTID	(-1)
#define	VCPU_UTIL_HIST_TIME	(20000)	/* utilization history - in ms */
#define	VCPU_UTIL_POLL_INTV	(2000)	/* utilization poll interval - in ms */
#define	VCPU_UTIL_TAB_NROWS	(VCPU_UTIL_HIST_TIME / VCPU_UTIL_POLL_INTV)

/*
 * Increment the index of the vcpu's utilization history table and add the
 * utilization value and time to the table.
 */
#define	ADD_VCPU_UTIL(vcpu, val, t) {					\
	vcpu->util_idx = (vcpu->util_idx + 1) % VCPU_UTIL_TAB_NROWS;	\
	vcpu->util_table[vcpu->util_idx].value = val;			\
	vcpu->util_table[vcpu->util_idx].time = t;			\
}

/*
 * Entry in the utilization history table
 */
typedef struct vcpu_util {
	double		value;		/* utilization value */
	hrtime_t	time;		/* time util value was obtained */
} vcpu_util_t;

typedef struct vcpu vcpu_t;

struct vcpu {
	int		vid;		/* caches vcpuid */
	int 		fraction;
	resid_t		*partid;	/* MMU partition id */
	resid_t		*vcpuid;
	pri_cpu_t 	*cpu;
	ldom_t 		*ldp;		/* ldom bound to */
	struct vmau	*vmau;
	CHAIN_LINK(vcpu_t, pcpu); 	/* bindings on same physical cpu */
	CHAIN_LINK(vcpu_t, ldom); 	/* bindings for a guest */
	CHAIN_LINK(vcpu_t, mau);	/* bindings on same MAU */
	bool_t		flag_isconfigd;
	double		util;		/* percent utilization */
	int		util_idx;	/* most recent entry in util hist */
	vcpu_util_t	util_table[VCPU_UTIL_TAB_NROWS]; /* util hist table */
};

enum {
	CPU_ADM_NUM,
	CPU_ADM_IDS
};

typedef struct cpu_adm {
	int		type;
	uint64_t	num;
	int		ids[];
} cpu_adm_t;

int ldom_num_bound_vcpus(ldom_t *ldp);
int ldom_num_rsvd_vcpus(ldom_t *ldp);
int ldom_num_faulted_vcpus(ldom_t *ldp);
void cpu_unbind_all(ldom_t *ldp);
vcpu_t *pid_to_vcpu(ldom_t *ldp, int pid);
pri_cpu_t *pid_to_pcpu(int pid);
vcpu_t *vid_to_vcpu(ldom_t *ldp, int vid);

bool_t cpu_add(ldom_t *ldp, cpu_adm_t *cpu_adm, response_t *resp);
bool_t cpu_del(ldom_t *ldp, cpu_adm_t *cpu_adm, response_t *resp, bool_t force);
bool_t cpu_gen_hvmd_nodes(ldom_t *ldp, md_t *mdp, md_node_t *cpusp,
    md_node_t *guestp, bool_t include_pm_status);
bool_t pid_to_vid(int pid, ldom_t **ldp, int *vid);

int	check_cpu_dr_capable(ldom_t *ldp);
int	cli_check_cpu_dr_capable(ldom_t *ldp, response_t *resp);
void	cpu_release_reserved(ldom_t *ldp);
int	get_cores_in_use(ldom_t *ldp, pri_core_t ***cores);
bool_t	cpu_reserve_core(ldom_t *ldp, pri_core_t *core, response_t *resp);
bool_t	cpu_mig_restore(ldom_t *ldp, int num_req, response_t *resp);
bool_t	cpu_mig_add_pm_bindings(ldom_t *ldp, md_t *mdp);
int	cpu_mig_reduce_to_one(ldom_t *ldp, response_t *resp);
int	num_free_vcpus(void);
int	num_bound_vcpus(void);

#ifdef __cplusplus
}
#endif

#endif	/* _VCPU_H */
