/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2009 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#ifndef	_VDISK_H
#define	_VDISK_H

#ifdef __cplusplus
extern "C" {
#endif

#include "ldmd_types.h"
#include "chain.h"
#include "ptrvec.h"
#include "ldcs.h"
#include "mac_addrs.h"
#include "resource_id.h"
#include "response.h"
#include "dispctl.h"

#define	VDISK_DEFAULT_TIMEOUT	(-1)
#define	VDISK_NO_NEW_TIMEOUT	(-2)

#define	VDISK_TIMEOUT_PROP	"vdc-timeout"
#define	VDISK_MIN_PORT		(0)

struct vdisk_attributes {
	char		*name;
	char		*compatible;
	char		*md_name;
	char		*cli_name;
	char		*device_type;
	char		*port_name;
};

struct vdisk {
	ldom_t 		*ldp;		/* owner ldom */
	char		*name;
	char		*service;	/* name of vds service */
	char		*volname;
	int		timeout;
	int		diskid;		/* vdisk id */
	vdisk_binding_t *binding;
	CHAIN_LINK(vdisk_t, ldom);	/* for list of vdisks owned by a ldom */
};

struct vdisk_binding {
	vdisk_t		*vdisk;		/* point to the vdisk */
	resid_t 	*cfg_handle;
	res_pool_t	free_ports;	/* free port ids */
	struct {
		char *db_service;
		char *db_volume;
	} work;
	CHAIN(vdisk_port_t, vdisk_ports); 	/* list of vdisk ports */
};

struct vdisk_port {
	ldcs_t		ldcs;		/* LDC channels to the vds port */
	vdisk_t		*vdisk;		/* back pointer to vdisk */
	vds_port_t	*vds_port;	/* vds-port this vdisk-port is bound */
	resid_t 	*port;		/* id of port */
	CHAIN_LINK(vdisk_port_t, link);
};

extern struct vdisk_attributes vdisk_attrs;

ident_val_t vdisk_devid(vdisk_t *vdisk);
void vdisk_add_from_mdnode(ldom_t *ldp, md_t *guest_md, md_node_t *vdev);
int link_vdisk_clients(ldom_t *ldp);
void vdisk_list(ldom_t *ldp, response_t *resp, dispctl_t *dctl);
vdisk_t *vdisk_new(ldom_t *ldp, char *name, char *service, char *volname,
    int timeout, int diskid);
void vdisk_gen_ldc_endpoints(ldom_t *ldp, md_t *hv_md, md_node_t *hv_guest,
    md_node_t *ldc_endpoints);
void vdisk_gen_guest_md(ldom_t *ldp, md_t *md, md_node_t *cdevs,
    md_node_t *chan_endpoints);
int vdisk_add(ldom_t *ldp, char *diskname, char *service, char *volname,
    int timeout, int diskid, response_t *resp);
int vdisk_modify(ldom_t *ldp, char *diskname, char *service, char *volname,
    int timeout, response_t *resp);
int vdisk_remove(ldom_t *ldp, char *diskname, bool_t force, response_t *resp);
int vdisk_bind_all(ldom_t *ldp, response_t *resp);
int vdisk_unbind_all(ldom_t *ldp, response_t *resp);
void vdisk_delete_all(ldom_t *ldp);
int vdisk_bind_check(ldom_t *ldp, response_t *resp);
void vdisk_print_bindings(ldom_t *ldp, response_t *resp);
int vdisk_mig_add_bindings(ldom_t *ldp, md_t *mdp);

#ifdef __cplusplus
}
#endif

#endif	/* _VDISK_H */
