/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2008 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#ifndef	_VDPCC_H
#define	_VDPCC_H

#ifdef __cplusplus
extern "C" {
#endif

#include "ldmd_types.h"
#include "chain.h"
#include "ptrvec.h"
#include "ldcs.h"
#include "mac_addrs.h"
#include "resource_id.h"
#include "response.h"
#include "dispctl.h"

#define	VDPCC_CLASS	"vdpcc"

struct vdpcc_attributes {
	char		*name;
	char		*compatible;
	char		*md_name;
	char		*cli_name;
	char		*device_type;
	char		*port_name;
};

struct vdpcc {
	ldom_t 		*ldp;		/* owner ldom */
	char		*name;		/* this client's name */
	char		*service;	/* name of vdpcs service */
	vdpcc_binding_t	*binding;
	CHAIN_LINK(vdpcc_t, ldom);	/* for list of vdpccs owned by a ldom */
};

struct vdpcc_binding {
	ldcs_t		ldcs;		/* LDC channels to the vdpcs server */
	resid_t 	*cfg_handle;
	vdpcs_port_t	*vdpcs_port;	/* server port connected to */
};

extern struct vdpcc_attributes vdpcc_attrs;

vdpcc_t *vdpcc_new(ldom_t *ldp, char *name, char *service);
int vdpcc_bind(vdpcc_t *vdpcc, int cfg_handle);
int vdpcc_bind_all(ldom_t *ldp, response_t *resp);
int vdpcc_unbind_all(ldom_t *ldp, response_t *resp);
void vdpcc_delete_all(ldom_t *ldp);
void vdpcc_add_from_mdnode(ldom_t *ldp, md_t *guest_md, md_node_t *vdev);
int link_vdpcc_clients(ldom_t *ldp);
void vdpcc_list(ldom_t *ldp, response_t *resp, dispctl_t *dctl);
void vdpcc_gen_ldc_endpoints(ldom_t *ldp, md_t *hv_md, md_node_t *hv_guest,
    md_node_t *ldc_endpoints);
void vdpcc_gen_guest_md(ldom_t *ldp, md_t *md, md_node_t *cdevs,
    md_node_t *chan_endpoints);
int vdpcc_add(ldom_t *ldp, char *vdpcc_name, char *service, response_t *resp);
int vdpcc_remove(ldom_t *ldp, char *vdpcc_name, bool_t force, response_t *resp);
int vdpcc_bind_check(ldom_t *ldp, response_t *resp);

#ifdef __cplusplus
}
#endif

#endif	/* _VDPCC_H */
