/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2008 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#ifndef	_VDPCS_H
#define	_VDPCS_H

#ifdef __cplusplus
extern "C" {
#endif

#include "ldmd_types.h"
#include "chain.h"
#include "ptrvec.h"
#include "mac_addrs.h"
#include "resource_id.h"
#include "response.h"
#include "dispctl.h"

struct vdpcs_attributes {
	char		*name;
	char		*compatible;
	char		*md_name;
	char		*cli_name;
	char		*device_type;
	char		*port_name;
};

struct vdpcs {
	ldom_t		*ldp;		/* owner ldom */
	char		*service;	/* service name */
	vdpcs_binding_t	*binding;
	CHAIN_LINK(vdpcs_t, link);	/* all vdpcs's */
	CHAIN_LINK(vdpcs_t, ldom);	/* next vdpcs on domain */
};

struct vdpcs_binding {
	resid_t		*cfg_handle;
	res_pool_t	free_ports;	/* free port ids */
	CHAIN(vdpcs_port_t, vports);	/* list of bound ports */
};

struct vdpcs_port {
	ldcs_t		ldcs;
	resid_t		*port;		/* port id */
	vdpcc_t		*vdpcc;		/* the vdpcc bound to this port  */
	vdpcs_t		*vdpcs;		/* the vdpcs this port is under */
	CHAIN_LINK(vdpcs_port_t, link);
};

extern struct vdpcs_attributes vdpcs_attrs;

vdpcs_t *vdpcs_new(ldom_t *ldp, char *name);
int vdpcs_bind_all(ldom_t *ldp, response_t *resp);
int vdpcs_unbind_all(ldom_t *ldp, response_t *resp);
void vdpcs_delete_all(ldom_t *ldp);
void vdpcs_add_from_mdnode(ldom_t *ldp, md_t *guest_md, md_node_t *vdev);
void vdpcs_list(ldom_t *ldp, response_t *resp, dispctl_t *dctl);
vdpcs_port_t *vdpcs_port_find_by_ldcid(ldom_t *ldp, uint64_t ldcid);
void vdpcs_gen_guest_md(ldom_t *ldp, md_t *md, md_node_t *cdevs,
    md_node_t *chan_endpoints);
void vdpcs_gen_ldc_endpoints(ldom_t *ldp, md_t *hv_md, md_node_t *hv_guest,
    md_node_t *ldc_endpoints);
int vdpcs_add(ldom_t *ldp, char *service_name, response_t *resp);
int vdpcs_remove(vdpcs_t *vdpcs, bool_t force, response_t *resp);
vdpcs_t *vdpcs_find_by_name(char *service_name);
vdpcs_port_t *vdpcs_add_port(vdpcs_t *vdpcs, vdpcc_t *vdpcc, uint64_t port);
void vdpcs_unbind_port(vdpcs_port_t *vdpcs_port);
void vdpcs_destroy_port(vdpcs_port_t *vdpcs_port);

#ifdef __cplusplus
}
#endif

#endif	/* _VDPCS_H */
