/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2009 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#ifndef	_VDS_H
#define	_VDS_H

#ifdef __cplusplus
extern "C" {
#endif

#include "ldmd_types.h"
#include "chain.h"
#include "ptrvec.h"
#include "mac_addrs.h"
#include "resource_id.h"
#include "response.h"
#include "dispctl.h"

#define	VDS_BLOCK_DEV		"vds-block-device"
#define	VDS_BLOCK_DEV_OPTS	"vds-block-device-opts"

#define	VDSDEV_OPTS_RONLY	"ro"
#define	VDSDEV_OPTS_SLICE	"slice"
#define	VDSDEV_OPTS_EXCL	"excl"

#define	VOLNAME_PROP	"vds-block-device-name"
#define	MPGROUP_PROP	"vds-mpgroup-name"

#define	VDS_DUP_OPTS	((uint_t)-1)

/*
 * Enum to decode an option string. The above definition of duplicate
 * option works as long as there are less than 31 distinct options
 */
enum {
	VDEV_RONLY	= 0x01,
	VDEV_SLICE	= 0x02,
	VDEV_EXCL	= 0x04
} vdev_optval_t;

extern char *vdsdev_opts[];
extern int vdsdev_optval[];

struct vds_attributes {
	char		*name;
	char		*compatible;
	char		*md_name;
	char		*cli_name;
	char		*device_type;
	char		*port_name;
};

struct vds {
	ldom_t		*ldp;		/* owner ldom */
	char		*service;	/* service name */
	CHAIN(vds_volume_t, volumes);	/* list of volumes on server */
	vds_binding_t	*binding;
	CHAIN_LINK(vds_t, link);	/* all vds's */
	CHAIN_LINK(vds_t, ldom);	/* next vds on domain */
};

struct vds_binding {
	resid_t		*cfg_handle;
	res_pool_t	free_ports;	/* free port ids */
	CHAIN(vds_port_t, vports);	/* list of bound ports */
};

struct vds_port {
	ldcs_t		ldcs;
	resid_t		*port;		/* port id */
	int		minor;
	vds_volume_t	*vol;		/* disk volume associated with port */
	vdisk_port_t	*vdisk_port;	/* Matching vdisk port */
	vds_t		*vds;		/* the vds this port is under */
	struct {
		char *md_volume;
		char *md_mpgroup;	/* save mpgroup from MD */
	} work;
	CHAIN_LINK(vds_port_t, link);
};

struct vds_volume {
	char		*volname;
	char		*blkdev;
	char		*options;
	vds_t		*vds;		/* the vds this volume belongs to */
	vds_port_t	*vds_port;	/* port associated with (when bound) */
	mpgroup_t	*mpgroup;
	CHAIN_LINK(vds_volume_t, link);
	CHAIN_LINK(vds_volume_t, mpgroup_link); /* volume in same mpgroup */
};

/*
 * An mpgroup_t structure contains a list of volumes. Each volume may or may
 * not be a member of an mpgroup. The mpgroups are linked together in the
 * zeus mpgroup chain, zeus.mpgroups
 */
struct mpgroup {
	char *name;
	CHAIN(vds_volume_t, volumes);
	CHAIN_LINK(mpgroup_t, link);
};


extern struct vds_attributes vds_attrs;

vds_t *vds_new(ldom_t *ldp, char *name);
int vds_bind_all(ldom_t *ldp, response_t *resp);
int vds_unbind_all(ldom_t *ldp, response_t *resp);
void vds_delete_all(ldom_t *ldp);
void vds_add_from_mdnode(ldom_t *ldp, md_t *guest_md, md_node_t *vdev);
int vds_add(ldom_t *ldp, char *service_name, response_t *resp);
int vds_remove(vds_t *vds, bool_t force, response_t *resp);
void vds_list(ldom_t *ldp, response_t *resp, dispctl_t *dctl);
vds_port_t *vds_port_find_by_ldcid(ldom_t *ldp, uint64_t ldcid);
void vds_add_volume(vds_t *vds, char *volname, char *blkdev, char *options,
	char *mpgname);
void vds_gen_ldc_endpoints(ldom_t *ldp, md_t *hv_md, md_node_t *hv_guest,
    md_node_t *ldc_endpoints);
void vds_gen_guest_md(ldom_t *ldp, md_t *md, md_node_t *cdevs,
    md_node_t *chan_endpoints);
vds_t *vds_find_by_name(char *service_name);
int vds_add_dev(vds_t *vds, char *volume, char *device, char *options,
    char *mpgname, char *servicep, bool_t force, response_t *resp);
int vds_modify_dev(vds_t *vds, char *volname, char *options,
    char *mpgname, bool_t force, response_t *resp);
int vds_remove_dev(vds_t *vds, char *volume, bool_t force, response_t *resp);
void vds_unbind_port(vds_port_t *vds_port);
void vds_destroy_port(vds_port_t *vds_port);
vds_volume_t *vds_find_volume_by_name(vds_t *vds, char *volname);
vds_port_t *vds_add_port(vds_t *vds, vds_volume_t *vol,
    uint64_t port);
uint_t vds_decode_opts(char *options);

#ifdef __cplusplus
}
#endif

#endif	/* _VDS_H */
