/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2009 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#ifndef	_VIO_H
#define	_VIO_H

#ifdef __cplusplus
extern "C" {
#endif

#include "dispctl.h"
#include "ldom.h"
#include "mdesc_impl.h"

/* includes for all the servers/clients */
#include "vcc.h"
#include "console.h"
#include "vsw.h"
#include "vnet.h"
#include "vds.h"
#include "vdisk.h"
#include "vdpcs.h"
#include "vdpcc.h"
#include "violdc.h"

#define	INVALID_DEVID	(-1)

#define	VDEVS_CFG_HANDLE	0x100
#define	CHANNELS_CFG_HANDLE	0x200

#define	CHANNELS_NAME		"channel-devices"
#define	CHANNELS_COMPAT 	"SUNW,sun4v-channel-devices"
#define	CHANNELS_DEV_TYPE	"channel-devices"
#define	CHANNEL_ENDPOINTS	"channel-endpoints"
#define	CHANNEL_ENDPOINT	"channel-endpoint"

#define	VDEV_NAME		"virtual-device"
#define	VDEVS_NAME		"virtual-devices"
#define	VDEVS_COMPAT		"SUNW,sun4v-virtual-devices"
#define	VDEVS_DEV_TYPE		"virtual-devices"
#define	VDEV_PORT		"virtual-device-port"

#define	VLDS_MD_NAME		"virtual-domain-service"
#define	VLDS_COMPAT		"SUNW,sun4v-domain-service"

#define	VLDS_DOM_HANDLE		"vlds-domain-handle"
#define	VLDS_PORT_NAME		"vlds-port"

#define	VLDC_MD_NAME		"virtual-channel"
#define	HVCTL_MD_NAME		"hvctl"

#define	VLDC_DEV_INST		0

/*
 * Each LDom must have either an vldc or libds DS channel
 */
#define	LIBDS_SVC_PREFIX	"libds-"

#define	VIO_MODE_HYBRID		"hybrid"
#define	VIO_HYBRID_VAL		0

#define	SERVICE_NAME		"svc-name"
#define	DEVALIAS		"devalias"


/* Control Domain VLDC instance numbers for HVCTL, Domain Services */
#define	VLDC_INSTANCE	0
#define	VLDC_DEV_INST	0

/*
 * Name of SP Domain Services device (need to maintain
 * value pre-defined by vBSC)
 */
#define	SP_VLDC_DEV_INST	3
#define	SP_VLDC_DEV_NAME	"spds"

/*
 * Name of vlds device for libds communication
 */
#define	VLDS_DEV_NAME	"vlds"

/*
 * Other vldc port names
 */
#define	VLDC_HVCTL_SVC_NAME	"hvctl"
#define	VLDC_FMACTL_SVC_NAME	"fmactl"

#define	DS_NODE_NAME		"domain-services"
#define	DS_PORT_NODE_NAME	"domain-services-port"

#define	DEFAULT_VID_PROP	"default-vlan-id"
#define	PVID_PROP		"port-vlan-id"
#define	VID_PROP		"vlan-id"

#define	MTU_PROP		"mtu"
#define	LINK_PROP		"linkprop"
#define	ILLEGAL_LNKP		(0xffffffffffffffffULL)
#define	SET_PHYS_STATE		"phys-state"

#define	PORT_PVID_PROP		"remote-port-vlan-id"
#define	PORT_VID_PROP		"remote-vlan-id"

#define	VID_MIN			1
#define	VID_MAX			4094
#define	VID_IN_RANGE(V)		(((V) >= VID_MIN) && ((V) <= VID_MAX))

#define	VIO_NONEMTU		0
#define	VIO_ETHERMTU		1500
#define	ETHERMTU_MIN		VIO_ETHERMTU
#define	ETHERMTU_MAX		16000
#define	MTU_IN_RANGE(V)	(((V) >= ETHERMTU_MIN) && ((V) <= ETHERMTU_MAX))

/*
 * When parsing incoming integer arguments, -1 means no argument is supplied,
 * 0 is used to indicate that the default should be set
 */
#define	VIO_NOARGSUP		(-1)
#define	VIO_SETDEFARG		0

/*
 * Macros for generating tx-ino and rx-ino property values:
 *
 * LDC ID	tx-ino	rx-ino
 * 0		0	1
 * 1		2	3
 * ...		...	...
 */
#define	TX_INO(ldc_id)	((ldc_id) * 2)
#define	RX_INO(ldc_id)	((ldc_id) * 2 + 1)

#define	_AUTO_STR_	""
#define	IS_EMPTY_STR(str)		((str)[0] == '\0')

enum {
	NO_UPD = 0,
	DELAYED_UPD = 1,
	DYNAMIC_UPD = 2,
	OTHER_UPD = 4
} update_type_t;

enum {
	MD_NONE = 0,
	MD_SELF = 1,
	MD_PEERS = 2,
	MD_SERVER = 4
} md_update_t;

#define	_AUTO_VAL_	(0)
#define	IS_VAL_AUTO(p)	((p) == _AUTO_VAL_)

void grab_guest_vio_state(ldom_t *ldp, md_t *guest_mdp, md_node_t *hv_guestp,
    md_t *hv_mdp);
ldc_epdb_t *ldc_epdb_find(int, int);
void add_sp_vio(ldom_t *ldp);
void finalize_vio_bindings(void);
void print_server_header(char *server, char *servertype, int nclients,
    response_t *resp, dispctl_t *dctl);
void print_vio(ldom_t *ldp, response_t *resp, dispctl_t *dctl);
void print_group_vio(response_t *resp, dispctl_t *dctl);
char *vio_canonical_client_name(ldom_t *ldp, char *client_type, int instance);
char *vio_canonical_service_name(ldom_t *ldp, char *service_type, int instance);
void vio_gen_ldc_endpoint_nodes(ldom_t *ldp, md_t *hv_md, md_node_t *hv_guest,
    md_node_t *ldc_endpoints);
void vio_gen_ldcs_endpoint_nodes(md_t *md, md_node_t *hv_guest,
    md_node_t *parent, uint64_t type, uint64_t target_type, ldcs_t *ldcs,
    uint64_t target_guest, ldcs_t *target_ldcs, md_node_t ***nodes);
md_node_t *vio_gen_ldc_endpoint_node(md_t *md, md_node_t *hv_guest,
    md_node_t *parent, uint64_t my_type, uint64_t target_type,
    ldc_chan_t *channel, uint64_t target_guest, ldc_chan_t *target_channel);
void vio_gen_guest_md(ldom_t *ldp, md_t *md);
md_node_t *vio_gen_vdev_port_node(md_t *md, md_node_t *vdev,
    md_node_t *chan_endpoints, char *name, uint64_t port_id, ldcs_t *ldcs);
void gen_guest_ldc_nodes(md_t *md, md_node_t *vport, md_node_t *chan_endpoints,
    ldcs_t *ldcs);
resid_t *vio_get_cfg_handle(int cfg_handle, res_pool_t *free_cfg_handles);
int vio_bind(ldom_t *ldp, response_t *resp);
int vio_mig_bind(ldom_t *ldp, md_t *mdp, response_t *resp);
int vio_mig_restore(ldom_t *ldp, md_t *mdp, md_t *hvmdp, response_t *resp);
int vio_mig_add_bindings(ldom_t *ldp, md_t *mdp);
int vio_unbind(ldom_t *ldp, response_t *resp);
void vio_delete(ldom_t *ldp);
bool_t is_providing_services(ldom_t *ldp);
bool_t vio_check_dr_capable(ldom_t *ldp, response_t *resp);
int are_vio_dr_services_online(ldom_t *ldp);
int vio_dr_add(ldom_t *ldp, char *type, ident_val_t devid, response_t *resp);
int vio_dr_rm_phase1(ldom_t *, char *, ident_val_t, response_t *);
int vio_dr_rm_phase2(ldom_t *);
int *parse_num_list(char *vlans, int *num);
char *comma_sepa_str(int num_vlans, int *vlans);
int vid_check_one(response_t *resp, int vid);
int vid_check(response_t *resp, int num_vlans, int *vlans);
void add_vlan_md_props(md_t *md, md_node_t *node, int pvid, int num_vlans,
	int *vlans, bool_t is_port);
void vio_print_bindings(ldom_t *ldp, response_t *resp);

#ifdef __cplusplus
}
#endif

#endif	/* _VIO_H */
