/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2009 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#ifndef	_VIOLDC_H
#define	_VIOLDC_H

#ifdef __cplusplus
extern "C" {
#endif

#include "ldmd_types.h"
#include "ldom.h"
#include "ldcs.h"
#include "resource_id.h"

/*
 * VLDC port types.
 */
typedef enum {
	VLDC_TYPE_HVCTL,
	VLDC_TYPE_SP,
	VLDC_TYPE_DS,
	VLDC_TYPE_FMA,
} vldc_type_t;

#define	VLDS_DOM_NAME_PROP	"vlds-domain-name"
#define	VLDS_REM_DOM_NAME_PROP	"vlds-remote-domain-name"
#define	VLDS_REM_DOM_HDL_PROP	"vlds-remote-domain-handle"

#define	NEW_DS_PORT		(-1)

#define	VLDC_SVC_NAME_PROP	"vldc-svc-name"

/*
 * Name of control domain's Domain Services device (need to maintain
 * value pre-defined by VBSC).
 */
#define	CD_VLDC_DEV_NAME	"ldom-primary"

/*
 * Other vldc port names.
 */
#define	VLDC_HVCTL_SVC_NAME	"hvctl"
#define	VLDC_FMACTL_SVC_NAME	"fmactl"
#define	VLDCC_FMA_SVC_NAME	"ldmfma"

/*
 * Prefix for domain services port names. Suffix is the domain name.
 */
#define	VLDC_DS_SVC_PREFIX	"ldom-"


struct vldc_attributes {
	char		*name;
	char		*compatible;
	char		*md_name;
	char		*cli_name;
	char		*device_type;
	char		*port_name;
};

struct vldcc_attributes {
	char		*name;
	char		*compatible;
	char		*md_name;
	char		*cli_name;
	char		*device_type;
	char		*port_name;
};

struct hvctl_attributes {
	char		*name;
	char		*compatible;
	char		*md_name;
	char		*device_type;
	char		*port_name;
};

/*
 * VLDC server.
 */
struct vldc {
	ldom_t 		*ldp;		/* owner ldom  */
	char		*service;	/* service name */
	resid_t 	*cfg_handle;
	res_pool_t	free_ports;	/* free port ids */
	CHAIN(struct vldc_port, vports); /* list of ports */
	CHAIN_LINK(struct vldc, link);	/* all vldcs */
	CHAIN_LINK(struct vldc, ldom);	/* next vldc on domain */
};

struct vldc_port {
	ldcs_t		ldcs;
	vldc_type_t	type;		/* port type, HVCTL, SP etc. */
	resid_t		*port;		/* port id */
	char		*svc_name;
	vldc_t		*vldc;		/* parent vldc server */
	/*
	 * Data for the peer, the port is connected to.
	 * Varies by port type.
	 */
	union {
		vldcc_t	*vldcc;
		ds_port_t *ds;
		sp_guest_ldc_t *sp_ldc;
		hvctl_port_t *hvctl;
	} peer;
	CHAIN_LINK(struct vldc_port, link);
};

/*
 * VLDC client.
 */
struct vldcc {
	ldom_t 		*ldp;
	resid_t 	*cfg_handle;
	char		*name;
	vldcc_port_t	*vport;		/* there is only ever a single port */
	CHAIN_LINK(struct vldcc, ldom);
};

struct vldcc_port {
	ldcs_t		ldcs;
	vldc_type_t	type;
	char		*svc_name;
	vldcc_t		*vldcc;
	/*
	 * Data for the peer, the port is connected to.
	 * Either a vldc port or a SP LDC.
	 */
	union {
		vldc_port_t *vldc_port;
		sp_guest_ldc_t *sp_ldc;
	} peer;
	CHAIN_LINK(struct vldcc_port, link);
};

/*
 * Represents a domain services client port. This is just for domain services
 * between a guest and the LDom Manager and not for SP domain services.
 */
struct ds_port {
	ldom_t		*ldp;		/* owner LDom */
	ldcs_t		ldcs;
	resid_t		*port;
	vldc_port_t 	*vldc_port;	/* the port on the LDom Manager side */
	ds_port_t	*peer_ds_port;	/* for userland DS driver */
	struct {
		uint64_t port; /* preserve the port # on conv from vldc */
	} work;
	CHAIN_LINK(struct ds_port, link);
	CHAIN_LINK(struct ds_port, ldom);
};
#define	DS_PORT_IS_VLDC(ds) ((ds)->vldc_port != NULL && \
		(ds)->peer_ds_port == NULL)
#define	DS_PORT_IS_VLDS(ds) ((ds)->peer_ds_port != NULL)

struct hvctl_port {
	ldc_chan_t	ldc;
};

extern struct vldc_attributes vldc_attrs;
extern struct vldcc_attributes vldcc_attrs;
extern struct hvctl_attributes hvctl_attrs;

void vldc_add_from_mdnode(ldom_t *ldp, md_t *guest_md, md_node_t *vdev);
void vldcc_add_from_mdnode(ldom_t *ldp, md_t *guest_md, md_node_t *vdev);
void add_hvctl_from_md(md_t *hv_md, ldom_t *ldp);
ds_port_t *ds_port_new(ldom_t *ldp, int id);
int link_vldc_sp_ports(ldom_t *ldp);
int link_vldcc_clients(ldom_t *ldp);
void vldc_gen_ldc_endpoints(ldom_t *ldp, md_t *hv_md, md_node_t *hv_guest,
    md_node_t *ldc_endpoints);
void vldcc_gen_ldc_endpoints(ldom_t *ldp, md_t *hv_md, md_node_t *hv_guest,
    md_node_t *ldc_endpoints);
void vldc_gen_guest_md(ldom_t *ldp, md_t *md, md_node_t *cdevs,
    md_node_t *chan_endpoints);
void vldcc_gen_guest_md(ldom_t *ldp, md_t *md, md_node_t *cdevs,
    md_node_t *chan_endpoints);
void vldc_list(ldom_t *ldp, response_t *resp, dispctl_t *dctl);
void vldcc_list(ldom_t *ldp, response_t *resp, dispctl_t *dctl);

void new_ldom_vldc(ldom_t *ldp, response_t *resp);
void link_domain_services(ldom_t *ldp);
void ds_gen_guest_md(ldom_t *ldp, md_t *md, md_node_t *chan_endpoints);
int ds_bind(ldom_t *ldp, response_t *resp);
void ds_unbind(ldom_t *ldp);
void ds_delete(ldom_t *ldp, ds_port_t *port);
void ds_bind_check(ldom_t *ldp);
void ds_print_bindings(ldom_t *ldp, response_t *resp);

#ifdef __cplusplus
}
#endif

#endif	/* _VIOLDC_H */
