/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2009 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#ifndef	_VMAU_H
#define	_VMAU_H

#pragma ident	"@(#)vmau.h	1.4	09/05/14 SMI"

#ifdef __cplusplus
extern "C" {
#endif

#include "basics.h"
#include "ldom.h"
#include "mdesc_impl.h"
#include "resource_id.h"
#include "response.h"
#include "zpri.h"

#define	NEW_VMAUID	(-1)

typedef struct vmau vmau_t;

/*
 * Bound Modular Arithmetic Unit(MAU). For now, each MAU can be exclusively
 * owned by only one guest.
 */
struct vmau {
	uint64_t	vid;
	resid_t		*vmauid;
	ldom_t		*ldp;		/* ldom bound to */
	pri_mau_t	*mau;		/* direct back to the phys mau */
	CHAIN_LINK(vmau_t, pmau); 	/* bindings on same physical mau */
	CHAIN_LINK(vmau_t, ldom); 	/* bindings on same guest */
	CHAIN(struct vcpu, vcpus);	/* vcpus sharing this vmau */
	bool_t		flag_isconfigd;
	struct {
		md_node_t *mau_node;	/* temp pointer to HV mau md node */
		md_node_t *cwq_node;	/* temp pointer to HV cwq md node */
	} temp;
};

enum {
	MAU_ADM_NUM,
	MAU_ADM_IDS
};

typedef struct mau_adm {
	int		type;
	uint32_t	num;
	int		ids[];
} mau_adm_t;

typedef enum {
	CRYPTO_DEV_TYPE_INVAL,
	CRYPTO_DEV_TYPE_MAU,
	CRYPTO_DEV_TYPE_CWQ
} crypto_dev_type_t;

int ldom_num_bound_maus(ldom_t *ldp);
bool_t mau_bind(ldom_t *ldp, pri_mau_t *mau, int vid);
void mau_unbind_all(ldom_t *ldp);
bool_t mau_add(ldom_t *ldp, mau_adm_t *mau_adm, response_t *resp);
bool_t mau_del(ldom_t *ldp, mau_adm_t *mau_adm, response_t *resp);
bool_t mau_gen_hvmd_nodes(ldom_t *ldp, md_t *mdp, md_node_t *mausp,
    md_node_t *guestp);
bool_t mau_accessible_to_ldom(ldom_t *ldp, pri_mau_t *mau);
int	check_crypto_dr_services(ldom_t *ldp);
int	check_crypto_dr_capable(ldom_t *ldp);
int	cli_check_crypto_dr_capable(ldom_t *ldp, response_t *resp);

bool_t mau_mig_del(ldom_t *ldp, response_t *resp);
bool_t mau_mig_add(ldom_t *ldp, int num_req, response_t *resp);

#ifdef __cplusplus
}
#endif

#endif	/* _VMAU_H */
