/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2009 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#ifndef	_VSW_H
#define	_VSW_H

#ifdef __cplusplus
extern "C" {
#endif

#include "ldmd_types.h"
#include "chain.h"
#include "ptrvec.h"
#include "mac_addrs.h"
#include "resource_id.h"
#include "response.h"
#include "dispctl.h"

#define	VSW_SW_MODE_PROP	"vsw-switch-mode"
#define	VSW_PHYS_DEV_PROP	"vsw-phys-dev"
#define	VSW_LOCAL_MAC_PROP	"local-mac-address"
#define	VSW_REMOTE_MAC_PROP	"remote-mac-address"
#define	VSW_ETHER_TYPE_PROP	"priority-ether-types"
#define	VSW_HYBRID_MODE_PROP	"hybrid"


#define	VSW_MODE_SC		"sc"

#define	VSW_SWITCHED		"switched"
#define	VSW_PROMISC		"promiscuous"
#define	VSW_ROUTED		"routed"

#define	DEFAULT_VLAN_ID		1

struct vsw_attributes {
	char		*name;
	char		*compatible;
	char		*md_name;
	char		*cli_name;
	char		*device_type;
	char		*port_name;
};

typedef enum {
	VSW_MAC = 0x1,
	VSW_MODE = 0x2,
	VSW_PVID = 0x4,
	VSW_VID = 0x8,
	VSW_DEF_VID = 0x10,
	VSW_NET_DEV = 0x20,
	VSW_MTU = 0x40,
	VSW_PSTATE = 0x80
} vsw_args_t;

struct vsw {
	ldom_t		*ldp;		/* owner ldom */
	char		*service;	/* service name */
	char		*phys_dev;	/* physical network device (if any) */
	uint64_t	phys_state;	/* IPMP network link status */
	char		*mode;		/* mode (CLI format) */
	mac_addr_t	mac;
	uint64_t	*ether_types;	/* array of priority ether types */
	int		num_ether_types; /* length of array */
	int		*vlans;
	int		num_vlans;
	int		default_vid;
	int		pvid;
	uint32_t	mtu;
	int		switchid;	/* switch id */
	vsw_binding_t	*binding;
	CHAIN_LINK(vsw_t, link);	/* all vsws */
	CHAIN_LINK(vsw_t, ldom);	/* next vsw on domain */
};

struct vsw_binding {
	resid_t		*cfg_handle;
	res_pool_t	free_ports;	/* free port ids */
	CHAIN(vsw_port_t, vports);	/* list of bound ports */
};

struct vsw_port {
	ldcs_t		ldcs;
	resid_t		*port;		/* port id */
	struct {
		char		*vnet_mode;	/* hybrid-io desired */
	} work;
	vnet_t		*vnet;		/* the vnet bound to this port  */
	vsw_t		*vsw;		/* the vsw this port is under */
	CHAIN_LINK(vsw_port_t, link);
};

/* vsw mode definitions */
typedef struct {
	char *cli;
	uint64_t md;
}  vsw_mode_options_t;

extern struct vsw_attributes vsw_attrs;
extern vsw_mode_options_t vsw_mode_options[];

vsw_t *vsw_new(ldom_t *ldp, char *service_name, char *phys_dev, mac_addr_t mac,
    char *policies, int default_vid, int pvid, int num_vlans, int *vlans,
    int mtu, uint64_t phys_state, int switchid);
void vsw_add_from_mdnode(ldom_t *ldp, md_t *guest_md, md_node_t *vdev);
vsw_port_t *vsw_add_port(vsw_t *vsw, vnet_t *vnet, uint64_t port,
    char *vnet_mode);
void vsw_unbind_port(vsw_port_t *vsw_port);
void vsw_destroy_port(vsw_port_t *vsw_port);
void vsw_list(ldom_t *ldp, response_t *resp, dispctl_t *dctl);
vsw_t *vsw_find_by_name(char *service_name);
vsw_port_t *vsw_port_find_by_ldcid(ldom_t *ldp, uint64_t ldcid);
int vsw_add(ldom_t *ldp, char *service_name, char *dev_path,
    mac_addr_t mac_address, char *mode, int default_vlan, int pvid, char *vids,
    int mtu, uint64_t phys_state, int switchid, response_t *resp);
int vsw_bind(vsw_t *vsw, int cfg_handle);
int vsw_bind_all(ldom_t *ldp, response_t *resp);
int vsw_unbind_all(ldom_t *ldp, response_t *resp);
void vsw_delete_all(ldom_t *ldp);
int vsw_set(vsw_t *vsw, char *phys_dev, mac_addr_t mac, char *mode,
    int pvid, char *vids, int mtu, uint64_t phys_state, response_t *resp);
int vsw_remove(vsw_t *vsw, bool_t force, response_t *resp);
void vsw_gen_ldc_endpoints(ldom_t *ldp, md_t *hv_md, md_node_t *hv_guest,
    md_node_t *ldc_endpoints);
void vsw_gen_guest_md(ldom_t *ldp, md_t *md, md_node_t *cdevs,
    md_node_t *chan_endpoints);
void vsw_get_sw_mode(char *net_dev, int *sw_mode_len, uint8_t **sw_mode,
    char **sw_mode_xml);
ident_val_t vsw_devid(vsw_t *vsw);

#ifdef __cplusplus
}
#endif

#endif	/* _VSW_H */
