/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2006 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#ifndef	_WAITER_H
#define	_WAITER_H

#pragma ident	"@(#)waiter.h	1.2	06/12/03 SMI"

/*
 * For managing CLI clients waiting on LDom state changes.
 */

#include "basics.h"
#include "cli_conn.h"
#include "clp.h"
#include "ldom.h"

#ifdef __cplusplus
extern "C" {
#endif

/*
 * Records a dependency a client has on an event occurring on a LDom.
 */
typedef struct waiter {
	ldom_t		*ldom;
	struct cli_conn *client;
	int		event;
	ldm_cmd_num_t	cmd;		/* client command */
	/* the next LDom the client is dependent on */
	CHAIN_LINK(struct waiter, next_ldom);
	/* the next client with a dependency on the LDom */
	CHAIN_LINK(struct waiter, next_client);
} waiter_t;

void add_waiter(ldom_t *ldom, struct cli_conn *client, int event,
    ldm_cmd_num_t cmd);
void release_waiters(ldom_t *ldom, struct cli_conn *client, int event);
void free_clients_waiters(struct cli_conn *client);

#ifdef __cplusplus
}
#endif

#endif	/* _WAITER_H */
