/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2010 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#ifndef	_XML_CNTRL_COMMON_H_
#define	_XML_CNTRL_COMMON_H_

#ifdef __cplusplus
extern "C" {
#endif

#include <libxml/tree.h>
#include <libxml/parser.h>
#include "cli_conn.h"

#define	XML_VERSION	((xmlChar *)"1.0") /* version of XML used */

#define	XML_ADD_DOMAIN		((xmlChar *)"add-domain")
#define	XML_SET_DOMAIN		((xmlChar *)"set-domain")
#define	XML_DELETE		((xmlChar *)"remove-domain")
#define	XML_STOP		((xmlChar *)"stop-domain")
#define	XML_START_DOMAIN	((xmlChar *)"start-domain")
#define	XML_BIND_DOMAIN		((xmlChar *)"bind-domain")
#define	XML_UNBIND		((xmlChar *)"unbind-domain")
#define	XML_PANIC_DOMAIN	((xmlChar *)"panic-domain")
#define	XML_LIST_CONST		((xmlChar *)"list-constraints")
#define	XML_LIST_BIND		((xmlChar *)"list-bindings")
#define	XML_LIST		((xmlChar *)"list-domain")
#define	XML_LIST_DEVICES	((xmlChar *)"list-devices")
#define	XML_LIST_SERVICES	((xmlChar *)"list-services")
#define	XML_LIST_SETS		((xmlChar *)"list-spconfig")
#define	XML_LIST_VARS		((xmlChar *)"list-variable")
#define	XML_ADD_VCPU		((xmlChar *)"add-vcpu")
#define	XML_ADD_MAU		((xmlChar *)"add-mau")
#define	XML_ADD_CRYPTO		((xmlChar *)"add-crypto")
#define	XML_ADD_MEM		((xmlChar *)"add-memory")
#define	XML_ADD_IO		((xmlChar *)"add-io")
#define	XML_ADD_VAR		((xmlChar *)"add-variable")
#define	XML_ADD_VCONSCON	((xmlChar *)"add-vconscon")
#define	XML_ADD_VDISK		((xmlChar *)"add-vdisk")
#define	XML_ADD_VDS		((xmlChar *)"add-vdiskserver")
#define	XML_ADD_VDSDEV		((xmlChar *)"add-vdiskserverdevice")
#define	XML_ADD_VNET		((xmlChar *)"add-vnet")
#define	XML_ADD_VSW		((xmlChar *)"add-vswitch")
#define	XML_ADD_SPCONFIG	((xmlChar *)"add-spconfig")
#define	XML_ADD_VDPCS		((xmlChar *)"add-vdpcs")
#define	XML_ADD_VDPCC		((xmlChar *)"add-vdpcc")
#define	XML_ADD_POLICY		((xmlChar *)"add-policy")
#define	XML_SET_VCPU		((xmlChar *)"set-vcpu")
#define	XML_SET_MAU		((xmlChar *)"set-mau")
#define	XML_SET_CRYPTO		((xmlChar *)"set-crypto")
#define	XML_SET_MEM		((xmlChar *)"set-memory")
#define	XML_SET_VAR		((xmlChar *)"set-variable")
#define	XML_SET_VCONSCON	((xmlChar *)"set-vconscon")
#define	XML_SET_VDISK		((xmlChar *)"set-vdisk")
#define	XML_SET_VDSDEV		((xmlChar *)"set-vdiskserverdevice")
#define	XML_SET_VNET		((xmlChar *)"set-vnet")
#define	XML_SET_VSW		((xmlChar *)"set-vswitch")
#define	XML_SET_SPCONFIG	((xmlChar *)"set-spconfig")
#define	XML_SET_VCONSOLE	((xmlChar *)"set-vconsole")
#define	XML_SET_POLICY		((xmlChar *)"set-policy")
#define	XML_REMOVE_VCPU		((xmlChar *)"remove-vcpu")
#define	XML_REMOVE_MAU		((xmlChar *)"remove-mau")
#define	XML_REMOVE_CRYPTO	((xmlChar *)"remove-crypto")
#define	XML_REMOVE_MEM		((xmlChar *)"remove-memory")
#define	XML_REMOVE_IO		((xmlChar *)"remove-io")
#define	XML_REMOVE_VAR		((xmlChar *)"remove-variable")
#define	XML_REMOVE_VCONSCON	((xmlChar *)"remove-vconscon")
#define	XML_REMOVE_VDISK	((xmlChar *)"remove-vdisk")
#define	XML_REMOVE_VDS		((xmlChar *)"remove-vdiskserver")
#define	XML_REMOVE_VDSDEV	((xmlChar *)"remove-vdiskserverdevice")
#define	XML_REMOVE_VNET		((xmlChar *)"remove-vnet")
#define	XML_REMOVE_VSW		((xmlChar *)"remove-vswitch")
#define	XML_REMOVE_SPCONFIG	((xmlChar *)"remove-spconfig")
#define	XML_REMOVE_RECONF	((xmlChar *)"remove-reconf")
#define	XML_REMOVE_VDPCS	((xmlChar *)"remove-vdpcs")
#define	XML_REMOVE_VDPCC	((xmlChar *)"remove-vdpcc")
#define	XML_REMOVE_POLICY	((xmlChar *)"remove-policy")
#define	XML_DOMAIN_RESET	((xmlChar *)"domain-reset")
#define	XML_GET_COOKIE		((xmlChar *)"get-migration-cookie")
#define	XML_MIGRATE_DOMAIN	((xmlChar *)"migrate-domain")
#define	XML_CANCEL_OP		((xmlChar *)"cancel-operation")
#define	XML_REBOOT_PRIMARY	((xmlChar *)"reboot-primary")

#define	XML_LDM_INTERFACE	((xmlChar *)"LDM_interface")
#define	XML_ACTION		((xmlChar *)"action")
#define	XML_DATA		((xmlChar *)"data")
#define	XML_OPTIONS		((xmlChar *)"options")
#define	XML_SNMP_USER		((xmlChar *)"snmp_user")
#define	XML_CMD			((xmlChar *)"cmd")
#define	XML_LDM_INFO		((xmlChar *)"ldom_info")
#define	XML_LDM_NAME		((xmlChar *)"ldom_name")
#define	XML_RESPONSE		((xmlChar *)"response")
#define	XML_STATUS		((xmlChar *)"status")
#define	XML_RESP_MSG		((xmlChar *)"resp_msg")
#define	XML_BINDING		((xmlChar *)"binding")
#define	XML_FREE		((xmlChar *)"free")
#define	XML_PID			((xmlChar *)"pid")
#define	XML_VID			((xmlChar *)"vid")
#define	XML_STRAND_PERCENT	((xmlChar *)"strand_percent")
#define	XML_UTIL_PERCENT	((xmlChar *)"util_percent")
#define	XML_REAL_ADDR		((xmlChar *)"real_addr")
#define	XML_PHYS_ADDR		((xmlChar *)"phys_addr")
#define	XML_MODE		((xmlChar *)"mode")
#define	XML_CPUSET		((xmlChar *)"cpuset")
#define	XML_DEV_TYPE		((xmlChar *)"device_type")
#define	XML_PORT		((xmlChar *)"port")
#define	XML_SPCONFIG		((xmlChar *)"spconfig")
#define	XML_SPCONFIG_NAME	((xmlChar *)"spconfig_name")
#define	XML_SPCONFIG_STATUS	((xmlChar *)"spconfig_status")
#define	XML_SERVICE_DOMAIN	((xmlChar *)"service_domain")
#define	XML_PHYSIO_DEVICE	((xmlChar *)"physio_device")
#define	XML_PSEUDONYM		((xmlChar *)"pseudonym")
#define	XML_HOSTID		((xmlChar *)"hostid")
#define	XML_FAILURE_POLICY	((xmlChar *)"failure-policy")
#define	XML_MASTER		((xmlChar *)"master")
#define	XML_COOKIE		((xmlChar *)"cookie")
#define	XML_TARGET		((xmlChar *)"target")
#define	XML_USERNAME		((xmlChar *)"username")
#define	XML_PASSWORD		((xmlChar *)"password")
#define	XML_NEW_LDOM		((xmlChar *)"new_ldom")
#define	XML_DRY_RUN		((xmlChar *)"dry_run")
#define	XML_FORCE		((xmlChar *)"force")
#define	XML_UTIL		((xmlChar *)"utilization")
#define	XML_UPTIME		((xmlChar *)"uptime")
#define	XML_WEIGHTED_MEAN_UTIL	((xmlChar *)"weighted_mean_util")
#define	XML_FLAGS		((xmlChar *)"flags")
#define	XML_DEVICE		((xmlChar *)"device")
#define	XML_MKFILE		((xmlChar *)"mkfile")
#define	XML_ZFS			((xmlChar *)"zvol")
#define	XML_AUTO_ADJ		((xmlChar *)"auto-adj")

#define	XML_CURRENT_STATUS	((xmlChar *)"current")
#define	XML_NEXT_STATUS		((xmlChar *)"next")

#define	XML_SUCCESS		"success"
#define	XML_FAILURE		"failure"

void xml_update_status(cli_conn_t *client, int cmd, char *ldm_name,
    char *status_str, char *resp_msg_str, ...);
void xml_add_status(xmlNodePtr parent, char *status_str,
    char *resp_msg_str, ...);
int xml_call_cli_cmd(int argc, void *argv, int cmd_num, xmlNodePtr data,
    cli_conn_t *client);
void free_argv(void **argv, int argc);
xmlNodePtr find_data_section(char *ldm_name, xmlNodePtr cmd);
int version_check(xmlNodePtr data_node, xmlNodePtr ret_cmd, int *maj_ver);
void set_missing_prop_msg(xmlNodePtr ret_data, xmlChar *resource,
    xmlChar *property);
xmlChar *get_section_resource_name(xmlNodePtr section);
xmlNodePtr xml_find_section_subnode(xmlNodePtr section, const xmlChar *name);
xmlNodePtr xml_find_v3_property(xmlNodePtr item, xmlChar *prop_name);
xmlNodePtr xml_find_section(xmlNodePtr node, const xmlChar *name);
void xml_create_return_doc(xmlDocPtr *ret_docp);
char *xml_parse_resp(char *text);
xmlNodePtr xml_find_data_section(xmlNodePtr cmd, char *name);

#ifdef __cplusplus
}
#endif

#endif /* _XML_CNTRL_COMMON_H_ */
