/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2010 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#ifndef	_XML_EVENT_H_
#define	_XML_EVENT_H_

#ifdef __cplusplus
extern "C" {
#endif

#include "cli_conn.h"

#define	LDOM_EVENT_VERSION	((xmlChar *)"1.1")

#define	XML_LDM_EVENT		((xmlChar *)"LDM_event")

#define	EVENT_REG_ALL		((xmlChar *)"reg-all-events")
#define	EVENT_REG_DOM		((xmlChar *)"reg-domain-events")
#define	EVENT_REG_RES		((xmlChar *)"reg-resource-events")
#define	EVENT_REG_HW		((xmlChar *)"reg-hardware-events")
#define	EVENT_REG_BIND_RES	((xmlChar *)"reg-bind-resource-event")
#define	EVENT_REG_PROGRESS	((xmlChar *)"reg-progress-events")
#define	EVENT_UNREG_ALL		((xmlChar *)"unreg-all-events")
#define	EVENT_UNREG_DOM		((xmlChar *)"unreg-domain-events")
#define	EVENT_UNREG_RES		((xmlChar *)"unreg-resource-events")
#define	EVENT_UNREG_HW		((xmlChar *)"unreg-hardware-events")
#define	EVENT_UNREG_BIND_RES	((xmlChar *)"unreg-bind-resource-event")
#define	EVENT_UNREG_PROGRESS	((xmlChar *)"unreg-progress-events")

#define	MIGRATION_PROGRESS	((xmlChar *)"migration-progress")
#define	MEMDR_PROGRESS		((xmlChar *)"memdr-progress")
#define	XML_PROGRESS		((xmlChar *)"progress")
#define	XML_SOURCE		((xmlChar *)"source")
#define	XML_ONGOING		((xmlChar *)"ongoing")
#define	XML_COMPLETE		((xmlChar *)"complete")

typedef struct EVENT_CLIENT		event_client_t;
typedef struct EVENT_CLIENT_LIST	event_client_list_t;

struct EVENT_CLIENT {
	cli_conn_t	*client;
	CHAIN_LINK(event_client_t, link);
};

struct EVENT_CLIENT_LIST {
	CHAIN(event_client_t, client_list);
};

void init_event_lists();
int event_add_all_event(xmlNodePtr data_node, xmlNodePtr ret_cmd,
    cli_conn_t *client,	int cmd_num);
int event_add_event(xmlNodePtr data_node, xmlNodePtr ret_cmd,
    cli_conn_t *client,	int cmd_num);

int event_rm_all_event(xmlNodePtr data_node, xmlNodePtr ret_cmd,
    cli_conn_t *client,	int cmd_num);
int event_rm_event(xmlNodePtr data_node, xmlNodePtr ret_cmd,
    cli_conn_t *client,	int cmd_num);

void event_send_dom_event(xmlChar *event, ldom_t *ldp, cli_conn_t *client);
void event_send_res_event(xmlChar *event, ldom_t *ldp, char *name,
    xmlChar *res_type, cli_conn_t *client);
void event_send_hardware_event(xmlChar *event, ldom_t *ldp, char *name,
    cli_conn_t *client);
void event_send_progress_event(xmlChar *event, ldom_t *ldp, xmlChar *status);


#ifdef __cplusplus
}
#endif

#endif /* _XML_EVENT_H_ */
