/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2009 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#ifndef	_XML_PARSE_H_
#define	_XML_PARSE_H_

#pragma ident	"@(#)xml_parse.h	1.12	09/04/08 SMI"

#ifdef __cplusplus
extern "C" {
#endif

#include "cli_conn.h"

#define	LDOM_INTERFACE_VERSION	((xmlChar *)"1.2")
#define	LDOM_INTERFACE_VERSION_MAJOR	1
#define	LDOM_INTERFACE_VERSION_MINOR	2
#define	LDOM_DATA_VERSION	((xmlChar *)"2.0")
#define	LDOM_V3_DATA_VERSION	((xmlChar *)"3.0")

/*
 * The reboot file is used for the reboot-primary command.  This file will be
 * created when we want to bind and start guest domain upon reboot of the
 * system.  Also it will also be used to have LDMD save an SP configuration
 * after the bind and start of guest domains.
 */
#define	REBOOT_FILE		"reboot_file.xml"

#define	AUTHORIZATION_CHECKED	(-1)

typedef struct {
	int	len;
	char	data[];
} xml_msg_t;

typedef enum {
	XML_ADD_RES = 0,
	XML_REMOVE_RES,
	XML_MODIFY_RES
} xml_action_t;

typedef struct {
	xmlChar		*act_name;	/* command name */
	int		(*funcp)(xmlNodePtr root_node, xmlNodePtr ret_cmd,
	    cli_conn_t *client, int cmd_num); /* function to handle this cmd */
	int		cmd_num;
	int		major;	/* ldom interface major version number */
	int		minor;	/* ldom interface minor version number */
} xml_cmd_t;

typedef struct {
	xmlChar		*resource_name;	/* resource name */
	int		(*funcp)(xmlNodePtr res_node, ldom_t *ldp,
	    int action, xmlNodePtr data, cli_conn_t *client);
} xml_resource_t;

typedef struct {
	int		argc;
	char		**argv;
	cli_conn_t	*client;
	int		cmd;
} xml_sequence_args_t;

int zeus_ldom_xml_command(cli_conn_t *cli_conn);

int parse_xml_file(xmlDocPtr doc, xmlDocPtr ret_doc, cli_conn_t *client);

int goto_resource_routine(char *name, xmlNodePtr data, ldom_t *ldp,
    int action, xmlNodePtr res_node, cli_conn_t *client);
int goto_action_routine(char *name, xmlNodePtr ret_cmd, xmlNodePtr data_node,
    cli_conn_t *client);
void *read_reboot_file(void *unused);

#ifdef __cplusplus
}
#endif

#endif /* _XML_PARSE_H_ */
