/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2009 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#pragma ident	"@(#)xml_resources.c	1.14	09/04/01 SMI"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>

#include "basics.h"
#include "support.h"
#include "allocate.h"
#include "database.h"
#include "response.h"
#include "ldom.h"
#include "fatal.h"
#include "clp.h"
#include "zprintf.h"
#include "database_internal.h"
#include "xml_parse.h"
#include "xml_cntrl_common.h"
#include "xml_common.h"
#include "xml_output.h"

int
xml_vcpu(xmlNodePtr res_node, ldom_t *ldp, int action, xmlNodePtr ret_data,
	cli_conn_t *client)
{
	int		ret = 0, cmd_num;
	const int	vcpu_args = 2;
	void		*argv[vcpu_args];
	xmlNodePtr	subnode = NULL;
	xmlChar		*number;
	int		argc = 0, i;
	bool_t		result;

	subnode = res_node->xmlChildrenNode;
	if (subnode == NULL) {
		xml_add_status(ret_data, XML_FAILURE, "Malformed XML Document:"
		    " vcpu tag contains no children tags");
		return (-1);
	}

	subnode = xml_find_subnode(res_node, NUMBER_NODE);
	if (subnode == NULL) {
		set_missing_prop_msg(ret_data, CPU_NODE, NUMBER_NODE);
		return (-1);
	}

	number = xmlNodeGetContent(subnode);
	argv[argc++] = Xstrdup((char *)number);
	xmlFree(number);

	argv[argc++] = Xstrdup(ldp->namep);

	DBG_BLOCK {
		for (i = 0; i < argc; i++)
			zprintf("Argv[%d] = %s\n", i, argv[i]);
	}

	switch (action) {
	case XML_ADD_RES:
		cmd_num = LDM_ADD_VCPU;
		break;
	case XML_MODIFY_RES:
		cmd_num = LDM_MODIFY_VCPU;
		break;
	case XML_REMOVE_RES:
		cmd_num = LDM_REMOVE_VCPU;
		break;
	}
	result = xml_call_cli_cmd(argc, argv, cmd_num, ret_data, client);
	free_argv(argv, argc);
	if (result == false)
		return (-1);
	else
		return (ret);
}

int
xml_vcrypto(xmlNodePtr res_node, ldom_t *ldp, int action, xmlNodePtr ret_data,
    cli_conn_t *client)
{
	int		ret = 0, cmd_num;
	const int	vcrypto_args = 2;
	void		*argv[vcrypto_args];
	xmlNodePtr	subnode = NULL;
	xmlChar		*number;
	int		argc = 0, i;
	bool_t		result;

	subnode = res_node->xmlChildrenNode;
	if (subnode == NULL) {
		xml_add_status(ret_data, XML_FAILURE, "Malformed XML Document:"
		    " mau tag contains no children tags");
		return (-1);
	}

	subnode = xml_find_subnode(res_node, NUMBER_NODE);
	if (subnode == NULL) {
		set_missing_prop_msg(ret_data, MAU_NODE, NUMBER_NODE);
		return (-1);
	}

	number = xmlNodeGetContent(subnode);
	argv[argc++] = Xstrdup((char *)number);
	xmlFree(number);

	argv[argc++] = Xstrdup(ldp->namep);

	DBG_BLOCK {
		for (i = 0; i < argc; i++)
			zprintf("Argv[%d] = %s\n", i, argv[i]);
	}

	switch (action) {
	case XML_ADD_RES:
		cmd_num = LDM_ADD_CRYPTO;
		break;
	case XML_MODIFY_RES:
		cmd_num = LDM_MODIFY_CRYPTO;
		break;
	case XML_REMOVE_RES:
		cmd_num = LDM_REMOVE_CRYPTO;
		break;
	}
	result = xml_call_cli_cmd(argc, argv, cmd_num, ret_data, client);
	free_argv(argv, argc);
	if (result == false)
		return (-1);
	else
		return (ret);
}

int
xml_vmem(xmlNodePtr res_node, ldom_t *ldp, int action, xmlNodePtr ret_data,
    cli_conn_t *client)
{
	int		ret = 0, cmd_num;
	const int	vmem_args = 2;
	void		*argv[vmem_args];
	xmlNodePtr	subnode = NULL;
	xmlChar		*mem_size;
	int		argc = 0, i;
	bool_t		result;

	subnode = res_node->xmlChildrenNode;
	if (subnode == NULL) {
		xml_add_status(ret_data, XML_FAILURE, "Malformed XML Document:"
		    " memory tag contains no children tags");
		return (-1);
	}

	subnode = xml_find_subnode(res_node, SIZE_NODE);
	if (subnode == NULL) {
		set_missing_prop_msg(ret_data, MEMORY_NODE, SIZE_NODE);
		return (-1);
	}

	mem_size = xmlNodeGetContent(subnode);
	argv[argc++] = Xstrdup((char *)mem_size);
	xmlFree(mem_size);

	argv[argc++] = Xstrdup(ldp->namep);

	DBG_BLOCK {
		for (i = 0; i < argc; i++)
			zprintf("Argv[%d] = %s\n", i, argv[i]);
	}

	switch (action) {
	case XML_ADD_RES:
		cmd_num = LDM_ADD_MEM;
		break;
	case XML_MODIFY_RES:
		cmd_num = LDM_MODIFY_MEM;
		break;
	case XML_REMOVE_RES:
		cmd_num = LDM_REMOVE_MEM;
		break;
	}
	result = xml_call_cli_cmd(argc, argv, cmd_num, ret_data, client);
	free_argv(argv, argc);
	if (result == false)
		return (-1);
	else
		return (ret);
}

int
xml_vds(xmlNodePtr res_node, ldom_t *ldp, int action, xmlNodePtr ret_data,
    cli_conn_t *client)
{
	int		ret = 0, cmd_num;
	const int	vds_args = 2;
	void		*argv[vds_args];
	xmlNodePtr	subnode = NULL;
	xmlChar		*vds_name;
	int		argc = 0, i;
	bool_t		result;

	subnode = res_node->xmlChildrenNode;
	if (subnode == NULL) {
		xml_add_status(ret_data, XML_FAILURE, "Malformed XML Document:"
		    " vds tag contains no children tags");
		return (-1);
	}

	subnode = xml_find_subnode(res_node, SERVICE_NAME_NODE);
	if (subnode == NULL) {
		set_missing_prop_msg(ret_data, VDS_NODE, SERVICE_NAME_NODE);
		return (-1);
	}

	vds_name = xmlNodeGetContent(subnode);
	argv[argc++] = Xstrdup((char *)vds_name);
	xmlFree(vds_name);

	if (action == XML_ADD_RES) {
		argv[argc++] = Xstrdup(ldp->namep);
		cmd_num = LDM_ADD_VDS;
	} else {
		cmd_num = LDM_REMOVE_VDS;
	}

	DBG_BLOCK {
		for (i = 0; i < argc; i++)
			zprintf("Argv[%d] = %s\n", i, argv[i]);
	}

	result = xml_call_cli_cmd(argc, argv, cmd_num, ret_data, client);
	free_argv(argv, argc);
	if (result == false)
		return (-1);
	else
		return (ret);
}

int
xml_vdsdev(xmlNodePtr res_node, ldom_t *ldp, int action, xmlNodePtr ret_data,
    cli_conn_t *client)
{
	int		ret = 0, cmd_num;
	const int	vdsdev_args = 3;
	void		*argv[vdsdev_args];
	xmlNodePtr	subnode = NULL;
	xmlChar		*vds_name, *vdsdev_name, *vds_device, *options;
	int		argc = 0, i;
	bool_t		result;

	subnode = res_node->xmlChildrenNode;
	if (subnode == NULL) {
		xml_add_status(ret_data, XML_FAILURE, "Malformed XML Document:"
		    " vds_volume tag contains no children tags");
		return (-1);
	}

	if (action != XML_REMOVE_RES) {
		subnode = xml_find_subnode(res_node, VOLUME_OPTS_NODE);
		if (subnode != NULL) {
			options = xmlNodeGetContent(subnode);
			if (options == NULL)
				argv[argc++] = zsprintf("options=");
			else {
				argv[argc++] = zsprintf("options=%s", options);
				xmlFree(options);
			}
		} else if (action == XML_MODIFY_RES) {
			set_missing_prop_msg(ret_data, VDS_VOLUME_NODE,
			    VOLUME_OPTS_NODE);
			return (-1);
		}
		if (action == XML_ADD_RES) {
			subnode = xml_find_subnode(res_node, BLOCKDEV_NODE);
			if (subnode == NULL) {
				set_missing_prop_msg(ret_data, VDS_VOLUME_NODE,
				    BLOCKDEV_NODE);
				free_argv(argv, argc);
				return (-1);
			}
			vds_device = xmlNodeGetContent(subnode);
			argv[argc++] = Xstrdup((char *)vds_device);
			xmlFree(vds_device);
			cmd_num = LDM_ADD_VDSDEV;
		} else if (action == XML_MODIFY_RES) {
			cmd_num = LDM_MODIFY_VDSDEV;
		}
	} else {
		cmd_num = LDM_REMOVE_VDSDEV;
	}

	/* Get the volume name and the service name */
	subnode = xml_find_subnode(res_node, VOLUME_NAME_NODE);
	if (subnode == NULL) {
		set_missing_prop_msg(ret_data, VDS_VOLUME_NODE,
		    VOLUME_NAME_NODE);
		free_argv(argv, argc);
		return (-1);
	}
	vdsdev_name = xmlNodeGetContent(subnode);

	subnode = xml_find_subnode(res_node, SERVICE_NAME_NODE);
	if (subnode == NULL) {
		set_missing_prop_msg(ret_data, VDS_VOLUME_NODE,
		    SERVICE_NAME_NODE);
		free_argv(argv, argc);
		return (-1);
	}
	vds_name = xmlNodeGetContent(subnode);

	/*
	 * Put the volume and service name in the
	 * <volume_name>@<service_name> format and make it the 2nd arguement
	 */
	argv[argc++] = zsprintf("%s@%s", vdsdev_name, vds_name);
	xmlFree(vdsdev_name);
	xmlFree(vds_name);

	DBG_BLOCK {
		for (i = 0; i < argc; i++)
			zprintf("Argv[%d] = %s\n", i, argv[i]);
	}

	result = xml_call_cli_cmd(argc, argv, cmd_num, ret_data, client);
	free_argv(argv, argc);
	if (result == false)
		return (-1);
	else
		return (ret);
}

int
xml_vdisk(xmlNodePtr res_node, ldom_t *ldp, int action, xmlNodePtr ret_data,
    cli_conn_t *client)
{
	int		ret = 0, cmd_num;
	const int	vdisk_args = 4;
	void		*argv[vdisk_args];
	xmlNodePtr	subnode = NULL;
	xmlChar		*vds_name, *vdsdev_name, *vdisk_name, *timeout;
	int		argc = 0, i;
	bool_t		result;

	subnode = res_node->xmlChildrenNode;
	if (subnode == NULL) {
		xml_add_status(ret_data, XML_FAILURE, "Malformed XML Document:"
		    " disk tag contains no children tags");
		return (-1);
	}

	/*
	 * parse timeout, vds server and volume nodes for add-vdisk
	 * and set-vdisk
	 */
	if (action != XML_REMOVE_RES) {
		xmlNodePtr volume_node = NULL;
		xmlNodePtr service_node = NULL;
		xmlNodePtr timeout_node = NULL;

		/* option timeout */
		timeout_node = xml_find_subnode(res_node, TIMEOUT_NODE);
		if (timeout_node != NULL) {
			timeout = xmlNodeGetContent(timeout_node);
			if (timeout == NULL)
				argv[argc++] = zsprintf("timeout=");
			else {
				argv[argc++] = zsprintf("timeout=%s", timeout);
				xmlFree(timeout);
			}
		}

		service_node = xml_find_subnode(res_node, SERVICE_NAME_NODE);
		volume_node = xml_find_subnode(res_node, VOLUME_NAME_NODE);

		if (action == XML_ADD_RES) {
			/*
			 * for add-vdisk, both vds server and volume are
			 * non NULL strings.
			 */
			if (volume_node == NULL) {
				set_missing_prop_msg(ret_data, DISK_NODE,
				    VOLUME_NAME_NODE);
				free_argv(argv, argc);
				return (-1);
			}

			if (service_node == NULL) {
				set_missing_prop_msg(ret_data, DISK_NODE,
				    SERVICE_NAME_NODE);
				free_argv(argv, argc);
				return (-1);
			}

			vds_name = xmlNodeGetContent(service_node);
			vdsdev_name = xmlNodeGetContent(volume_node);
		} else {
			/*
			 * for set-vdisk, either both vds server and volume are
			 * non NULL strings or both are NULL strings.
			 */
			if (service_node == NULL && volume_node != NULL) {
				set_missing_prop_msg(ret_data, DISK_NODE,
				    SERVICE_NAME_NODE);
				free_argv(argv, argc);
				return (-1);
			}

			if (volume_node == NULL && service_node != NULL) {
				xml_add_status(ret_data, XML_FAILURE,
				    "VDISK node missing all properties");
				free_argv(argv, argc);
				return (-1);
			}

			if (volume_node == NULL && service_node == NULL &&
			    timeout_node == NULL) {
				set_missing_prop_msg(ret_data, DISK_NODE,
				    TIMEOUT_NODE);
				free_argv(argv, argc);
				return (-1);
			}

			if (service_node != NULL) {
				vds_name = xmlNodeGetContent(service_node);
				vdsdev_name = xmlNodeGetContent(volume_node);
				argv[argc++] = zsprintf("volume=%s@%s",
				    vdsdev_name, vds_name);
				xmlFree(vds_name);
				xmlFree(vdsdev_name);
			}

		}
	}

	/* parse vdisk name */
	subnode = xml_find_subnode(res_node, VDISK_NAME_NODE);
	if (subnode == NULL) {
		set_missing_prop_msg(ret_data, DISK_NODE, VDISK_NAME_NODE);
		free_argv(argv, argc);
		return (-1);
	}
	vdisk_name = xmlNodeGetContent(subnode);
	argv[argc++] = Xstrdup((char *)vdisk_name);
	xmlFree(vdisk_name);

	if (action == XML_ADD_RES) {
		argv[argc++] = zsprintf("%s@%s", vdsdev_name, vds_name);
		xmlFree(vdsdev_name);
		xmlFree(vds_name);
		cmd_num = LDM_ADD_VDISK;
	} else if (action == XML_MODIFY_RES) {
		cmd_num = LDM_MODIFY_VDISK;
	} else {
		cmd_num = LDM_REMOVE_VDISK;
	}

	argv[argc++] = Xstrdup(ldp->namep);

	DBG_BLOCK {
		for (i = 0; i < argc; i++)
			zprintf("Argv[%d] = %s\n", i, argv[i]);
	}

	result = xml_call_cli_cmd(argc, argv, cmd_num, ret_data, client);
	free_argv(argv, argc);
	if (result == false)
		return (-1);
	else
		return (ret);
}

int
xml_vsw(xmlNodePtr res_node, ldom_t *ldp, int action, xmlNodePtr ret_data,
    cli_conn_t *client)
{
	int		ret = 0, cmd_num;
	const int	vsw_args = 4;
	void		*argv[vsw_args];
	xmlNodePtr	subnode = NULL;
	xmlChar		*nws_name, *mac_addr;
	xmlChar		*net_dev;
	xmlChar		*mode;
	int		argc = 0, i;
	bool_t		result;

	subnode = res_node->xmlChildrenNode;
	if (subnode == NULL) {
		xml_add_status(ret_data, XML_FAILURE, "Malformed XML Document:"
		    " vsw tag contains no children tags");
		return (-1);
	}

	subnode = xml_find_subnode(res_node, MAC_ADDRESS_NODE);
	if (subnode != NULL) {
		mac_addr = xmlNodeGetContent(subnode);
		if (strcmp((char *)mac_addr, XML_AUTO_ALLOCATED_MACADDR) != 0) {
			argv[argc++] = zsprintf("mac-addr=%s", mac_addr);
		}
		xmlFree(mac_addr);
	}

	subnode = xml_find_subnode(res_node, DEV_PATH_NODE);
	if (subnode != NULL) {
		net_dev = xmlNodeGetContent(subnode);
		argv[argc++] = zsprintf("net-dev=%s", net_dev);
		xmlFree(net_dev);
	}

	subnode = xml_find_subnode(res_node, VSW_MODE_NODE);
	if (subnode != NULL) {
		mode = xmlNodeGetContent(subnode);
		if (mode == NULL)
			argv[argc++] = zsprintf("mode=");
		else {
			argv[argc++] = zsprintf("mode=%s", mode);
			xmlFree(mode);
		}
	}

	subnode = xml_find_subnode(res_node, SERVICE_NAME_NODE);
	if (subnode == NULL) {
		set_missing_prop_msg(ret_data, VSW_NODE, SERVICE_NAME_NODE);
		free_argv(argv, argc);
		return (-1);
	}
	nws_name = xmlNodeGetContent(subnode);
	argv[argc++] = Xstrdup((char *)nws_name);
	xmlFree(nws_name);

	if (action == XML_ADD_RES) {
		argv[argc++] = Xstrdup(ldp->namep);
	}

	DBG_BLOCK {
		for (i = 0; i < argc; i++)
			zprintf("Argv[%d] = %s\n", i, argv[i]);
	}

	switch (action) {
	case XML_ADD_RES:
		cmd_num = LDM_ADD_VSW;
		break;
	case XML_MODIFY_RES:
		cmd_num = LDM_MODIFY_VSW;
		break;
	case XML_REMOVE_RES:
		cmd_num = LDM_REMOVE_VSW;
		break;
	}
	result = xml_call_cli_cmd(argc, argv, cmd_num, ret_data, client);
	free_argv(argv, argc);
	if (result == false)
		return (-1);
	else
		return (ret);
}

static int
fill_vsw(xmlNodePtr res_node, xmlNodePtr ret_data, int action, int *argc,
    void **argv)
{
	xmlNodePtr	subnode;
	xmlChar		*nws_name;

	subnode = xml_find_subnode(res_node, SERVICE_NAME_NODE);
	if ((action == XML_ADD_RES) && (subnode == NULL)) {
		set_missing_prop_msg(ret_data, VNET_NODE, SERVICE_NAME_NODE);
		return (-1);
	} else if (subnode == NULL) {
		return (0);
	}

	nws_name = xmlNodeGetContent(subnode);
	if (action == XML_ADD_RES) {
		argv[(*argc)++] = Xstrdup((char *)nws_name);
		xmlFree(nws_name);
	} else {
		argv[(*argc)++] = zsprintf("vswitch=%s", nws_name);
		xmlFree(nws_name);
	}
	return (0);
}

int
xml_vnet(xmlNodePtr res_node, ldom_t *ldp, int action, xmlNodePtr ret_data,
    cli_conn_t *client)
{
	int		ret = 0, cmd_num;
	const int	vnet_args = 4;
	void		*argv[vnet_args];
	xmlNodePtr	subnode = NULL;
	xmlChar		*mac_addr;
	xmlChar		*vnet_name;
	int		argc = 0, i;
	bool_t		result;

	subnode = res_node->xmlChildrenNode;
	if (subnode == NULL) {
		xml_add_status(ret_data, XML_FAILURE, "Malformed XML Document:"
		    " network tag contains no children tags");
		return (-1);
	}

	if (action != XML_REMOVE_RES) {
		subnode = xml_find_subnode(res_node, MAC_ADDRESS_NODE);
		if (subnode != NULL) {
			mac_addr = xmlNodeGetContent(subnode);
			if (strcmp((char *)mac_addr, XML_AUTO_ALLOCATED_MACADDR)
			    != 0) {
				argv[argc++] = zsprintf("mac-addr=%s",
				    mac_addr);
			}
			xmlFree(mac_addr);
		}
	}

	if (action == XML_MODIFY_RES) {
		ret = fill_vsw(res_node, ret_data, action, &argc, argv);
		if (ret == -1) {
			free_argv(argv, argc);
			return (-1);
		}
	}

	subnode = xml_find_subnode(res_node, VNET_NAME_NODE);
	if (subnode == NULL) {
		set_missing_prop_msg(ret_data, VNET_NODE, VNET_NAME_NODE);
		free_argv(argv, argc);
		return (-1);
	}
	vnet_name = xmlNodeGetContent(subnode);
	argv[argc++] = Xstrdup((char *)vnet_name);
	xmlFree(vnet_name);

	if (action == XML_ADD_RES) {
		ret = fill_vsw(res_node, ret_data, action, &argc, argv);
		if (ret == -1) {
			free_argv(argv, argc);
			return (-1);
		}
	}

	argv[argc++] = Xstrdup(ldp->namep);

	DBG_BLOCK {
		for (i = 0; i < argc; i++)
			zprintf("Argv[%d] = %s\n", i, argv[i]);
	}

	switch (action) {
	case XML_ADD_RES:
		cmd_num = LDM_ADD_VNET;
		break;
	case XML_MODIFY_RES:
		cmd_num = LDM_MODIFY_VNET;
		break;
	case XML_REMOVE_RES:
		cmd_num = LDM_REMOVE_VNET;
		break;
	}
	result = xml_call_cli_cmd(argc, argv, cmd_num, ret_data, client);
	free_argv(argv, argc);
	if (result == false)
		return (-1);
	else
		return (ret);
}


int
xml_vcc(xmlNodePtr res_node, ldom_t *ldp, int action, xmlNodePtr ret_data,
    cli_conn_t *client)
{
	int		ret = 0, cmd_num;
	const int	vcc_args = 3;
	void		*argv[vcc_args];
	xmlNodePtr	subnode = NULL;
	xmlChar		*vcc_name, *port_low, *port_high;
	int		argc = 0, i;
	bool_t		result;

	subnode = res_node->xmlChildrenNode;
	if (subnode == NULL) {
		xml_add_status(ret_data, XML_FAILURE, "Malformed XML Document:"
		    " vcc tag contains no children tags");
		return (-1);
	}

	if (action != XML_REMOVE_RES) {
		subnode = xml_find_subnode(res_node, MIN_PORT_NODE);
		if (subnode == NULL) {
			set_missing_prop_msg(ret_data, VCC_NODE,
			    MIN_PORT_NODE);
			return (-1);
		}
		port_low = xmlNodeGetContent(subnode);

		subnode = xml_find_subnode(res_node, MAX_PORT_NODE);
		if (subnode == NULL) {
			set_missing_prop_msg(ret_data, VCC_NODE,
			    MAX_PORT_NODE);
			return (-1);
		}
		port_high = xmlNodeGetContent(subnode);
		argv[argc++] = zsprintf("port-range=%s-%s", port_low,
		    port_high);
		xmlFree(port_low);
		xmlFree(port_high);
	}

	subnode = xml_find_subnode(res_node, SERVICE_NAME_NODE);
	if (subnode == NULL) {
		set_missing_prop_msg(ret_data, VCC_NODE, SERVICE_NAME_NODE);
		free_argv(argv, argc);
		return (-1);
	}
	vcc_name = xmlNodeGetContent(subnode);
	argv[argc++] = Xstrdup((char *)vcc_name);
	xmlFree(vcc_name);

	if (action == XML_ADD_RES) {
		argv[argc++] = Xstrdup(ldp->namep);
	}

	DBG_BLOCK {
		for (i = 0; i < argc; i++)
			zprintf("Argv[%d] = %s\n", i, argv[i]);
	}

	switch (action) {
	case XML_ADD_RES:
		cmd_num = LDM_ADD_VCC;
		break;
	case XML_MODIFY_RES:
		cmd_num = LDM_MODIFY_VCC;
		break;
	case XML_REMOVE_RES:
		cmd_num = LDM_REMOVE_VCC;
		break;
	}
	result = xml_call_cli_cmd(argc, argv, cmd_num, ret_data,
	    client);
	free_argv(argv, argc);
	if (result == false)
		return (-1);
	else
		return (ret);
}

int
xml_physio(xmlNodePtr res_node, ldom_t *ldp, int action, xmlNodePtr ret_data,
    cli_conn_t *client)
{
	int		ret = 0, cmd_num;
	const int	physio_args = 2;
	void		*argv[physio_args];
	xmlNodePtr	subnode = NULL;
	xmlChar		*bus;
	int		argc = 0, i;
	bool_t		result;

	subnode = res_node->xmlChildrenNode;
	if (subnode == NULL) {
		xml_add_status(ret_data, XML_FAILURE, "Malformed XML Document:"
		    " physio_device tag contains no children tags");
		return (-1);
	}

	subnode = xml_find_subnode(res_node, NAME_NODE);
	if (subnode == NULL) {
		set_missing_prop_msg(ret_data, XML_PHYSIO_DEVICE, NAME_NODE);
		return (-1);
	}
	bus = xmlNodeGetContent(subnode);
	argv[argc++] = Xstrdup((char *)bus);
	xmlFree(bus);

	argv[argc++] = Xstrdup(ldp->namep);

	DBG_BLOCK {
		for (i = 0; i < argc; i++)
			zprintf("Argv[%d] = %s\n", i, argv[i]);
	}

	switch (action) {
	case XML_ADD_RES:
		cmd_num = LDM_ADD_IO;
		break;
	case XML_REMOVE_RES:
		cmd_num = LDM_REMOVE_IO;
		break;
	}
	result = xml_call_cli_cmd(argc, argv, cmd_num, ret_data, client);
	free_argv(argv, argc);
	if (result == false)
		return (-1);
	else
		return (ret);
}

int
xml_var(xmlNodePtr res_node, ldom_t *ldp, int action, xmlNodePtr ret_data,
    cli_conn_t *client)
{
	int		ret = 0, cmd_num;
	const int	var_args = 2;
	void		*argv[var_args];
	xmlNodePtr	subnode = NULL;
	xmlChar		*name, *value;
	int		argc = 0, i;
	bool_t		result;

	subnode = res_node->xmlChildrenNode;
	if (subnode == NULL) {
		xml_add_status(ret_data, XML_FAILURE, "Malformed XML Document:"
		    " var tag contains no children tags");
		return (-1);
	}

	subnode = xml_find_subnode(res_node, NAME_NODE);
	if (subnode == NULL) {
		set_missing_prop_msg(ret_data, VAR_NODE, NAME_NODE);
		return (-1);
	}
	name = xmlNodeGetContent(subnode);

	if (action == XML_ADD_RES) {
		subnode = xml_find_subnode(res_node, VALUE_NODE);
		if (subnode == NULL) {
			set_missing_prop_msg(ret_data, VAR_NODE, VALUE_NODE);
			return (-1);
		}
		value = xmlNodeGetContent(subnode);
		argv[argc++] = zsprintf("%s=%s", name, value);
		xmlFree(name);
		xmlFree(value);
		cmd_num = LDM_SET_VAR;
	} else {
		argv[argc++] = Xstrdup((char *)name);
		xmlFree(name);
		cmd_num = LDM_UNSET_VAR;
	}

	argv[argc++] = Xstrdup(ldp->namep);

	DBG_BLOCK {
		for (i = 0; i < argc; i++)
			zprintf("Argv[%d] = %s\n", i, argv[i]);
	}

	result = xml_call_cli_cmd(argc, argv, cmd_num, ret_data, client);
	free_argv(argv, argc);
	if (result == false)
		return (-1);
	else
		return (ret);
}

int
xml_spconfig(xmlNodePtr res_node, ldom_t *ldp, int action, xmlNodePtr ret_data,
    cli_conn_t *client)
{
	int		ret = 0, cmd_num;
	const int	var_args = 1;
	void		*argv[var_args];
	xmlNodePtr	subnode = NULL;
	xmlChar		*name;
	int		argc = 0, i;
	bool_t		result;

	subnode = res_node->xmlChildrenNode;
	if (subnode == NULL) {
		xml_add_status(ret_data, XML_FAILURE, "Malformed XML Document:"
		    " spconfig tag contains no children tags");
		return (-1);
	}

	subnode = xml_find_subnode(res_node, XML_SPCONFIG_NAME);
	if (subnode == NULL) {
		set_missing_prop_msg(ret_data, XML_SPCONFIG,
		    XML_SPCONFIG_NAME);
		return (-1);
	}
	name = xmlNodeGetContent(subnode);
	argv[argc++] = Xstrdup((char *)name);
	xmlFree(name);

	DBG_BLOCK {
		for (i = 0; i < argc; i++)
			zprintf("Argv[%d] = %s\n", i, argv[i]);
	}

	switch (action) {
	case XML_ADD_RES:
		cmd_num = LDM_ADD_SET;
		break;
	case XML_MODIFY_RES:
		cmd_num = LDM_USE_SET;
		break;
	case XML_REMOVE_RES:
		cmd_num = LDM_REMOVE_SET;
		break;
	}
	result = xml_call_cli_cmd(argc, argv, cmd_num, ret_data, client);
	free_argv(argv, argc);
	if (result == false)
		return (-1);
	else
		return (ret);
}

int
xml_console(xmlNodePtr res_node, ldom_t *ldp, int action, xmlNodePtr ret_data,
    cli_conn_t *client)
{
	int		ret = 0;
	const int	var_args = 4;
	void		*argv[var_args];
	xmlNodePtr	subnode = NULL;
	xmlChar		*group = NULL, *vcc_name, *port;
	int		argc = 0, i;
	bool_t		result;

	subnode = res_node->xmlChildrenNode;
	if (subnode == NULL) {
		xml_add_status(ret_data, XML_FAILURE, "Malformed XML Document:"
		    " console tag contains no children tags");
		return (-1);
	}

	subnode = xml_find_subnode(res_node, VCONS_PORT_NODE);
	if (subnode != NULL) {
		port = xmlNodeGetContent(subnode);
		argv[argc++] = zsprintf("port=%s", port);
		xmlFree(port);
	}

	subnode = xml_find_subnode(res_node, GROUP_NODE);
	if (subnode != NULL) {
		group = xmlNodeGetContent(subnode);
		argv[argc++] = zsprintf("group=%s", group);
		xmlFree(group);
	}

	subnode = xml_find_subnode(res_node, SERVICE_NAME_NODE);
	if (subnode != NULL) {
		vcc_name = xmlNodeGetContent(subnode);
		argv[argc++] = zsprintf("service=%s", vcc_name);
		xmlFree(vcc_name);
	}

	if (argc == 0) {
		xml_add_status(ret_data, XML_FAILURE, "Malformed XML Document:"
		    " console Section contains no children tags");
		return (-1);
	}

	argv[argc++] = Xstrdup(ldp->namep);

	DBG_BLOCK {
		for (i = 0; i < argc; i++)
			zprintf("Argv[%d] = %s\n", i, argv[i]);
	}

	result = xml_call_cli_cmd(argc, argv, LDM_MODIFY_VCONS, ret_data,
	    client);
	free_argv(argv, argc);
	if (result == false)
		return (-1);
	else
		return (ret);
}

int
xml_vdpcs(xmlNodePtr res_node, ldom_t *ldp, int action, xmlNodePtr ret_data,
    cli_conn_t *client)
{
	int		ret = 0, cmd_num;
	const int	vdpcs_args = 2;
	void		*argv[vdpcs_args];
	xmlNodePtr	subnode = NULL;
	xmlChar		*vdpcs_name;
	int		argc = 0, i;
	bool_t		result;

	subnode = res_node->xmlChildrenNode;
	if (subnode == NULL) {
		xml_add_status(ret_data, XML_FAILURE, "Malformed XML Document:"
		    " vdpcs tag contains no children tags");
		return (-1);
	}

	subnode = xml_find_subnode(res_node, SERVICE_NAME_NODE);
	if (subnode == NULL) {
		set_missing_prop_msg(ret_data, VDPCS_NODE, SERVICE_NAME_NODE);
		return (-1);
	}

	vdpcs_name = xmlNodeGetContent(subnode);
	argv[argc++] = Xstrdup((char *)vdpcs_name);
	xmlFree(vdpcs_name);

	if (action == XML_ADD_RES) {
		argv[argc++] = Xstrdup(ldp->namep);
		cmd_num = LDM_ADD_VDPCS;
	} else {
		cmd_num = LDM_REMOVE_VDPCS;
	}

	DBG_BLOCK {
		for (i = 0; i < argc; i++)
			zprintf("Argv[%d] = %s\n", i, argv[i]);
	}

	result = xml_call_cli_cmd(argc, argv, cmd_num, ret_data, client);
	free_argv(argv, argc);
	if (result == false)
		return (-1);
	else
		return (ret);
}

int
xml_vdpcc(xmlNodePtr res_node, ldom_t *ldp, int action, xmlNodePtr ret_data,
    cli_conn_t *client)
{
	int		ret = 0, cmd_num;
	const int	vdpcc_args = 3;
	void		*argv[vdpcc_args];
	xmlNodePtr	subnode = NULL;
	xmlChar		*vdpcs_name, *vdpcc_name;
	int		argc = 0, i;
	bool_t		result;

	subnode = res_node->xmlChildrenNode;
	if (subnode == NULL) {
		xml_add_status(ret_data, XML_FAILURE, "Malformed XML Document:"
		    " vdpcc tag contains no children tags");
		return (-1);
	}

	subnode = xml_find_subnode(res_node, VDPCC_NAME_NODE);
	if (subnode == NULL) {
		set_missing_prop_msg(ret_data, VDPCC_NODE, VDPCC_NAME_NODE);
		return (-1);
	}
	vdpcc_name = xmlNodeGetContent(subnode);
	argv[argc++] = Xstrdup((char *)vdpcc_name);
	xmlFree(vdpcc_name);

	if (action == XML_ADD_RES) {
		subnode = xml_find_subnode(res_node, SERVICE_NAME_NODE);
		if (subnode == NULL) {
			set_missing_prop_msg(ret_data, VDPCC_NODE,
			    SERVICE_NAME_NODE);
			free_argv(argv, argc);
			return (-1);
		}
		vdpcs_name = xmlNodeGetContent(subnode);
		argv[argc++] = Xstrdup((char *)vdpcs_name);
		xmlFree(vdpcs_name);
		cmd_num = LDM_ADD_VDPCC;
	} else {
		cmd_num = LDM_REMOVE_VDPCC;
	}

	argv[argc++] = Xstrdup(ldp->namep);

	DBG_BLOCK {
		for (i = 0; i < argc; i++)
			zprintf("Argv[%d] = %s\n", i, argv[i]);
	}

	result = xml_call_cli_cmd(argc, argv, cmd_num, ret_data, client);
	free_argv(argv, argc);
	if (result == false)
		return (-1);
	else
		return (ret);
}

int
xml_empty_res(xmlNodePtr res_node, ldom_t *ldp, int action,
    xmlNodePtr ret_data, cli_conn_t *client)
{
	return (0);
}
